@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2015-2022  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2015-2024  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -29,14 +29,15 @@
 
 #pragma once
 
-#include <QtGlobal>
 #include <QtContainerFwd>
+#include <QtTypes>
 #include <QMetaType>
 #include <QString>
 
 #include "base/3rdparty/expected.hpp"
 #include "base/pathfwd.h"
 #include "base/tagset.h"
+#include "sharelimitaction.h"
 #include "torrentcontenthandler.h"
 
 class QBitArray;
@@ -47,12 +48,17 @@ class QUrl;
 namespace BitTorrent
 {
     enum class DownloadPriority;
+
     class InfoHash;
     class PeerInfo;
+    class Session;
     class TorrentID;
     class TorrentInfo;
+
     struct PeerAddress;
+    struct SSLParameters;
     struct TrackerEntry;
+    struct TrackerEntryStatus;
 
     // Using `Q_ENUM_NS()` without a wrapper namespace in our case is not advised
     // since `Q_NAMESPACE` cannot be used when the same namespace resides at different files.
@@ -91,8 +97,8 @@ namespace BitTorrent
         CheckingUploading,
         CheckingDownloading,
 
-        PausedDownloading,
-        PausedUploading,
+        StoppedDownloading,
+        StoppedUploading,
 
         Moving,
 
@@ -100,11 +106,7 @@ namespace BitTorrent
         Error
     };
 
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
     std::size_t qHash(TorrentState key, std::size_t seed = 0);
-#else
-    uint qHash(TorrentState key, uint seed = 0);
-#endif
 
     class Torrent : public TorrentContentHandler
     {
@@ -135,6 +137,8 @@ namespace BitTorrent
 
         using TorrentContentHandler::TorrentContentHandler;
 
+        virtual Session *session() const = 0;
+
         virtual InfoHash infoHash() const = 0;
         virtual QString name() const = 0;
         virtual QDateTime creationDate() const = 0;
@@ -203,24 +207,40 @@ namespace BitTorrent
         virtual bool setCategory(const QString &category) = 0;
 
         virtual TagSet tags() const = 0;
-        virtual bool hasTag(const QString &tag) const = 0;
-        virtual bool addTag(const QString &tag) = 0;
-        virtual bool removeTag(const QString &tag) = 0;
+        virtual bool hasTag(const Tag &tag) const = 0;
+        virtual bool addTag(const Tag &tag) = 0;
+        virtual bool removeTag(const Tag &tag) = 0;
         virtual void removeAllTags() = 0;
 
         virtual int piecesCount() const = 0;
         virtual int piecesHave() const = 0;
         virtual qreal progress() const = 0;
+
         virtual QDateTime addedTime() const = 0;
+        virtual QDateTime completedTime() const = 0;
+        virtual QDateTime lastSeenComplete() const = 0;
+        virtual qlonglong activeTime() const = 0;
+        virtual qlonglong finishedTime() const = 0;
+        virtual qlonglong timeSinceUpload() const = 0;
+        virtual qlonglong timeSinceDownload() const = 0;
+        virtual qlonglong timeSinceActivity() const = 0;
+
+        // Share limits
         virtual qreal ratioLimit() const = 0;
+        virtual void setRatioLimit(qreal limit) = 0;
         virtual int seedingTimeLimit() const = 0;
+        virtual void setSeedingTimeLimit(int limit) = 0;
         virtual int inactiveSeedingTimeLimit() const = 0;
+        virtual void setInactiveSeedingTimeLimit(int limit) = 0;
+        virtual ShareLimitAction shareLimitAction() const = 0;
+        virtual void setShareLimitAction(ShareLimitAction action) = 0;
 
         virtual PathList filePaths() const = 0;
+        virtual PathList actualFilePaths() const = 0;
 
         virtual TorrentInfo info() const = 0;
         virtual bool isFinished() const = 0;
-        virtual bool isPaused() const = 0;
+        virtual bool isStopped() const = 0;
         virtual bool isQueued() const = 0;
         virtual bool isForced() const = 0;
         virtual bool isChecking() const = 0;
@@ -237,13 +257,11 @@ namespace BitTorrent
         virtual bool hasMissingFiles() const = 0;
         virtual bool hasError() const = 0;
         virtual int queuePosition() const = 0;
-        virtual QVector<TrackerEntry> trackers() const = 0;
+        virtual QVector<TrackerEntryStatus> trackers() const = 0;
         virtual QVector<QUrl> urlSeeds() const = 0;
         virtual QString error() const = 0;
         virtual qlonglong totalDownload() const = 0;
         virtual qlonglong totalUpload() const = 0;
-        virtual qlonglong activeTime() const = 0;
-        virtual qlonglong finishedTime() const = 0;
         virtual qlonglong eta() const = 0;
         virtual int seedsCount() const = 0;
         virtual int peersCount() const = 0;
@@ -251,11 +269,6 @@ namespace BitTorrent
         virtual int totalSeedsCount() const = 0;
         virtual int totalPeersCount() const = 0;
         virtual int totalLeechersCount() const = 0;
-        virtual QDateTime lastSeenComplete() const = 0;
-        virtual QDateTime completedTime() const = 0;
-        virtual qlonglong timeSinceUpload() const = 0;
-        virtual qlonglong timeSinceDownload() const = 0;
-        virtual qlonglong timeSinceActivity() const = 0;
         virtual int downloadLimit() const = 0;
         virtual int uploadLimit() const = 0;
         virtual bool superSeeding() const = 0;
@@ -271,6 +284,7 @@ namespace BitTorrent
         virtual int maxSeedingTime() const = 0;
         virtual int maxInactiveSeedingTime() const = 0;
         virtual qreal realRatio() const = 0;
+        virtual qreal popularity() const = 0;
         virtual int uploadPayloadRate() const = 0;
         virtual int downloadPayloadRate() const = 0;
         virtual qlonglong totalPayloadUpload() const = 0;
@@ -282,14 +296,11 @@ namespace BitTorrent
         virtual void setName(const QString &name) = 0;
         virtual void setSequentialDownload(bool enable) = 0;
         virtual void setFirstLastPiecePriority(bool enabled) = 0;
-        virtual void pause() = 0;
-        virtual void resume(TorrentOperatingMode mode = TorrentOperatingMode::AutoManaged) = 0;
+        virtual void stop() = 0;
+        virtual void start(TorrentOperatingMode mode = TorrentOperatingMode::AutoManaged) = 0;
         virtual void forceReannounce(int index = -1) = 0;
         virtual void forceDHTAnnounce() = 0;
         virtual void forceRecheck() = 0;
-        virtual void setRatioLimit(qreal limit) = 0;
-        virtual void setSeedingTimeLimit(int limit) = 0;
-        virtual void setInactiveSeedingTimeLimit(int limit) = 0;
         virtual void setUploadLimit(int limit) = 0;
         virtual void setDownloadLimit(int limit) = 0;
         virtual void setSuperSeeding(bool enable) = 0;
@@ -307,6 +318,8 @@ namespace BitTorrent
 
         virtual StopCondition stopCondition() const = 0;
         virtual void setStopCondition(StopCondition stopCondition) = 0;
+        virtual SSLParameters getSSLParameters() const = 0;
+        virtual void setSSLParameters(const SSLParameters &sslParams) = 0;
 
         virtual QString createMagnetURI() const = 0;
         virtual nonstd::expected<QByteArray, QString> exportToBuffer() const = 0;
@@ -318,7 +331,7 @@ namespace BitTorrent
         virtual void fetchDownloadingPieces(std::function<void (QBitArray)> resultHandler) const = 0;
 
         TorrentID id() const;
-        bool isResumed() const;
+        bool isRunning() const;
         qlonglong remainingSize() const;
 
         void toggleSequentialDownload();
