@@ -51,7 +51,7 @@
 
 using namespace BitTorrent;
 
-const int torrentInfoId = qRegisterMetaType<TorrentInfo>();
+const int TORRENTINFO_TYPEID = qRegisterMetaType<TorrentInfo>();
 
 TorrentInfo::TorrentInfo(const lt::torrent_info &nativeInfo)
     : m_nativeInfo {std::make_shared<const lt::torrent_info>(nativeInfo)}
@@ -279,7 +279,7 @@ QVector<TrackerEntry> TorrentInfo::trackers() const
     QVector<TrackerEntry> ret;
     ret.reserve(static_cast<decltype(ret)::size_type>(trackers.size()));
     for (const lt::announce_entry &tracker : trackers)
-        ret.append({QString::fromStdString(tracker.url), tracker.tier});
+        ret.append({.url = QString::fromStdString(tracker.url), .tier = tracker.tier});
 
     return ret;
 }
@@ -401,6 +401,34 @@ TorrentInfo::PieceRange TorrentInfo::filePieces(const int fileIndex) const
     return makeInterval(beginIdx, endIdx);
 }
 
+bool TorrentInfo::matchesInfoHash(const InfoHash &otherInfoHash) const
+{
+    if (!isValid())
+        return false;
+
+    const InfoHash thisInfoHash = infoHash();
+
+    if (thisInfoHash.v1().isValid() && otherInfoHash.v1().isValid()
+            && (thisInfoHash.v1() != otherInfoHash.v1()))
+    {
+        return false;
+    }
+
+    if (thisInfoHash.v2().isValid() && otherInfoHash.v2().isValid()
+            && (thisInfoHash.v2() != otherInfoHash.v2()))
+    {
+        return false;
+    }
+
+    if (!thisInfoHash.v1().isValid() && otherInfoHash.v1().isValid())
+        return false;
+
+    if (!thisInfoHash.v2().isValid() && otherInfoHash.v2().isValid())
+        return false;
+
+    return true;
+}
+
 int TorrentInfo::fileIndex(const Path &filePath) const
 {
     // the check whether the object is valid is not needed here
