@@ -32,7 +32,6 @@
 
 #include <QTcpSocket>
 
-#include "base/logger.h"
 #include "irequesthandler.h"
 #include "requestparser.h"
 #include "responsegenerator.h"
@@ -45,6 +44,11 @@ Connection::Connection(QTcpSocket *socket, IRequestHandler *requestHandler, QObj
     , m_requestHandler(requestHandler)
 {
     m_socket->setParent(this);
+    connect(m_socket, &QAbstractSocket::disconnected, this, &Connection::closed);
+
+    // reserve common size for requests, don't use the max allowed size which is too big for
+    // memory constrained platforms
+    m_receivedData.reserve(1024 * 1024);
 
     // reset timer when there are activity
     m_idleTimer.start();
@@ -59,14 +63,20 @@ Connection::Connection(QTcpSocket *socket, IRequestHandler *requestHandler, QObj
     });
 }
 
-Connection::~Connection()
-{
-    m_socket->close();
-}
-
 void Connection::read()
 {
-    m_receivedData.append(m_socket->readAll());
+    // reuse existing buffer and avoid unnecessary memory allocation/relocation
+    const qsizetype previousSize = m_receivedData.size();
+    const qint64 bytesAvailable = m_socket->bytesAvailable();
+    m_receivedData.resize(previousSize + bytesAvailable);
+    const qint64 bytesRead = m_socket->read((m_receivedData.data() + previousSize), bytesAvailable);
+    if (bytesRead < 0) [[unlikely]]
+    {
+        m_socket->close();
+        return;
+    }
+    if (bytesRead < bytesAvailable) [[unlikely]]
+        m_receivedData.chop(bytesAvailable - bytesRead);
 
     while (!m_receivedData.isEmpty())
     {
@@ -79,11 +89,11 @@ void Connection::read()
                 const long bufferLimit = RequestParser::MAX_CONTENT_SIZE * 1.1;  // some margin for headers
                 if (m_receivedData.size() > bufferLimit)
                 {
-                    LogMsg(tr("Http request size exceeds limitation, closing socket. Limit: %1, IP: %2")
-                        .arg(bufferLimit).arg(m_socket->peerAddress().toString()), Log::WARNING);
+                    qWarning("%s", qUtf8Printable(tr("Http request size exceeds limitation, closing socket. Limit: %1, IP: %2")
+                        .arg(QString::number(bufferLimit), m_socket->peerAddress().toString())));
 
-                    Response resp(413, u"Payload Too Large"_qs);
-                    resp.headers[HEADER_CONNECTION] = u"close"_qs;
+                    Response resp(413, u"Payload Too Large"_s);
+                    resp.headers[HEADER_CONNECTION] = u"close"_s;
 
                     sendResponse(resp);
                     m_socket->close();
@@ -91,13 +101,26 @@ void Connection::read()
             }
             return;
 
+        case RequestParser::ParseStatus::BadMethod:
+            {
+                qWarning("%s", qUtf8Printable(tr("Bad Http request method, closing socket. IP: %1. Method: \"%2\"")
+                    .arg(m_socket->peerAddress().toString(), result.request.method)));
+
+                Response resp(501, u"Not Implemented"_s);
+                resp.headers[HEADER_CONNECTION] = u"close"_s;
+
+                sendResponse(resp);
+                m_socket->close();
+            }
+            return;
+
         case RequestParser::ParseStatus::BadRequest:
             {
-                LogMsg(tr("Bad Http request, closing socket. IP: %1")
-                    .arg(m_socket->peerAddress().toString()), Log::WARNING);
+                qWarning("%s", qUtf8Printable(tr("Bad Http request, closing socket. IP: %1")
+                    .arg(m_socket->peerAddress().toString())));
 
-                Response resp(400, u"Bad Request"_qs);
-                resp.headers[HEADER_CONNECTION] = u"close"_qs;
+                Response resp(400, u"Bad Request"_s);
+                resp.headers[HEADER_CONNECTION] = u"close"_s;
 
                 sendResponse(resp);
                 m_socket->close();
@@ -108,15 +131,31 @@ void Connection::read()
             {
                 const Environment env {m_socket->localAddress(), m_socket->localPort(), m_socket->peerAddress(), m_socket->peerPort()};
 
-                Response resp = m_requestHandler->processRequest(result.request, env);
+                if (result.request.method == HEADER_REQUEST_METHOD_HEAD)
+                {
+                    Request getRequest = result.request;
+                    getRequest.method = HEADER_REQUEST_METHOD_GET;
 
-                if (acceptsGzipEncoding(result.request.headers[u"accept-encoding"_qs]))
-                    resp.headers[HEADER_CONTENT_ENCODING] = u"gzip"_qs;
+                    Response resp = m_requestHandler->processRequest(getRequest, env);
 
-                resp.headers[HEADER_CONNECTION] = u"keep-alive"_qs;
+                    resp.headers[HEADER_CONNECTION] = u"keep-alive"_s;
+                    resp.headers[HEADER_CONTENT_LENGTH] = QString::number(resp.content.length());
+                    resp.content.clear();
 
-                sendResponse(resp);
-                m_receivedData = m_receivedData.mid(result.frameSize);
+                    sendResponse(resp);
+                }
+                else
+                {
+                    Response resp = m_requestHandler->processRequest(result.request, env);
+
+                    if (acceptsGzipEncoding(result.request.headers.value(u"accept-encoding"_s)))
+                        resp.headers[HEADER_CONTENT_ENCODING] = u"gzip"_s;
+                    resp.headers[HEADER_CONNECTION] = u"keep-alive"_s;
+
+                    sendResponse(resp);
+                }
+
+                m_receivedData.remove(0, result.frameSize);
             }
             break;
 
@@ -139,11 +178,6 @@ bool Connection::hasExpired(const qint64 timeout) const
         && m_idleTimer.hasExpired(timeout);
 }
 
-bool Connection::isClosed() const
-{
-    return (m_socket->state() == QAbstractSocket::UnconnectedState);
-}
-
 bool Connection::acceptsGzipEncoding(QString codings)
 {
     // [rfc7231] 5.3.4. Accept-Encoding
@@ -176,11 +210,11 @@ bool Connection::acceptsGzipEncoding(QString codings)
     if (list.isEmpty())
         return false;
 
-    const bool canGzip = isCodingAvailable(list, u"gzip"_qs);
+    const bool canGzip = isCodingAvailable(list, u"gzip"_s);
     if (canGzip)
         return true;
 
-    const bool canAny = isCodingAvailable(list, u"*"_qs);
+    const bool canAny = isCodingAvailable(list, u"*"_s);
     if (canAny)
         return true;
 
