@@ -30,7 +30,7 @@
 #pragma once
 
 #include <QtContainerFwd>
-#include <QtGlobal>
+#include <QtSystemDetection>
 #include <QObject>
 
 #include "base/pathfwd.h"
@@ -105,8 +105,8 @@ public:
     void setUseCustomUITheme(bool use);
     Path customUIThemePath() const;
     void setCustomUIThemePath(const Path &path);
-    bool deleteTorrentFilesAsDefault() const;
-    void setDeleteTorrentFilesAsDefault(bool del);
+    bool removeTorrentContent() const;
+    void setRemoveTorrentContent(bool remove);
     bool confirmOnExit() const;
     void setConfirmOnExit(bool confirm);
     bool speedInTitleBar() const;
@@ -130,6 +130,8 @@ public:
 #ifdef Q_OS_WIN
     bool WinStartup() const;
     void setWinStartup(bool b);
+    QString getStyle() const;
+    void setStyle(const QString &styleName);
 #endif
 
     // Downloads
@@ -287,15 +289,16 @@ public:
 #endif
     bool isRecursiveDownloadEnabled() const;
     void setRecursiveDownloadEnabled(bool enable);
-#ifdef Q_OS_WIN
-    bool neverCheckFileAssoc() const;
-    void setNeverCheckFileAssoc(bool check = true);
-#endif
-
     int getTrackerPort() const;
     void setTrackerPort(int port);
     bool isTrackerPortForwardingEnabled() const;
     void setTrackerPortForwardingEnabled(bool enabled);
+    bool isMarkOfTheWebEnabled() const;
+    void setMarkOfTheWebEnabled(bool enabled);
+    bool isIgnoreSSLErrors() const;
+    void setIgnoreSSLErrors(bool enabled);
+    Path getPythonExecutablePath() const;
+    void setPythonExecutablePath(const Path &path);
 #if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
     bool isUpdateCheckEnabled() const;
     void setUpdateCheckEnabled(bool enabled);
@@ -306,10 +309,10 @@ public:
     void setConfirmTorrentRecheck(bool enabled);
     bool confirmRemoveAllTags() const;
     void setConfirmRemoveAllTags(bool enabled);
-    bool confirmPauseAndResumeAll() const;
-    void setConfirmPauseAndResumeAll(bool enabled);
     bool confirmMergeTrackers() const;
     void setConfirmMergeTrackers(bool enabled);
+    bool confirmRemoveTrackerFromAllTorrents() const;
+    void setConfirmRemoveTrackerFromAllTorrents(bool enabled);
 #ifndef Q_OS_MACOS
     bool systemTrayEnabled() const;
     void setSystemTrayEnabled(bool enabled);
@@ -339,8 +342,6 @@ public:
     void setDNSLastUpd(const QDateTime &date);
     QString getDNSLastIP() const;
     void setDNSLastIP(const QString &ip);
-    bool getAcceptedLegal() const;
-    void setAcceptedLegal(bool accepted);
     QByteArray getMainGeometry() const;
     void setMainGeometry(const QByteArray &geometry);
     bool isFiltersSidebarVisible() const;
@@ -359,8 +360,8 @@ public:
     void setPropCurTab(int tab);
     bool getPropVisible() const;
     void setPropVisible(bool visible);
-    QByteArray getPropTrackerListState() const;
-    void setPropTrackerListState(const QByteArray &state);
+    QByteArray getTrackerListState() const;
+    void setTrackerListState(const QByteArray &state);
     QStringList getRssOpenFolders() const;
     void setRssOpenFolders(const QStringList &folders);
     QByteArray getRssSideSplitterState() const;
@@ -415,6 +416,14 @@ public:
     bool getSpeedWidgetGraphEnable(int id) const;
     void setSpeedWidgetGraphEnable(int id, bool enable);
 
+    // AddNewTorrentDialog
+    bool isAddNewTorrentDialogEnabled() const;
+    void setAddNewTorrentDialogEnabled(bool value);
+    bool isAddNewTorrentDialogTopLevel() const;
+    void setAddNewTorrentDialogTopLevel(bool value);
+    int addNewTorrentDialogSavePathHistoryLength() const;
+    void setAddNewTorrentDialogSavePathHistoryLength(int value);
+
 public slots:
     void setStatusFilterState(bool checked);
     void setCategoryFilterState(bool checked);
