@@ -37,14 +37,14 @@
 
 using namespace RSS;
 
-const QString Article::KeyId = u"id"_qs;
-const QString Article::KeyDate = u"date"_qs;
-const QString Article::KeyTitle = u"title"_qs;
-const QString Article::KeyAuthor = u"author"_qs;
-const QString Article::KeyDescription = u"description"_qs;
-const QString Article::KeyTorrentURL = u"torrentURL"_qs;
-const QString Article::KeyLink = u"link"_qs;
-const QString Article::KeyIsRead = u"isRead"_qs;
+const QString Article::KeyId = u"id"_s;
+const QString Article::KeyDate = u"date"_s;
+const QString Article::KeyTitle = u"title"_s;
+const QString Article::KeyAuthor = u"author"_s;
+const QString Article::KeyDescription = u"description"_s;
+const QString Article::KeyTorrentURL = u"torrentURL"_s;
+const QString Article::KeyLink = u"link"_s;
+const QString Article::KeyIsRead = u"isRead"_s;
 
 Article::Article(Feed *feed, const QVariantHash &varHash)
     : QObject(feed)
