@@ -33,13 +33,13 @@
 
 const std::optional<QString> TorrentFilter::AnyCategory;
 const std::optional<TorrentIDSet> TorrentFilter::AnyID;
-const std::optional<QString> TorrentFilter::AnyTag;
+const std::optional<Tag> TorrentFilter::AnyTag;
 
 const TorrentFilter TorrentFilter::DownloadingTorrent(TorrentFilter::Downloading);
 const TorrentFilter TorrentFilter::SeedingTorrent(TorrentFilter::Seeding);
 const TorrentFilter TorrentFilter::CompletedTorrent(TorrentFilter::Completed);
-const TorrentFilter TorrentFilter::PausedTorrent(TorrentFilter::Paused);
-const TorrentFilter TorrentFilter::ResumedTorrent(TorrentFilter::Resumed);
+const TorrentFilter TorrentFilter::StoppedTorrent(TorrentFilter::Stopped);
+const TorrentFilter TorrentFilter::RunningTorrent(TorrentFilter::Running);
 const TorrentFilter TorrentFilter::ActiveTorrent(TorrentFilter::Active);
 const TorrentFilter TorrentFilter::InactiveTorrent(TorrentFilter::Inactive);
 const TorrentFilter TorrentFilter::StalledTorrent(TorrentFilter::Stalled);
@@ -52,19 +52,21 @@ const TorrentFilter TorrentFilter::ErroredTorrent(TorrentFilter::Errored);
 using BitTorrent::Torrent;
 
 TorrentFilter::TorrentFilter(const Type type, const std::optional<TorrentIDSet> &idSet
-                             , const std::optional<QString> &category, const std::optional<QString> &tag)
-    : m_type(type)
-    , m_category(category)
-    , m_tag(tag)
-    , m_idSet(idSet)
+        , const std::optional<QString> &category, const std::optional<Tag> &tag, const std::optional<bool> isPrivate)
+    : m_type {type}
+    , m_category {category}
+    , m_tag {tag}
+    , m_idSet {idSet}
+    , m_private {isPrivate}
 {
 }
 
 TorrentFilter::TorrentFilter(const QString &filter, const std::optional<TorrentIDSet> &idSet
-                             , const std::optional<QString> &category, const std::optional<QString> &tag)
-    : m_category(category)
-    , m_tag(tag)
-    , m_idSet(idSet)
+        , const std::optional<QString> &category, const std::optional<Tag> &tag, const std::optional<bool> isPrivate)
+    : m_category {category}
+    , m_tag {tag}
+    , m_idSet {idSet}
+    , m_private {isPrivate}
 {
     setTypeByName(filter);
 }
@@ -90,10 +92,10 @@ bool TorrentFilter::setTypeByName(const QString &filter)
         type = Seeding;
     else if (filter == u"completed")
         type = Completed;
-    else if (filter == u"paused")
-        type = Paused;
-    else if (filter == u"resumed")
-        type = Resumed;
+    else if (filter == u"stopped")
+        type = Stopped;
+    else if (filter == u"running")
+        type = Running;
     else if (filter == u"active")
         type = Active;
     else if (filter == u"inactive")
@@ -136,7 +138,7 @@ bool TorrentFilter::setCategory(const std::optional<QString> &category)
     return false;
 }
 
-bool TorrentFilter::setTag(const std::optional<QString> &tag)
+bool TorrentFilter::setTag(const std::optional<Tag> &tag)
 {
     if (m_tag != tag)
     {
@@ -147,19 +149,31 @@ bool TorrentFilter::setTag(const std::optional<QString> &tag)
     return false;
 }
 
+bool TorrentFilter::setPrivate(const std::optional<bool> isPrivate)
+{
+    if (m_private != isPrivate)
+    {
+        m_private = isPrivate;
+        return true;
+    }
+
+    return false;
+}
+
 bool TorrentFilter::match(const Torrent *const torrent) const
 {
     if (!torrent) return false;
 
-    return (matchState(torrent) && matchHash(torrent) && matchCategory(torrent) && matchTag(torrent));
+    return (matchState(torrent) && matchHash(torrent) && matchCategory(torrent) && matchTag(torrent) && matchPrivate(torrent));
 }
 
 bool TorrentFilter::matchState(const BitTorrent::Torrent *const torrent) const
 {
+    const BitTorrent::TorrentState state = torrent->state();
+
     switch (m_type)
     {
     case All:
-    default:
         return true;
     case Downloading:
         return torrent->isDownloading();
@@ -167,29 +181,32 @@ bool TorrentFilter::matchState(const BitTorrent::Torrent *const torrent) const
         return torrent->isUploading();
     case Completed:
         return torrent->isCompleted();
-    case Paused:
-        return torrent->isPaused();
-    case Resumed:
-        return torrent->isResumed();
+    case Stopped:
+        return torrent->isStopped();
+    case Running:
+        return torrent->isRunning();
     case Active:
         return torrent->isActive();
     case Inactive:
         return torrent->isInactive();
     case Stalled:
-        return (torrent->state() ==  BitTorrent::TorrentState::StalledUploading)
-                || (torrent->state() ==  BitTorrent::TorrentState::StalledDownloading);
+        return (state == BitTorrent::TorrentState::StalledUploading)
+                || (state == BitTorrent::TorrentState::StalledDownloading);
     case StalledUploading:
-        return torrent->state() ==  BitTorrent::TorrentState::StalledUploading;
+        return state == BitTorrent::TorrentState::StalledUploading;
     case StalledDownloading:
-        return torrent->state() ==  BitTorrent::TorrentState::StalledDownloading;
+        return state == BitTorrent::TorrentState::StalledDownloading;
     case Checking:
-        return (torrent->state() == BitTorrent::TorrentState::CheckingUploading)
-                || (torrent->state() == BitTorrent::TorrentState::CheckingDownloading)
-                || (torrent->state() == BitTorrent::TorrentState::CheckingResumeData);
+        return (state == BitTorrent::TorrentState::CheckingUploading)
+                || (state == BitTorrent::TorrentState::CheckingDownloading)
+                || (state == BitTorrent::TorrentState::CheckingResumeData);
     case Moving:
         return torrent->isMoving();
     case Errored:
         return torrent->isErrored();
+    default:
+        Q_ASSERT(false);
+        return false;
     }
 }
 
@@ -220,3 +237,11 @@ bool TorrentFilter::matchTag(const BitTorrent::Torrent *const torrent) const
 
     return torrent->hasTag(*m_tag);
 }
+
+bool TorrentFilter::matchPrivate(const BitTorrent::Torrent *const torrent) const
+{
+    if (!m_private)
+        return true;
+
+    return m_private == torrent->isPrivate();
+}
