@@ -30,44 +30,25 @@
 
 #include <optional>
 
-#ifdef Q_OS_WIN
-#include <memory>
-
-#include <windows.h>
-#include <powrprof.h>
-#include <Shlobj.h>
-#else
-#include <sys/types.h>
-#include <unistd.h>
-#endif
-
-#ifdef Q_OS_MACOS
-#include <Carbon/Carbon.h>
-#include <CoreServices/CoreServices.h>
-#endif
-
 #include <boost/version.hpp>
 #include <libtorrent/version.hpp>
 #include <openssl/crypto.h>
 #include <openssl/opensslv.h>
 #include <zlib.h>
 
+#include <QtAssert>
 #include <QCoreApplication>
 #include <QDebug>
 #include <QLocale>
 #include <QMimeDatabase>
 #include <QRegularExpression>
 #include <QSet>
+#include <QString>
+#include <QStringView>
 #include <QSysInfo>
-#include <QVector>
 
-#if (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)) && defined(QT_DBUS_LIB)
-#include <QDBusInterface>
-#endif
-
-#include "base/types.h"
+#include "base/path.h"
 #include "base/unicodestrings.h"
-#include "base/utils/fs.h"
 #include "base/utils/string.h"
 
 namespace
@@ -94,7 +75,7 @@ namespace
         Utils::Misc::SizeUnit unit;
     };
 
-    std::optional<SplitToFriendlyUnitResult> splitToFriendlyUnit(const qint64 bytes)
+    std::optional<SplitToFriendlyUnitResult> splitToFriendlyUnit(const qint64 bytes, const int unitThreshold = 1024)
     {
         if (bytes < 0)
             return std::nullopt;
@@ -102,7 +83,7 @@ namespace
         int i = 0;
         auto value = static_cast<qreal>(bytes);
 
-        while ((value >= 1024) && (i < static_cast<int>(Utils::Misc::SizeUnit::ExbiByte)))
+        while ((value >= unitThreshold) && (i < static_cast<int>(Utils::Misc::SizeUnit::ExbiByte)))
         {
             value /= 1024;
             ++i;
@@ -111,148 +92,6 @@ namespace
     }
 }
 
-void Utils::Misc::shutdownComputer(const ShutdownDialogAction &action)
-{
-#if defined(Q_OS_WIN)
-    HANDLE hToken;            // handle to process token
-    TOKEN_PRIVILEGES tkp;     // pointer to token structure
-    if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken))
-        return;
-    // Get the LUID for shutdown privilege.
-    LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME,
-                         &tkp.Privileges[0].Luid);
-
-    tkp.PrivilegeCount = 1; // one privilege to set
-    tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
-
-    // Get shutdown privilege for this process.
-
-    AdjustTokenPrivileges(hToken, FALSE, &tkp, 0,
-                          (PTOKEN_PRIVILEGES) NULL, 0);
-
-    // Cannot test the return value of AdjustTokenPrivileges.
-
-    if (GetLastError() != ERROR_SUCCESS)
-        return;
-
-    if (action == ShutdownDialogAction::Suspend)
-    {
-        ::SetSuspendState(FALSE, FALSE, FALSE);
-    }
-    else if (action == ShutdownDialogAction::Hibernate)
-    {
-        ::SetSuspendState(TRUE, FALSE, FALSE);
-    }
-    else
-    {
-        const QString msg = QCoreApplication::translate("misc", "qBittorrent will shutdown the computer now because all downloads are complete.");
-        auto msgWchar = std::make_unique<wchar_t[]>(msg.length() + 1);
-        msg.toWCharArray(msgWchar.get());
-        ::InitiateSystemShutdownW(nullptr, msgWchar.get(), 10, TRUE, FALSE);
-    }
-
-    // Disable shutdown privilege.
-    tkp.Privileges[0].Attributes = 0;
-    AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES) NULL, 0);
-
-#elif defined(Q_OS_MACOS)
-    AEEventID EventToSend;
-    if (action != ShutdownDialogAction::Shutdown)
-        EventToSend = kAESleep;
-    else
-        EventToSend = kAEShutDown;
-    AEAddressDesc targetDesc;
-    const ProcessSerialNumber kPSNOfSystemProcess = {0, kSystemProcess};
-    AppleEvent eventReply = {typeNull, NULL};
-    AppleEvent appleEventToSend = {typeNull, NULL};
-
-    OSStatus error = AECreateDesc(typeProcessSerialNumber, &kPSNOfSystemProcess,
-                                  sizeof(kPSNOfSystemProcess), &targetDesc);
-
-    if (error != noErr)
-        return;
-
-    error = AECreateAppleEvent(kCoreEventClass, EventToSend, &targetDesc,
-                               kAutoGenerateReturnID, kAnyTransactionID, &appleEventToSend);
-
-    AEDisposeDesc(&targetDesc);
-    if (error != noErr)
-        return;
-
-    error = AESend(&appleEventToSend, &eventReply, kAENoReply,
-                   kAENormalPriority, kAEDefaultTimeout, NULL, NULL);
-
-    AEDisposeDesc(&appleEventToSend);
-    if (error != noErr)
-        return;
-
-    AEDisposeDesc(&eventReply);
-
-#elif (defined(Q_OS_UNIX) && defined(QT_DBUS_LIB))
-    // Use dbus to power off / suspend the system
-    if (action != ShutdownDialogAction::Shutdown)
-    {
-        // Some recent systems use systemd's logind
-        QDBusInterface login1Iface(u"org.freedesktop.login1"_qs, u"/org/freedesktop/login1"_qs,
-                                   u"org.freedesktop.login1.Manager"_qs, QDBusConnection::systemBus());
-        if (login1Iface.isValid())
-        {
-            if (action == ShutdownDialogAction::Suspend)
-                login1Iface.call(u"Suspend"_qs, false);
-            else
-                login1Iface.call(u"Hibernate"_qs, false);
-            return;
-        }
-        // Else, other recent systems use UPower
-        QDBusInterface upowerIface(u"org.freedesktop.UPower"_qs, u"/org/freedesktop/UPower"_qs,
-                                   u"org.freedesktop.UPower"_qs, QDBusConnection::systemBus());
-        if (upowerIface.isValid())
-        {
-            if (action == ShutdownDialogAction::Suspend)
-                upowerIface.call(u"Suspend"_qs);
-            else
-                upowerIface.call(u"Hibernate"_qs);
-            return;
-        }
-        // HAL (older systems)
-        QDBusInterface halIface(u"org.freedesktop.Hal"_qs, u"/org/freedesktop/Hal/devices/computer"_qs,
-                                u"org.freedesktop.Hal.Device.SystemPowerManagement"_qs,
-                                QDBusConnection::systemBus());
-        if (action == ShutdownDialogAction::Suspend)
-            halIface.call(u"Suspend"_qs, 5);
-        else
-            halIface.call(u"Hibernate"_qs);
-    }
-    else
-    {
-        // Some recent systems use systemd's logind
-        QDBusInterface login1Iface(u"org.freedesktop.login1"_qs, u"/org/freedesktop/login1"_qs,
-                                   u"org.freedesktop.login1.Manager"_qs, QDBusConnection::systemBus());
-        if (login1Iface.isValid())
-        {
-            login1Iface.call(u"PowerOff"_qs, false);
-            return;
-        }
-        // Else, other recent systems use ConsoleKit
-        QDBusInterface consolekitIface(u"org.freedesktop.ConsoleKit"_qs, u"/org/freedesktop/ConsoleKit/Manager"_qs,
-                                       u"org.freedesktop.ConsoleKit.Manager"_qs, QDBusConnection::systemBus());
-        if (consolekitIface.isValid())
-        {
-            consolekitIface.call(u"Stop"_qs);
-            return;
-        }
-        // HAL (older systems)
-        QDBusInterface halIface(u"org.freedesktop.Hal"_qs, u"/org/freedesktop/Hal/devices/computer"_qs,
-                                u"org.freedesktop.Hal.Device.SystemPowerManagement"_qs,
-                                QDBusConnection::systemBus());
-        halIface.call(u"Shutdown"_qs);
-    }
-
-#else
-    Q_UNUSED(action);
-#endif
-}
-
 QString Utils::Misc::unitString(const SizeUnit unit, const bool isSpeed)
 {
     const auto &unitString = units[static_cast<int>(unit)];
@@ -262,14 +101,33 @@ QString Utils::Misc::unitString(const SizeUnit unit, const bool isSpeed)
     return ret;
 }
 
-QString Utils::Misc::friendlyUnit(const qint64 bytes, const bool isSpeed)
+QString Utils::Misc::friendlyUnit(const qint64 bytes, const bool isSpeed, const int precision)
 {
     const std::optional<SplitToFriendlyUnitResult> result = splitToFriendlyUnit(bytes);
     if (!result)
         return QCoreApplication::translate("misc", "Unknown", "Unknown (size)");
-    return Utils::String::fromDouble(result->value, friendlyUnitPrecision(result->unit))
-           + C_NON_BREAKING_SPACE
-           + unitString(result->unit, isSpeed);
+
+    const int digitPrecision = (precision >= 0) ? precision : friendlyUnitPrecision(result->unit);
+    return Utils::String::fromDouble(result->value, digitPrecision)
+           + QChar::Nbsp + unitString(result->unit, isSpeed);
+}
+
+QString Utils::Misc::friendlyUnitCompact(const qint64 bytes)
+{
+    // avoid 1000-1023 values, use next larger unit instead
+    const std::optional<SplitToFriendlyUnitResult> result = splitToFriendlyUnit(bytes, 1000);
+    if (!result)
+        return QCoreApplication::translate("misc", "Unknown", "Unknown (size)");
+
+    int precision = 0;          // >= 100
+    if (result->value < 10)
+        precision = 2;          // 0 - 9.99
+    if (result->value < 100)
+        precision = 1;          // 10 - 99.9
+
+    return Utils::String::fromDouble(result->value, precision)
+           // use only one character for unit representation
+           + QChar::Nbsp + unitString(result->unit, false)[0];
 }
 
 int Utils::Misc::friendlyUnitPrecision(const SizeUnit unit)
@@ -308,53 +166,53 @@ bool Utils::Misc::isPreviewable(const Path &filePath)
 
     const QSet<QString> multimediaExtensions =
     {
-        u".3GP"_qs,
-        u".AAC"_qs,
-        u".AC3"_qs,
-        u".AIF"_qs,
-        u".AIFC"_qs,
-        u".AIFF"_qs,
-        u".ASF"_qs,
-        u".AU"_qs,
-        u".AVI"_qs,
-        u".FLAC"_qs,
-        u".FLV"_qs,
-        u".M3U"_qs,
-        u".M4A"_qs,
-        u".M4P"_qs,
-        u".M4V"_qs,
-        u".MID"_qs,
-        u".MKV"_qs,
-        u".MOV"_qs,
-        u".MP2"_qs,
-        u".MP3"_qs,
-        u".MP4"_qs,
-        u".MPC"_qs,
-        u".MPE"_qs,
-        u".MPEG"_qs,
-        u".MPG"_qs,
-        u".MPP"_qs,
-        u".OGG"_qs,
-        u".OGM"_qs,
-        u".OGV"_qs,
-        u".QT"_qs,
-        u".RA"_qs,
-        u".RAM"_qs,
-        u".RM"_qs,
-        u".RMV"_qs,
-        u".RMVB"_qs,
-        u".SWA"_qs,
-        u".SWF"_qs,
-        u".TS"_qs,
-        u".VOB"_qs,
-        u".WAV"_qs,
-        u".WMA"_qs,
-        u".WMV"_qs
+        u".3GP"_s,
+        u".AAC"_s,
+        u".AC3"_s,
+        u".AIF"_s,
+        u".AIFC"_s,
+        u".AIFF"_s,
+        u".ASF"_s,
+        u".AU"_s,
+        u".AVI"_s,
+        u".FLAC"_s,
+        u".FLV"_s,
+        u".M3U"_s,
+        u".M4A"_s,
+        u".M4P"_s,
+        u".M4V"_s,
+        u".MID"_s,
+        u".MKV"_s,
+        u".MOV"_s,
+        u".MP2"_s,
+        u".MP3"_s,
+        u".MP4"_s,
+        u".MPC"_s,
+        u".MPE"_s,
+        u".MPEG"_s,
+        u".MPG"_s,
+        u".MPP"_s,
+        u".OGG"_s,
+        u".OGM"_s,
+        u".OGV"_s,
+        u".QT"_s,
+        u".RA"_s,
+        u".RAM"_s,
+        u".RM"_s,
+        u".RMV"_s,
+        u".RMVB"_s,
+        u".SWA"_s,
+        u".SWF"_s,
+        u".TS"_s,
+        u".VOB"_s,
+        u".WAV"_s,
+        u".WMA"_s,
+        u".WMV"_s
     };
     return multimediaExtensions.contains(filePath.extension().toUpper());
 }
 
-QString Utils::Misc::userFriendlyDuration(const qlonglong seconds, const qlonglong maxCap)
+QString Utils::Misc::userFriendlyDuration(const qlonglong seconds, const qlonglong maxCap, const TimeResolution resolution)
 {
     if (seconds < 0)
         return C_INFINITY;
@@ -362,50 +220,40 @@ QString Utils::Misc::userFriendlyDuration(const qlonglong seconds, const qlonglo
         return C_INFINITY;
 
     if (seconds == 0)
-        return u"0"_qs;
+        return u"0"_s;
 
     if (seconds < 60)
-        return QCoreApplication::translate("misc", "< 1m", "< 1 minute");
+    {
+        if (resolution == TimeResolution::Minutes)
+            return QCoreApplication::translate("misc", "< 1m", "< 1 minute");
+
+        return QCoreApplication::translate("misc", "%1s", "e.g: 10 seconds").arg(QString::number(seconds));
+    }
 
     qlonglong minutes = (seconds / 60);
     if (minutes < 60)
-        return QCoreApplication::translate("misc", "%1m", "e.g: 10minutes").arg(QString::number(minutes));
+        return QCoreApplication::translate("misc", "%1m", "e.g: 10 minutes").arg(QString::number(minutes));
 
     qlonglong hours = (minutes / 60);
     if (hours < 24)
     {
         minutes -= (hours * 60);
-        return QCoreApplication::translate("misc", "%1h %2m", "e.g: 3hours 5minutes").arg(QString::number(hours), QString::number(minutes));
+        return QCoreApplication::translate("misc", "%1h %2m", "e.g: 3 hours 5 minutes").arg(QString::number(hours), QString::number(minutes));
     }
 
     qlonglong days = (hours / 24);
     if (days < 365)
     {
         hours -= (days * 24);
-        return QCoreApplication::translate("misc", "%1d %2h", "e.g: 2days 10hours").arg(QString::number(days), QString::number(hours));
+        return QCoreApplication::translate("misc", "%1d %2h", "e.g: 2 days 10 hours").arg(QString::number(days), QString::number(hours));
     }
 
     qlonglong years = (days / 365);
     days -= (years * 365);
-    return QCoreApplication::translate("misc", "%1y %2d", "e.g: 2years 10days").arg(QString::number(years), QString::number(days));
+    return QCoreApplication::translate("misc", "%1y %2d", "e.g: 2 years 10 days").arg(QString::number(years), QString::number(days));
 }
 
-QString Utils::Misc::getUserIDString()
-{
-    QString uid = u"0"_qs;
-#ifdef Q_OS_WIN
-    const int UNLEN = 256;
-    WCHAR buffer[UNLEN + 1] = {0};
-    DWORD buffer_len = sizeof(buffer) / sizeof(*buffer);
-    if (::GetUserNameW(buffer, &buffer_len))
-        uid = QString::fromWCharArray(buffer);
-#else
-    uid = QString::number(getuid());
-#endif
-    return uid;
-}
-
-QString Utils::Misc::languageToLocalizedString(const QString &localeStr)
+QString Utils::Misc::languageToLocalizedString(const QStringView localeStr)
 {
     if (localeStr.startsWith(u"eo", Qt::CaseInsensitive))
     {
@@ -430,7 +278,7 @@ QString Utils::Misc::languageToLocalizedString(const QString &localeStr)
     case QLocale::Byelorussian: return C_LOCALE_BYELORUSSIAN;
     case QLocale::Catalan: return C_LOCALE_CATALAN;
     case QLocale::Chinese:
-        switch (locale.country())
+        switch (locale.territory())
         {
         case QLocale::China: return C_LOCALE_CHINESE_SIMPLIFIED;
         case QLocale::HongKong: return C_LOCALE_CHINESE_TRADITIONAL_HK;
@@ -441,7 +289,7 @@ QString Utils::Misc::languageToLocalizedString(const QString &localeStr)
     case QLocale::Danish: return C_LOCALE_DANISH;
     case QLocale::Dutch: return C_LOCALE_DUTCH;
     case QLocale::English:
-        switch (locale.country())
+        switch (locale.territory())
         {
         case QLocale::Australia: return C_LOCALE_ENGLISH_AUSTRALIA;
         case QLocale::UnitedKingdom: return C_LOCALE_ENGLISH_UNITEDKINGDOM;
@@ -471,7 +319,7 @@ QString Utils::Misc::languageToLocalizedString(const QString &localeStr)
     case QLocale::Persian: return C_LOCALE_PERSIAN;
     case QLocale::Polish: return C_LOCALE_POLISH;
     case QLocale::Portuguese:
-        if (locale.country() == QLocale::Brazil)
+        if (locale.territory() == QLocale::Brazil)
             return C_LOCALE_PORTUGUESE_BRAZIL;
         return C_LOCALE_PORTUGUESE;
     case QLocale::Romanian: return C_LOCALE_ROMANIAN;
@@ -540,15 +388,15 @@ QString Utils::Misc::parseHtmlLinks(const QString &rawText)
         u"[a-zA-Z]{2,}"                              // one ab  (2 char or longer) --> us
         u"([a-zA-Z0-9\\?%=&/_\\.:#;-]*)"             // everything to 1st non-URI char, maybe nothing  in case of del.icio.us/path
         u")"
-        u")"_qs
+        u")"_s
         );
 
     // Capture links
-    result.replace(reURL, u"\\1<a href=\"\\2\">\\2</a>"_qs);
+    result.replace(reURL, u"\\1<a href=\"\\2\">\\2</a>"_s);
 
     // Capture links without scheme
-    const QRegularExpression reNoScheme(u"<a\\s+href=\"(?!https?)([a-zA-Z0-9\\?%=&/_\\.-:#]+)\\s*\">"_qs);
-    result.replace(reNoScheme, u"<a href=\"http://\\1\">"_qs);
+    const QRegularExpression reNoScheme(u"<a\\s+href=\"(?!https?)([a-zA-Z0-9\\?%=&/_\\.-:#]+)\\s*\">"_s);
+    result.replace(reNoScheme, u"<a href=\"http://\\1\">"_s);
 
     // to preserve plain text formatting
     result = u"<p style=\"white-space: pre-wrap;\">" + result + u"</p>";
@@ -559,7 +407,7 @@ QString Utils::Misc::osName()
 {
     // static initialization for usage in signal handler
     static const QString name =
-        u"%1 %2 %3"_qs
+        u"%1 %2 %3"_s
         .arg(QSysInfo::prettyProductName()
             , QSysInfo::kernelVersion()
             , QSysInfo::currentCpuArchitecture());
@@ -569,7 +417,7 @@ QString Utils::Misc::osName()
 QString Utils::Misc::boostVersionString()
 {
     // static initialization for usage in signal handler
-    static const QString ver = u"%1.%2.%3"_qs
+    static const QString ver = u"%1.%2.%3"_s
         .arg(QString::number(BOOST_VERSION / 100000)
             , QString::number((BOOST_VERSION / 100) % 1000)
             , QString::number(BOOST_VERSION % 100));
@@ -585,12 +433,10 @@ QString Utils::Misc::libtorrentVersionString()
 
 QString Utils::Misc::opensslVersionString()
 {
-#if (OPENSSL_VERSION_NUMBER >= 0x1010000f)
-    static const auto version {QString::fromLatin1(OpenSSL_version(OPENSSL_VERSION))};
-#else
-    static const auto version {QString::fromLatin1(SSLeay_version(SSLEAY_VERSION))};
-#endif
-    return version.section(u' ', 1, 1);
+    // static initialization for usage in signal handler
+    static const auto version {QString::fromLatin1(::OpenSSL_version(OPENSSL_VERSION))
+        .section(u' ', 1, 1)};
+    return version;
 }
 
 QString Utils::Misc::zlibVersionString()
@@ -599,16 +445,3 @@ QString Utils::Misc::zlibVersionString()
     static const auto version {QString::fromLatin1(zlibVersion())};
     return version;
 }
-
-#ifdef Q_OS_WIN
-Path Utils::Misc::windowsSystemPath()
-{
-    static const Path path = []() -> Path
-    {
-        WCHAR systemPath[MAX_PATH] = {0};
-        GetSystemDirectoryW(systemPath, sizeof(systemPath) / sizeof(WCHAR));
-        return Path(QString::fromWCharArray(systemPath));
-    }();
-    return path;
-}
-#endif
