@@ -28,20 +28,14 @@
 
 #pragma once
 
-#include <QtGlobal>
+#include <QtTypes>
 
-#ifdef Q_OS_WIN
-#include <Windows.h>
-#endif
+#include "base/pathfwd.h"
 
-#include <QString>
-
-#include "base/path.h"
-
-enum class ShutdownDialogAction;
+class QString;
+class QStringView;
 
 /*  Miscellaneous functions that can be useful */
-
 namespace Utils::Misc
 {
     // use binary prefix standards from IEC 60027-2
@@ -60,9 +54,13 @@ namespace Utils::Misc
         // YobiByte,   // 1024^8
     };
 
-    QString parseHtmlLinks(const QString &rawText);
+    enum class TimeResolution
+    {
+        Seconds,
+        Minutes
+    };
 
-    void shutdownComputer(const ShutdownDialogAction &action);
+    QString parseHtmlLinks(const QString &rawText);
 
     QString osName();
     QString boostVersionString();
@@ -74,7 +72,8 @@ namespace Utils::Misc
 
     // return the best user friendly storage unit (B, KiB, MiB, GiB, TiB)
     // value must be given in bytes
-    QString friendlyUnit(qint64 bytes, bool isSpeed = false);
+    QString friendlyUnit(qint64 bytes, bool isSpeed = false, int precision = -1);
+    QString friendlyUnitCompact(qint64 bytes);
     int friendlyUnitPrecision(SizeUnit unit);
     qint64 sizeInBytes(qreal size, SizeUnit unit);
 
@@ -82,19 +81,7 @@ namespace Utils::Misc
 
     // Take a number of seconds and return a user-friendly
     // time duration like "1d 2h 10m".
-    QString userFriendlyDuration(qlonglong seconds, qlonglong maxCap = -1);
-    QString getUserIDString();
+    QString userFriendlyDuration(qlonglong seconds, qlonglong maxCap = -1, TimeResolution resolution = TimeResolution::Minutes);
 
-    QString languageToLocalizedString(const QString &localeStr);
-
-#ifdef Q_OS_WIN
-    Path windowsSystemPath();
-
-    template <typename T>
-    T loadWinAPI(const QString &source, const char *funcName)
-    {
-        const std::wstring path = (windowsSystemPath() / Path(source)).toString().toStdWString();
-        return reinterpret_cast<T>(::GetProcAddress(::LoadLibraryW(path.c_str()), funcName));
-    }
-#endif // Q_OS_WIN
+    QString languageToLocalizedString(QStringView localeStr);
 }
