@@ -29,6 +29,7 @@
 #pragma once
 
 #include <optional>
+#include <utility>
 
 #include <QtContainerFwd>
 #include <QHostAddress>
@@ -40,7 +41,7 @@ class QString;
 namespace Utils::Net
 {
     // alias for `QHostAddress::parseSubnet()` return type
-    using Subnet = QPair<QHostAddress, int>;
+    using Subnet = std::pair<QHostAddress, int>;
 
     bool isValidIP(const QString &ip);
     std::optional<Subnet> parseSubnet(const QString &subnetStr);
@@ -52,6 +53,4 @@ namespace Utils::Net
     inline const int MAX_SSL_FILE_SIZE = 1024 * 1024;
     QList<QSslCertificate> loadSSLCertificate(const QByteArray &data);
     bool isSSLCertificatesValid(const QByteArray &data);
-    QSslKey loadSSLKey(const QByteArray &data);
-    bool isSSLKeyValid(const QByteArray &data);
 }
