@@ -32,15 +32,10 @@
 #include <cmath>
 
 #include <QLocale>
+#include <QRegularExpression>
 #include <QStringList>
 #include <QVector>
 
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
-#include <QRegularExpression>
-#else
-#include <QRegExp>
-#endif
-
 // to send numbers instead of strings with suffixes
 QString Utils::String::fromDouble(const double n, const int precision)
 {
@@ -54,21 +49,10 @@ QString Utils::String::fromDouble(const double n, const int precision)
     return QLocale::system().toString(std::floor(n * prec) / prec, 'f', precision);
 }
 
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
 QString Utils::String::wildcardToRegexPattern(const QString &pattern)
 {
     return QRegularExpression::wildcardToRegularExpression(pattern, QRegularExpression::UnanchoredWildcardConversion);
 }
-#else
-// This is marked as internal in QRegExp.cpp, but is exported. The alternative would be to
-// copy the code from QRegExp::wc2rx().
-QString qt_regexp_toCanonical(const QString &pattern, QRegExp::PatternSyntax patternSyntax);
-
-QString Utils::String::wildcardToRegexPattern(const QString &pattern)
-{
-    return qt_regexp_toCanonical(pattern, QRegExp::Wildcard);
-}
-#endif
 
 QStringList Utils::String::splitCommand(const QString &command)
 {
@@ -135,14 +119,3 @@ std::optional<double> Utils::String::parseDouble(const QString &string)
 
     return std::nullopt;
 }
-
-QString Utils::String::join(const QList<QStringView> &strings, const QStringView separator)
-{
-    if (strings.empty())
-        return {};
-
-    QString ret = strings[0].toString();
-    for (int i = 1; i < strings.count(); ++i)
-        ret += (separator + strings[i]);
-    return ret;
-}
