@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2022  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2022-2024  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2012  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -33,23 +33,17 @@
 
 #include <QDialog>
 
-#include "base/bittorrent/addtorrentparams.h"
-#include "base/bittorrent/magneturi.h"
-#include "base/bittorrent/torrentinfo.h"
 #include "base/path.h"
 #include "base/settingvalue.h"
+#include "filterpatternformat.h"
 
 class LineEdit;
-class TorrentFileGuard;
 
 namespace BitTorrent
 {
-    class InfoHash;
-}
-
-namespace Net
-{
-    struct DownloadResult;
+    class TorrentDescriptor;
+    class TorrentInfo;
+    struct AddTorrentParams;
 }
 
 namespace Ui
@@ -63,72 +57,59 @@ class AddNewTorrentDialog final : public QDialog
     Q_DISABLE_COPY_MOVE(AddNewTorrentDialog)
 
 public:
-    static const int minPathHistoryLength = 0;
-    static const int maxPathHistoryLength = 99;
-
+    explicit AddNewTorrentDialog(const BitTorrent::TorrentDescriptor &torrentDescr
+            , const BitTorrent::AddTorrentParams &inParams, QWidget *parent);
     ~AddNewTorrentDialog() override;
 
-    static bool isEnabled();
-    static void setEnabled(bool value);
-    static bool isTopLevel();
-    static void setTopLevel(bool value);
-    static int savePathHistoryLength();
-    static void setSavePathHistoryLength(int value);
-#ifndef Q_OS_MACOS
-    static bool isAttached();
-    static void setAttached(bool value);
-#endif
+    bool isDoNotDeleteTorrentChecked() const;
+    void updateMetadata(const BitTorrent::TorrentInfo &metadata);
 
-    static void show(const QString &source, const BitTorrent::AddTorrentParams &inParams, QWidget *parent);
-    static void show(const QString &source, QWidget *parent);
+signals:
+    void torrentAccepted(const BitTorrent::TorrentDescriptor &torrentDescriptor, const BitTorrent::AddTorrentParams &addTorrentParams);
+    void torrentRejected(const BitTorrent::TorrentDescriptor &torrentDescriptor);
 
 private slots:
     void updateDiskSpaceLabel();
     void onSavePathChanged(const Path &newPath);
     void onDownloadPathChanged(const Path &newPath);
     void onUseDownloadPathChanged(bool checked);
-    void updateMetadata(const BitTorrent::TorrentInfo &metadata);
-    void handleDownloadFinished(const Net::DownloadResult &downloadResult);
     void TMMChanged(int index);
     void categoryChanged(int index);
     void contentLayoutChanged();
-    void doNotDeleteTorrentClicked(bool checked);
 
     void accept() override;
     void reject() override;
 
 private:
     class TorrentContentAdaptor;
+    struct Context;
 
-    explicit AddNewTorrentDialog(const BitTorrent::AddTorrentParams &inParams, QWidget *parent);
+    void showEvent(QShowEvent *event) override;
 
-    bool loadTorrentFile(const QString &source);
-    bool loadTorrentImpl();
-    bool loadMagnet(const BitTorrent::MagnetUri &magnetUri);
+    void setCurrentContext(std::shared_ptr<Context> context);
+    void updateCurrentContext();
     void populateSavePaths();
     void loadState();
     void saveState();
     void setMetadataProgressIndicator(bool visibleIndicator, const QString &labelText = {});
     void setupTreeview();
     void saveTorrentFile();
-    bool hasMetadata() const;
-
-    void showEvent(QShowEvent *event) override;
+    void showContentFilterContextMenu();
+    void setContentFilterPattern();
 
     Ui::AddNewTorrentDialog *m_ui = nullptr;
     std::unique_ptr<TorrentContentAdaptor> m_contentAdaptor;
-    BitTorrent::MagnetUri m_magnetURI;
-    BitTorrent::TorrentInfo m_torrentInfo;
     int m_savePathIndex = -1;
     int m_downloadPathIndex = -1;
     bool m_useDownloadPath = false;
     LineEdit *m_filterLine = nullptr;
-    std::unique_ptr<TorrentFileGuard> m_torrentGuard;
-    BitTorrent::AddTorrentParams m_torrentParams;
+
+    std::shared_ptr<Context> m_currentContext;
 
     SettingValue<QSize> m_storeDialogSize;
     SettingValue<QString> m_storeDefaultCategory;
     SettingValue<bool> m_storeRememberLastSavePath;
     SettingValue<QByteArray> m_storeTreeHeaderState;
     SettingValue<QByteArray> m_storeSplitterState;
+    SettingValue<FilterPatternFormat> m_storeFilterPatternFormat;
 };
