@@ -51,17 +51,17 @@ namespace
         return (Net::DownloadManager::hasSupportedScheme(str)
             || str.startsWith(u"magnet:", Qt::CaseInsensitive)
 #ifdef QBT_USES_LIBTORRENT2
-            || ((str.size() == 64) && !str.contains(QRegularExpression(u"[^0-9A-Fa-f]"_qs))) // v2 hex-encoded SHA-256 info-hash
+            || ((str.size() == 64) && !str.contains(QRegularExpression(u"[^0-9A-Fa-f]"_s))) // v2 hex-encoded SHA-256 info-hash
 #endif
-            || ((str.size() == 40) && !str.contains(QRegularExpression(u"[^0-9A-Fa-f]"_qs))) // v1 hex-encoded SHA-1 info-hash
-            || ((str.size() == 32) && !str.contains(QRegularExpression(u"[^2-7A-Za-z]"_qs)))); // v1 Base32 encoded SHA-1 info-hash
+            || ((str.size() == 40) && !str.contains(QRegularExpression(u"[^0-9A-Fa-f]"_s))) // v1 hex-encoded SHA-1 info-hash
+            || ((str.size() == 32) && !str.contains(QRegularExpression(u"[^2-7A-Za-z]"_s)))); // v1 Base32 encoded SHA-1 info-hash
     }
 }
 
 DownloadFromURLDialog::DownloadFromURLDialog(QWidget *parent)
     : QDialog(parent)
     , m_ui(new Ui::DownloadFromURLDialog)
-    , m_storeDialogSize(SETTINGS_KEY(u"Size"_qs))
+    , m_storeDialogSize(SETTINGS_KEY(u"Size"_s))
 {
     m_ui->setupUi(this);
 
@@ -90,11 +90,12 @@ DownloadFromURLDialog::DownloadFromURLDialog(QWidget *parent)
             urls << urlString;
     }
 
-    const QString text = urls.join(u'\n')
-        + (!urls.isEmpty() ? u"\n" : u"");
-
-    m_ui->textUrls->setText(text);
-    m_ui->textUrls->moveCursor(QTextCursor::End);
+    if (!urls.isEmpty())
+    {
+        m_ui->textUrls->setText(urls.join(u'\n') + u"\n");
+        m_ui->textUrls->moveCursor(QTextCursor::End);
+        m_ui->buttonBox->setFocus();
+    }
 
     if (const QSize dialogSize = m_storeDialogSize; dialogSize.isValid())
         resize(dialogSize);
