@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2022  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2022-2024  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -29,16 +29,12 @@
 
 #include "mainwindow.h"
 
-#include <QtGlobal>
+#include <QtSystemDetection>
 
 #include <algorithm>
 #include <chrono>
 
-#if defined(Q_OS_WIN)
-#include <Windows.h>
-#include <versionhelpers.h>  // must follow after Windows.h
-#endif
-
+#include <QAction>
 #include <QActionGroup>
 #include <QClipboard>
 #include <QCloseEvent>
@@ -49,6 +45,7 @@
 #include <QFileSystemWatcher>
 #include <QKeyEvent>
 #include <QLabel>
+#include <QMenu>
 #include <QMessageBox>
 #include <QMetaObject>
 #include <QMimeData>
@@ -58,7 +55,6 @@
 #include <QSplitter>
 #include <QStatusBar>
 #include <QString>
-#include <QtGlobal>
 #include <QTimer>
 
 #include "base/bittorrent/session.h"
@@ -75,25 +71,26 @@
 #include "base/utils/password.h"
 #include "base/version.h"
 #include "aboutdialog.h"
-#include "addnewtorrentdialog.h"
 #include "autoexpandabledialog.h"
 #include "cookiesdialog.h"
 #include "desktopintegration.h"
 #include "downloadfromurldialog.h"
 #include "executionlogwidget.h"
 #include "hidabletabwidget.h"
+#include "interfaces/iguiapplication.h"
 #include "lineedit.h"
 #include "optionsdialog.h"
 #include "powermanagement/powermanagement.h"
 #include "properties/peerlistwidget.h"
 #include "properties/propertieswidget.h"
-#include "properties/trackerlistwidget.h"
+#include "properties/proptabbar.h"
 #include "rss/rsswidget.h"
 #include "search/searchwidget.h"
 #include "speedlimitdialog.h"
 #include "statsdialog.h"
 #include "statusbar.h"
 #include "torrentcreatordialog.h"
+#include "trackerlist/trackerlistwidget.h"
 #include "transferlistfilterswidget.h"
 #include "transferlistmodel.h"
 #include "transferlistwidget.h"
@@ -102,7 +99,7 @@
 #include "utils.h"
 
 #ifdef Q_OS_MACOS
-#include "macutilities.h"
+#include "macosdockbadge/badger.h"
 #endif
 #if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
 #include "programupdater.h"
@@ -126,24 +123,30 @@ namespace
     }
 }
 
-MainWindow::MainWindow(IGUIApplication *app, WindowState initialState)
+MainWindow::MainWindow(IGUIApplication *app, const WindowState initialState, const QString &titleSuffix)
     : GUIApplicationComponent(app)
-    , m_ui(new Ui::MainWindow)
-    , m_storeExecutionLogEnabled(EXECUTIONLOG_SETTINGS_KEY(u"Enabled"_s))
-    , m_storeDownloadTrackerFavicon(SETTINGS_KEY(u"DownloadTrackerFavicon"_s))
-    , m_storeExecutionLogTypes(EXECUTIONLOG_SETTINGS_KEY(u"Types"_s), Log::MsgType::ALL)
+    , m_ui {new Ui::MainWindow}
+    , m_downloadRate {Utils::Misc::friendlyUnit(0, true)}
+    , m_uploadRate {Utils::Misc::friendlyUnit(0, true)}
+    , m_storeExecutionLogEnabled {EXECUTIONLOG_SETTINGS_KEY(u"Enabled"_s)}
+    , m_storeDownloadTrackerFavicon {SETTINGS_KEY(u"DownloadTrackerFavicon"_s)}
+    , m_storeExecutionLogTypes {EXECUTIONLOG_SETTINGS_KEY(u"Types"_s), Log::MsgType::ALL}
+#ifdef Q_OS_MACOS
+    , m_badger {std::make_unique<MacUtils::Badger>()}
+#endif // Q_OS_MACOS
 {
     m_ui->setupUi(this);
 
     Preferences *const pref = Preferences::instance();
     m_uiLocked = pref->isUILocked();
-    setWindowTitle(QStringLiteral("qBittorrent " QBT_VERSION));
     m_displaySpeedInTitle = pref->speedInTitleBar();
     // Setting icons
 #ifndef Q_OS_MACOS
     setWindowIcon(UIThemeManager::instance()->getIcon(u"qbittorrent"_s));
 #endif // Q_OS_MACOS
 
+    setTitleSuffix(titleSuffix);
+
 #if (defined(Q_OS_UNIX))
     m_ui->actionOptions->setText(tr("Preferences"));
 #endif
@@ -166,25 +169,40 @@ MainWindow::MainWindow(IGUIApplication *app, WindowState initialState)
     m_ui->actionExit->setIcon(UIThemeManager::instance()->getIcon(u"application-exit"_s));
     m_ui->actionLock->setIcon(UIThemeManager::instance()->getIcon(u"object-locked"_s));
     m_ui->actionOptions->setIcon(UIThemeManager::instance()->getIcon(u"configure"_s, u"preferences-system"_s));
-    m_ui->actionPause->setIcon(UIThemeManager::instance()->getIcon(u"torrent-stop"_s, u"media-playback-pause"_s));
-    m_ui->actionPauseAll->setIcon(UIThemeManager::instance()->getIcon(u"torrent-stop"_s, u"media-playback-pause"_s));
     m_ui->actionStart->setIcon(UIThemeManager::instance()->getIcon(u"torrent-start"_s, u"media-playback-start"_s));
-    m_ui->actionStartAll->setIcon(UIThemeManager::instance()->getIcon(u"torrent-start"_s, u"media-playback-start"_s));
+    m_ui->actionStop->setIcon(UIThemeManager::instance()->getIcon(u"torrent-stop"_s, u"media-playback-pause"_s));
+    m_ui->actionPauseSession->setIcon(UIThemeManager::instance()->getIcon(u"pause-session"_s, u"media-playback-pause"_s));
+    m_ui->actionResumeSession->setIcon(UIThemeManager::instance()->getIcon(u"torrent-start"_s, u"media-playback-start"_s));
     m_ui->menuAutoShutdownOnDownloadsCompletion->setIcon(UIThemeManager::instance()->getIcon(u"task-complete"_s, u"application-exit"_s));
     m_ui->actionManageCookies->setIcon(UIThemeManager::instance()->getIcon(u"browser-cookies"_s, u"preferences-web-browser-cookies"_s));
     m_ui->menuLog->setIcon(UIThemeManager::instance()->getIcon(u"help-contents"_s));
     m_ui->actionCheckForUpdates->setIcon(UIThemeManager::instance()->getIcon(u"view-refresh"_s));
 
+    m_ui->actionPauseSession->setVisible(!BitTorrent::Session::instance()->isPaused());
+    m_ui->actionResumeSession->setVisible(BitTorrent::Session::instance()->isPaused());
+    connect(BitTorrent::Session::instance(), &BitTorrent::Session::paused, this, [this]
+    {
+        m_ui->actionPauseSession->setVisible(false);
+        m_ui->actionResumeSession->setVisible(true);
+        refreshWindowTitle();
+        refreshTrayIconTooltip();
+    });
+    connect(BitTorrent::Session::instance(), &BitTorrent::Session::resumed, this, [this]
+    {
+        m_ui->actionPauseSession->setVisible(true);
+        m_ui->actionResumeSession->setVisible(false);
+        refreshWindowTitle();
+        refreshTrayIconTooltip();
+    });
+
     auto *lockMenu = new QMenu(m_ui->menuView);
     lockMenu->addAction(tr("&Set Password"), this, &MainWindow::defineUILockPassword);
     lockMenu->addAction(tr("&Clear Password"), this, &MainWindow::clearUILockPassword);
     m_ui->actionLock->setMenu(lockMenu);
 
-    // Creating Bittorrent session
     updateAltSpeedsBtn(BitTorrent::Session::instance()->isAltGlobalSpeedLimitEnabled());
 
     connect(BitTorrent::Session::instance(), &BitTorrent::Session::speedLimitModeChanged, this, &MainWindow::updateAltSpeedsBtn);
-    connect(BitTorrent::Session::instance(), &BitTorrent::Session::recursiveTorrentDownloadPossible, this, &MainWindow::askRecursiveTorrentDownloadConfirmation);
 
     qDebug("create tabWidget");
     m_tabs = new HidableTabWidget(this);
@@ -246,7 +264,6 @@ MainWindow::MainWindow(IGUIApplication *app, WindowState initialState)
     connect(m_columnFilterEdit, &LineEdit::textChanged, this, &MainWindow::applyTransferListFilter);
     connect(hSplitter, &QSplitter::splitterMoved, this, &MainWindow::saveSettings);
     connect(m_splitter, &QSplitter::splitterMoved, this, &MainWindow::saveSplitterSettings);
-    connect(BitTorrent::Session::instance(), &BitTorrent::Session::trackersChanged, m_propertiesWidget, &PropertiesWidget::loadTrackers);
 
 #ifdef Q_OS_MACOS
     // Increase top spacing to avoid tab overlapping
@@ -286,17 +303,14 @@ MainWindow::MainWindow(IGUIApplication *app, WindowState initialState)
 
     // Transfer list slots
     connect(m_ui->actionStart, &QAction::triggered, m_transferListWidget, &TransferListWidget::startSelectedTorrents);
-    connect(m_ui->actionStartAll, &QAction::triggered, m_transferListWidget, &TransferListWidget::resumeAllTorrents);
-    connect(m_ui->actionPause, &QAction::triggered, m_transferListWidget, &TransferListWidget::pauseSelectedTorrents);
-    connect(m_ui->actionPauseAll, &QAction::triggered, m_transferListWidget, &TransferListWidget::pauseAllTorrents);
+    connect(m_ui->actionStop, &QAction::triggered, m_transferListWidget, &TransferListWidget::stopSelectedTorrents);
+    connect(m_ui->actionPauseSession, &QAction::triggered, m_transferListWidget, &TransferListWidget::pauseSession);
+    connect(m_ui->actionResumeSession, &QAction::triggered, m_transferListWidget, &TransferListWidget::resumeSession);
     connect(m_ui->actionDelete, &QAction::triggered, m_transferListWidget, &TransferListWidget::softDeleteSelectedTorrents);
     connect(m_ui->actionTopQueuePos, &QAction::triggered, m_transferListWidget, &TransferListWidget::topQueuePosSelectedTorrents);
     connect(m_ui->actionIncreaseQueuePos, &QAction::triggered, m_transferListWidget, &TransferListWidget::increaseQueuePosSelectedTorrents);
     connect(m_ui->actionDecreaseQueuePos, &QAction::triggered, m_transferListWidget, &TransferListWidget::decreaseQueuePosSelectedTorrents);
     connect(m_ui->actionBottomQueuePos, &QAction::triggered, m_transferListWidget, &TransferListWidget::bottomQueuePosSelectedTorrents);
-#ifndef Q_OS_MACOS
-    connect(m_ui->actionToggleVisibility, &QAction::triggered, this, &MainWindow::toggleVisibility);
-#endif
     connect(m_ui->actionMinimize, &QAction::triggered, this, &MainWindow::minimizeWindow);
     connect(m_ui->actionUseAlternativeSpeedLimits, &QAction::triggered, this, &MainWindow::toggleAlternativeSpeeds);
 
@@ -322,13 +336,15 @@ MainWindow::MainWindow(IGUIApplication *app, WindowState initialState)
     // Initialise system sleep inhibition timer
     m_pwr = new PowerManagement(this);
     m_preventTimer = new QTimer(this);
+    m_preventTimer->setSingleShot(true);
     connect(m_preventTimer, &QTimer::timeout, this, &MainWindow::updatePowerManagementState);
-    m_preventTimer->start(PREVENT_SUSPEND_INTERVAL);
+    connect(pref, &Preferences::changed, this, &MainWindow::updatePowerManagementState);
+    updatePowerManagementState();
 
     // Configure BT session according to options
     loadPreferences();
 
-    connect(BitTorrent::Session::instance(), &BitTorrent::Session::statsUpdated, this, &MainWindow::reloadSessionStats);
+    connect(BitTorrent::Session::instance(), &BitTorrent::Session::statsUpdated, this, &MainWindow::loadSessionStats);
     connect(BitTorrent::Session::instance(), &BitTorrent::Session::torrentsUpdated, this, &MainWindow::reloadTorrentStats);
 
     // Accept drag 'n drops
@@ -386,7 +402,7 @@ MainWindow::MainWindow(IGUIApplication *app, WindowState initialState)
     // Load Window state and sizes
     loadSettings();
 
-    app->desktopIntegration()->setMenu(createDesktopIntegrationMenu());
+    populateDesktopIntegrationMenu();
 #ifndef Q_OS_MACOS
     m_ui->actionLock->setVisible(app->desktopIntegration()->isActive());
     connect(app->desktopIntegration(), &DesktopIntegration::stateChanged, this, [this, app]()
@@ -466,7 +482,7 @@ MainWindow::MainWindow(IGUIApplication *app, WindowState initialState)
     {
         m_transferListWidget->applyStatusFilter(pref->getTransSelFilter());
         m_transferListWidget->applyCategoryFilter(QString());
-        m_transferListWidget->applyTagFilter(QString());
+        m_transferListWidget->applyTagFilter(std::nullopt);
         m_transferListWidget->applyTrackerFilterAll();
     }
 
@@ -525,6 +541,16 @@ void MainWindow::setDownloadTrackerFavicon(const bool value)
     m_storeDownloadTrackerFavicon = value;
 }
 
+void MainWindow::setTitleSuffix(const QString &suffix)
+{
+    const auto emDash = QChar(0x2014);
+    const QString separator = u' ' + emDash + u' ';
+    m_windowTitle = QStringLiteral("qBittorrent " QBT_VERSION)
+        + (!suffix.isEmpty() ? (separator + suffix) : QString());
+
+    refreshWindowTitle();
+}
+
 void MainWindow::addToolbarContextMenu()
 {
     const Preferences *const pref = Preferences::instance();
@@ -671,7 +697,7 @@ void MainWindow::displayRSSTab(bool enable)
         // RSS tab
         if (!m_rssWidget)
         {
-            m_rssWidget = new RSSWidget(m_tabs);
+            m_rssWidget = new RSSWidget(app(), m_tabs);
             connect(m_rssWidget.data(), &RSSWidget::unreadCountUpdated, this, &MainWindow::handleRSSUnreadCountUpdated);
 #ifdef Q_OS_MACOS
             m_tabs->addTab(m_rssWidget, tr("RSS (%1)").arg(RSS::Session::instance()->rootFolder()->unreadCount()));
@@ -726,10 +752,18 @@ void MainWindow::displaySearchTab(bool enable)
     }
 }
 
-void MainWindow::focusSearchFilter()
+void MainWindow::toggleFocusBetweenLineEdits()
 {
-    m_columnFilterEdit->setFocus();
-    m_columnFilterEdit->selectAll();
+    if (m_columnFilterEdit->hasFocus() && (m_propertiesWidget->tabBar()->currentIndex() == PropTabBar::FilesTab))
+    {
+        m_propertiesWidget->contentFilterLine()->setFocus();
+        m_propertiesWidget->contentFilterLine()->selectAll();
+    }
+    else
+    {
+        m_columnFilterEdit->setFocus();
+        m_columnFilterEdit->selectAll();
+    }
 }
 
 void MainWindow::updateNbTorrents()
@@ -742,9 +776,8 @@ void MainWindow::on_actionDocumentation_triggered() const
     QDesktopServices::openUrl(QUrl(u"https://doc.qbittorrent.org"_s));
 }
 
-void MainWindow::tabChanged(int newTab)
+void MainWindow::tabChanged([[maybe_unused]] const int newTab)
 {
-    Q_UNUSED(newTab);
     // We cannot rely on the index newTab
     // because the tab order is undetermined now
     if (m_tabs->currentWidget() == m_splitter)
@@ -858,23 +891,23 @@ void MainWindow::createKeyboardShortcuts()
     const auto *switchExecutionLogShortcut = new QShortcut((Qt::ALT | Qt::Key_4), this);
     connect(switchExecutionLogShortcut, &QShortcut::activated, this, &MainWindow::displayExecutionLogTab);
     const auto *switchSearchFilterShortcut = new QShortcut(QKeySequence::Find, m_transferListWidget);
-    connect(switchSearchFilterShortcut, &QShortcut::activated, this, &MainWindow::focusSearchFilter);
+    connect(switchSearchFilterShortcut, &QShortcut::activated, this, &MainWindow::toggleFocusBetweenLineEdits);
     const auto *switchSearchFilterShortcutAlternative = new QShortcut((Qt::CTRL | Qt::Key_E), m_transferListWidget);
-    connect(switchSearchFilterShortcutAlternative, &QShortcut::activated, this, &MainWindow::focusSearchFilter);
+    connect(switchSearchFilterShortcutAlternative, &QShortcut::activated, this, &MainWindow::toggleFocusBetweenLineEdits);
 
     m_ui->actionDocumentation->setShortcut(QKeySequence::HelpContents);
     m_ui->actionOptions->setShortcut(Qt::ALT | Qt::Key_O);
     m_ui->actionStatistics->setShortcut(Qt::CTRL | Qt::Key_I);
     m_ui->actionStart->setShortcut(Qt::CTRL | Qt::Key_S);
-    m_ui->actionStartAll->setShortcut(Qt::CTRL | Qt::SHIFT | Qt::Key_S);
-    m_ui->actionPause->setShortcut(Qt::CTRL | Qt::Key_P);
-    m_ui->actionPauseAll->setShortcut(Qt::CTRL | Qt::SHIFT | Qt::Key_P);
+    m_ui->actionStop->setShortcut(Qt::CTRL | Qt::Key_P);
+    m_ui->actionPauseSession->setShortcut(Qt::CTRL | Qt::SHIFT | Qt::Key_P);
+    m_ui->actionResumeSession->setShortcut(Qt::CTRL | Qt::SHIFT | Qt::Key_S);
     m_ui->actionBottomQueuePos->setShortcut(Qt::CTRL | Qt::SHIFT | Qt::Key_Minus);
     m_ui->actionDecreaseQueuePos->setShortcut(Qt::CTRL | Qt::Key_Minus);
     m_ui->actionIncreaseQueuePos->setShortcut(Qt::CTRL | Qt::Key_Plus);
     m_ui->actionTopQueuePos->setShortcut(Qt::CTRL | Qt::SHIFT | Qt::Key_Plus);
 #ifdef Q_OS_MACOS
-    m_ui->actionMinimize->setShortcut(Qt::CTRL + Qt::Key_M);
+    m_ui->actionMinimize->setShortcut(Qt::CTRL | Qt::Key_M);
     addAction(m_ui->actionMinimize);
 #endif
 }
@@ -920,37 +953,6 @@ void MainWindow::displayExecutionLogTab()
 
 // End of keyboard shortcuts slots
 
-void MainWindow::askRecursiveTorrentDownloadConfirmation(const BitTorrent::Torrent *torrent)
-{
-    if (!Preferences::instance()->isRecursiveDownloadEnabled())
-        return;
-
-    const auto torrentID = torrent->id();
-
-    QMessageBox *confirmBox = new QMessageBox(QMessageBox::Question, tr("Recursive download confirmation")
-        , tr("The torrent '%1' contains .torrent files, do you want to proceed with their downloads?").arg(torrent->name())
-        , (QMessageBox::Yes | QMessageBox::No | QMessageBox::NoToAll), this);
-    confirmBox->setAttribute(Qt::WA_DeleteOnClose);
-
-    const QAbstractButton *yesButton = confirmBox->button(QMessageBox::Yes);
-    QAbstractButton *neverButton = confirmBox->button(QMessageBox::NoToAll);
-    neverButton->setText(tr("Never"));
-
-    connect(confirmBox, &QMessageBox::buttonClicked, this
-        , [torrentID, yesButton, neverButton](const QAbstractButton *button)
-    {
-        if (button == yesButton)
-        {
-            BitTorrent::Session::instance()->recursiveTorrentDownload(torrentID);
-        }
-        else if (button == neverButton)
-        {
-            Preferences::instance()->setRecursiveDownloadEnabled(false);
-        }
-    });
-    confirmBox->open();
-}
-
 void MainWindow::on_actionSetGlobalSpeedLimits_triggered()
 {
     auto *dialog = new SpeedLimitDialog {this};
@@ -1125,7 +1127,6 @@ void MainWindow::keyPressEvent(QKeyEvent *event)
 
         if (mimeData->hasText())
         {
-            const bool useTorrentAdditionDialog = AddNewTorrentDialog::isEnabled();
             const QStringList lines = mimeData->text().split(u'\n', Qt::SkipEmptyParts);
 
             for (QString line : lines)
@@ -1135,10 +1136,7 @@ void MainWindow::keyPressEvent(QKeyEvent *event)
                 if (!isTorrentLink(line))
                     continue;
 
-                if (useTorrentAdditionDialog)
-                    AddNewTorrentDialog::show(line, this);
-                else
-                    BitTorrent::Session::instance()->addTorrent(line);
+                app()->addTorrentManager()->addTorrent(line);
             }
 
             return;
@@ -1321,14 +1319,8 @@ void MainWindow::dropEvent(QDropEvent *event)
     }
 
     // Download torrents
-    const bool useTorrentAdditionDialog = AddNewTorrentDialog::isEnabled();
     for (const QString &file : asConst(torrentFiles))
-    {
-        if (useTorrentAdditionDialog)
-            AddNewTorrentDialog::show(file, this);
-        else
-            BitTorrent::Session::instance()->addTorrent(file);
-    }
+        app()->addTorrentManager()->addTorrent(file);
     if (!torrentFiles.isEmpty()) return;
 
     // Create torrent
@@ -1352,12 +1344,6 @@ void MainWindow::dragEnterEvent(QDragEnterEvent *event)
         event->acceptProposedAction();
 }
 
-/*****************************************************
-*                                                   *
-*                     Torrent                       *
-*                                                   *
-*****************************************************/
-
 // Display a dialog to allow user to add
 // torrents to download list
 void MainWindow::on_actionOpen_triggered()
@@ -1365,22 +1351,14 @@ void MainWindow::on_actionOpen_triggered()
     Preferences *const pref = Preferences::instance();
     // Open File Open Dialog
     // Note: it is possible to select more than one file
-    const QStringList pathsList =
-        QFileDialog::getOpenFileNames(this, tr("Open Torrent Files"), pref->getMainLastDir().data(),
-                                      tr("Torrent Files") + u" (*" + TORRENT_FILE_EXTENSION + u')');
+    const QStringList pathsList = QFileDialog::getOpenFileNames(this, tr("Open Torrent Files")
+            , pref->getMainLastDir().data(),  tr("Torrent Files") + u" (*" + TORRENT_FILE_EXTENSION + u')');
 
     if (pathsList.isEmpty())
         return;
 
-    const bool useTorrentAdditionDialog = AddNewTorrentDialog::isEnabled();
-
     for (const QString &file : pathsList)
-    {
-        if (useTorrentAdditionDialog)
-            AddNewTorrentDialog::show(file, this);
-        else
-            BitTorrent::Session::instance()->addTorrent(file);
-    }
+        app()->addTorrentManager()->addTorrent(file);
 
     // Save last dir to remember it
     const Path topDir {pathsList.at(0)};
@@ -1429,8 +1407,7 @@ void MainWindow::showFiltersSidebar(const bool show)
         connect(BitTorrent::Session::instance(), &BitTorrent::Session::trackersAdded, m_transferListFiltersWidget, &TransferListFiltersWidget::addTrackers);
         connect(BitTorrent::Session::instance(), &BitTorrent::Session::trackersRemoved, m_transferListFiltersWidget, &TransferListFiltersWidget::removeTrackers);
         connect(BitTorrent::Session::instance(), &BitTorrent::Session::trackersChanged, m_transferListFiltersWidget, &TransferListFiltersWidget::refreshTrackers);
-        connect(BitTorrent::Session::instance(), &BitTorrent::Session::trackerlessStateChanged, m_transferListFiltersWidget, &TransferListFiltersWidget::changeTrackerless);
-        connect(BitTorrent::Session::instance(), &BitTorrent::Session::trackerEntriesUpdated, m_transferListFiltersWidget, &TransferListFiltersWidget::trackerEntriesUpdated);
+        connect(BitTorrent::Session::instance(), &BitTorrent::Session::trackerEntryStatusesUpdated, m_transferListFiltersWidget, &TransferListFiltersWidget::trackerEntryStatusesUpdated);
 
         m_splitter->insertWidget(0, m_transferListFiltersWidget);
         m_splitter->setCollapsible(0, true);
@@ -1467,8 +1444,6 @@ void MainWindow::loadPreferences()
 
     showStatusBar(pref->isStatusbarDisplayed());
 
-    updatePowerManagementState();
-
     m_transferListWidget->setAlternatingRowColors(pref->useAlternatingRowColors());
     m_propertiesWidget->getFilesList()->setAlternatingRowColors(pref->useAlternatingRowColors());
     m_propertiesWidget->getTrackerList()->setAlternatingRowColors(pref->useAlternatingRowColors());
@@ -1531,35 +1506,21 @@ void MainWindow::loadPreferences()
     qDebug("GUI settings loaded");
 }
 
-void MainWindow::reloadSessionStats()
+void MainWindow::loadSessionStats()
 {
-    const BitTorrent::SessionStatus &status = BitTorrent::Session::instance()->status();
+    const auto *btSession = BitTorrent::Session::instance();
+    const BitTorrent::SessionStatus &status = btSession->status();
+    m_downloadRate = Utils::Misc::friendlyUnit(status.payloadDownloadRate, true);
+    m_uploadRate = Utils::Misc::friendlyUnit(status.payloadUploadRate, true);
 
     // update global information
 #ifdef Q_OS_MACOS
-    if (status.payloadDownloadRate > 0)
-    {
-        MacUtils::setBadgeLabelText(tr("%1/s", "s is a shorthand for seconds")
-            .arg(Utils::Misc::friendlyUnit(status.payloadDownloadRate)));
-    }
-    else if (!MacUtils::badgeLabelText().isEmpty())
-    {
-        MacUtils::setBadgeLabelText({});
-    }
+    m_badger->updateSpeed(status.payloadDownloadRate, status.payloadUploadRate);
 #else
-    const auto toolTip = u"%1\n%2"_s.arg(
-        tr("DL speed: %1", "e.g: Download speed: 10 KiB/s").arg(Utils::Misc::friendlyUnit(status.payloadDownloadRate, true))
-        , tr("UP speed: %1", "e.g: Upload speed: 10 KiB/s").arg(Utils::Misc::friendlyUnit(status.payloadUploadRate, true)));
-    app()->desktopIntegration()->setToolTip(toolTip); // tray icon
+    refreshTrayIconTooltip();
 #endif  // Q_OS_MACOS
 
-    if (m_displaySpeedInTitle)
-    {
-        setWindowTitle(tr("[D: %1, U: %2] qBittorrent %3", "D = Download; U = Upload; %3 is qBittorrent version")
-            .arg(Utils::Misc::friendlyUnit(status.payloadDownloadRate, true)
-                , Utils::Misc::friendlyUnit(status.payloadUploadRate, true)
-                , QStringLiteral(QBT_VERSION)));
-    }
+    refreshWindowTitle();
 }
 
 void MainWindow::reloadTorrentStats(const QVector<BitTorrent::Torrent *> &torrents)
@@ -1571,39 +1532,23 @@ void MainWindow::reloadTorrentStats(const QVector<BitTorrent::Torrent *> &torren
     }
 }
 
-/*****************************************************
-*                                                   *
-*                      Utils                        *
-*                                                   *
-*****************************************************/
-
 void MainWindow::downloadFromURLList(const QStringList &urlList)
 {
-    const bool useTorrentAdditionDialog = AddNewTorrentDialog::isEnabled();
     for (const QString &url : urlList)
-    {
-        if (useTorrentAdditionDialog)
-            AddNewTorrentDialog::show(url, this);
-        else
-            BitTorrent::Session::instance()->addTorrent(url);
-    }
+        app()->addTorrentManager()->addTorrent(url);
 }
 
-/*****************************************************
-*                                                   *
-*                     Options                       *
-*                                                   *
-*****************************************************/
-
-QMenu *MainWindow::createDesktopIntegrationMenu()
+void MainWindow::populateDesktopIntegrationMenu()
 {
-    auto *menu = new QMenu;
+    auto *menu = app()->desktopIntegration()->menu();
+    menu->clear();
 
 #ifndef Q_OS_MACOS
     connect(menu, &QMenu::aboutToShow, this, [this]()
     {
         m_ui->actionToggleVisibility->setText(isVisible() ? tr("Hide") : tr("Show"));
     });
+    connect(m_ui->actionToggleVisibility, &QAction::triggered, this, &MainWindow::toggleVisibility);
 
     menu->addAction(m_ui->actionToggleVisibility);
     menu->addSeparator();
@@ -1617,8 +1562,8 @@ QMenu *MainWindow::createDesktopIntegrationMenu()
     menu->addAction(m_ui->actionSetGlobalSpeedLimits);
     menu->addSeparator();
 
-    menu->addAction(m_ui->actionStartAll);
-    menu->addAction(m_ui->actionPauseAll);
+    menu->addAction(m_ui->actionResumeSession);
+    menu->addAction(m_ui->actionPauseSession);
 
 #ifndef Q_OS_MACOS
     menu->addSeparator();
@@ -1627,8 +1572,6 @@ QMenu *MainWindow::createDesktopIntegrationMenu()
 
     if (m_uiLocked)
         menu->setEnabled(false);
-
-    return menu;
 }
 
 void MainWindow::updateAltSpeedsBtn(const bool alternative)
@@ -1681,10 +1624,7 @@ void MainWindow::on_actionSpeedInTitleBar_triggered()
 {
     m_displaySpeedInTitle = static_cast<QAction *>(sender())->isChecked();
     Preferences::instance()->showSpeedInTitleBar(m_displaySpeedInTitle);
-    if (m_displaySpeedInTitle)
-        reloadSessionStats();
-    else
-        setWindowTitle(QStringLiteral("qBittorrent " QBT_VERSION));
+    refreshWindowTitle();
 }
 
 void MainWindow::on_actionRSSReader_triggered()
@@ -1695,11 +1635,11 @@ void MainWindow::on_actionRSSReader_triggered()
 
 void MainWindow::on_actionSearchWidget_triggered()
 {
-    if (!m_hasPython && m_ui->actionSearchWidget->isChecked())
+    if (m_ui->actionSearchWidget->isChecked())
     {
         const Utils::ForeignApps::PythonInfo pyInfo = Utils::ForeignApps::pythonInfo();
 
-        // Not installed
+        // Not found
         if (!pyInfo.isValid())
         {
             m_ui->actionSearchWidget->setChecked(false);
@@ -1739,7 +1679,6 @@ void MainWindow::on_actionSearchWidget_triggered()
             return;
         }
 
-        m_hasPython = true;
         m_ui->actionSearchWidget->setChecked(true);
         Preferences::instance()->setSearchEnabled(true);
     }
@@ -1747,12 +1686,6 @@ void MainWindow::on_actionSearchWidget_triggered()
     displaySearchTab(m_ui->actionSearchWidget->isChecked());
 }
 
-/*****************************************************
-*                                                   *
-*                 HTTP Downloader                   *
-*                                                   *
-*****************************************************/
-
 // Display an input dialog to prompt user for
 // an url
 void MainWindow::on_actionDownloadFromURL_triggered()
@@ -1929,23 +1862,26 @@ void MainWindow::on_actionAutoShutdown_toggled(bool enabled)
     Preferences::instance()->setShutdownWhenDownloadsComplete(enabled);
 }
 
-void MainWindow::updatePowerManagementState()
+void MainWindow::updatePowerManagementState() const
 {
-    const bool preventFromSuspendWhenDownloading = Preferences::instance()->preventFromSuspendWhenDownloading();
-    const bool preventFromSuspendWhenSeeding = Preferences::instance()->preventFromSuspendWhenSeeding();
+    const auto *pref = Preferences::instance();
+    const bool preventFromSuspendWhenDownloading = pref->preventFromSuspendWhenDownloading();
+    const bool preventFromSuspendWhenSeeding = pref->preventFromSuspendWhenSeeding();
 
     const QVector<BitTorrent::Torrent *> allTorrents = BitTorrent::Session::instance()->torrents();
     const bool inhibitSuspend = std::any_of(allTorrents.cbegin(), allTorrents.cend(), [&](const BitTorrent::Torrent *torrent)
     {
-        if (preventFromSuspendWhenDownloading && (!torrent->isFinished() && !torrent->isPaused() && !torrent->isErrored() && torrent->hasMetadata()))
+        if (preventFromSuspendWhenDownloading && (!torrent->isFinished() && !torrent->isStopped() && !torrent->isErrored() && torrent->hasMetadata()))
             return true;
 
-        if (preventFromSuspendWhenSeeding && (torrent->isFinished() && !torrent->isPaused()))
+        if (preventFromSuspendWhenSeeding && (torrent->isFinished() && !torrent->isStopped()))
             return true;
 
         return torrent->isMoving();
     });
     m_pwr->setActivityState(inhibitSuspend);
+
+    m_preventTimer->start(PREVENT_SUSPEND_INTERVAL);
 }
 
 void MainWindow::applyTransferListFilter()
@@ -1953,6 +1889,45 @@ void MainWindow::applyTransferListFilter()
     m_transferListWidget->applyFilter(m_columnFilterEdit->text(), m_columnFilterComboBox->currentData().value<TransferListModel::Column>());
 }
 
+void MainWindow::refreshWindowTitle()
+{
+    const auto *btSession = BitTorrent::Session::instance();
+    if (btSession->isPaused())
+    {
+        const QString title = tr("[PAUSED] %1", "%1 is the rest of the window title").arg(m_windowTitle);
+        setWindowTitle(title);
+    }
+    else
+    {
+        if (m_displaySpeedInTitle)
+        {
+            const QString title = tr("[D: %1, U: %2] %3", "D = Download; U = Upload; %3 is the rest of the window title")
+                    .arg(m_downloadRate, m_uploadRate, m_windowTitle);
+            setWindowTitle(title);
+        }
+        else
+        {
+            setWindowTitle(m_windowTitle);
+        }
+    }
+}
+
+void MainWindow::refreshTrayIconTooltip()
+{
+    const auto *btSession = BitTorrent::Session::instance();
+    if (!btSession->isPaused())
+    {
+        const auto toolTip = u"%1\n%2"_s.arg(
+                tr("DL speed: %1", "e.g: Download speed: 10 KiB/s").arg(m_downloadRate)
+                , tr("UP speed: %1", "e.g: Upload speed: 10 KiB/s").arg(m_uploadRate));
+        app()->desktopIntegration()->setToolTip(toolTip);
+    }
+    else
+    {
+        app()->desktopIntegration()->setToolTip(tr("Paused"));
+    }
+}
+
 #if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
 void MainWindow::checkProgramUpdate(const bool invokedByUser)
 {
@@ -1978,15 +1953,7 @@ void MainWindow::installPython()
 {
     setCursor(QCursor(Qt::WaitCursor));
     // Download python
-#ifdef QBT_APP_64BIT
-    const auto installerURL = ::IsWindows8OrGreater()
-        ? u"https://www.python.org/ftp/python/3.10.11/python-3.10.11-amd64.exe"_s
-        : u"https://www.python.org/ftp/python/3.8.10/python-3.8.10-amd64.exe"_s;
-#else
-    const auto installerURL = ::IsWindows8OrGreater()
-        ? u"https://www.python.org/ftp/python/3.10.11/python-3.10.11.exe"_s
-        : u"https://www.python.org/ftp/python/3.8.10/python-3.8.10.exe"_s;
-#endif
+    const auto installerURL = u"https://www.python.org/ftp/python/3.10.11/python-3.10.11-amd64.exe"_s;
     Net::DownloadManager::instance()->download(
             Net::DownloadRequest(installerURL).saveToFile(true)
             , Preferences::instance()->useProxyForGeneralPurposes()
