@@ -26,14 +26,14 @@
  * exception statement from your version.
  */
 
-#ifndef RAISEDMESSAGEBOX_H
-#define RAISEDMESSAGEBOX_H
+#pragma once
 
 #include <QMessageBox>
 
 class RaisedMessageBox final : public QMessageBox
 {
-  Q_OBJECT
+    Q_OBJECT
+    Q_DISABLE_COPY_MOVE(RaisedMessageBox)
 
 public:
     static QMessageBox::StandardButton critical(QWidget *parent, const QString &title, const QString &text, QMessageBox::StandardButtons buttons = QMessageBox::Ok, QMessageBox::StandardButton defaultButton = QMessageBox::NoButton);
@@ -46,11 +46,6 @@ protected:
 
 private:
     RaisedMessageBox(QMessageBox::Icon icon, const QString &title, const QString &text, QMessageBox::StandardButtons buttons = NoButton, QWidget *parent = nullptr, Qt::WindowFlags f = Qt::Dialog|Qt::MSWindowsFixedSizeDialogHint);
-    RaisedMessageBox();
-    RaisedMessageBox(RaisedMessageBox const&);
-    void operator=(RaisedMessageBox const&);
 
     static QMessageBox::StandardButton impl(const QMessageBox::Icon &icon, QWidget *parent, const QString &title, const QString &text, QMessageBox::StandardButtons buttons = QMessageBox::Ok, QMessageBox::StandardButton defaultButton = QMessageBox::NoButton);
 };
-
-#endif // RAISEDMESSAGEBOX_H
