@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2017  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2017, 2023  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2010  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -29,7 +29,6 @@
 
 #include "automatedrssdownloader.h"
 
-#include <QtGlobal>
 #include <QCursor>
 #include <QFileDialog>
 #include <QMenu>
@@ -50,50 +49,60 @@
 #include "base/utils/compare.h"
 #include "base/utils/io.h"
 #include "base/utils/string.h"
+#include "gui/addtorrentparamswidget.h"
 #include "gui/autoexpandabledialog.h"
 #include "gui/torrentcategorydialog.h"
 #include "gui/uithememanager.h"
 #include "gui/utils.h"
 #include "ui_automatedrssdownloader.h"
 
-const QString EXT_JSON = u".json"_qs;
-const QString EXT_LEGACY = u".rssrules"_qs;
+const QString EXT_JSON = u".json"_s;
+const QString EXT_LEGACY = u".rssrules"_s;
 
 AutomatedRssDownloader::AutomatedRssDownloader(QWidget *parent)
     : QDialog(parent)
-    , m_formatFilterJSON(u"%1 (*%2)"_qs.arg(tr("Rules"), EXT_JSON))
-    , m_formatFilterLegacy(u"%1 (*%2)"_qs.arg(tr("Rules (legacy)"), EXT_LEGACY))
-    , m_ui(new Ui::AutomatedRssDownloader)
-    , m_currentRuleItem(nullptr)
-    , m_storeDialogSize {u"RssFeedDownloader/geometrySize"_qs}
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
-    , m_storeHSplitterSize {u"GUI/Qt6/RSSFeedDownloader/HSplitterSizes"_qs}
-#else
-    , m_storeHSplitterSize {u"RssFeedDownloader/qt5/hsplitterSizes"_qs}
-#endif
-
+    , m_formatFilterJSON {u"%1 (*%2)"_s.arg(tr("Rules"), EXT_JSON)}
+    , m_formatFilterLegacy {u"%1 (*%2)"_s.arg(tr("Rules (legacy)"), EXT_LEGACY)}
+    , m_ui {new Ui::AutomatedRssDownloader}
+    , m_addTorrentParamsWidget {new AddTorrentParamsWidget}
+    , m_storeDialogSize {u"RssFeedDownloader/geometrySize"_s}
+    , m_storeMainSplitterState {u"GUI/Qt6/RSSFeedDownloader/HSplitterSizes"_s}
+    , m_storeRuleDefSplitterState {u"GUI/Qt6/RSSFeedDownloader/RuleDefSplitterState"_s}
 {
     m_ui->setupUi(this);
+
+    connect(m_ui->buttonBox, &QDialogButtonBox::accepted, this, &QDialog::accept);
+    connect(m_ui->buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
+
+    m_ui->torrentParametersGroupBox->layout()->addWidget(m_addTorrentParamsWidget);
+
+    m_ui->prioritySpinBox->setMinimum(std::numeric_limits<int>::min());
+    m_ui->prioritySpinBox->setMaximum(std::numeric_limits<int>::max());
+
+    connect(m_ui->addRuleBtn, &QPushButton::clicked, this, &AutomatedRssDownloader::onAddRuleBtnClicked);
+    connect(m_ui->removeRuleBtn, &QPushButton::clicked, this, &AutomatedRssDownloader::onRemoveRuleBtnClicked);
+    connect(m_ui->exportBtn, &QPushButton::clicked, this, &AutomatedRssDownloader::onExportBtnClicked);
+    connect(m_ui->importBtn, &QPushButton::clicked, this, &AutomatedRssDownloader::onImportBtnClicked);
+    connect(m_ui->renameRuleBtn, &QPushButton::clicked, this, &AutomatedRssDownloader::onRenameRuleBtnClicked);
+
     // Icons
-    m_ui->removeRuleBtn->setIcon(UIThemeManager::instance()->getIcon(u"edit-clear"_qs, u"list-remove"_qs));
-    m_ui->addRuleBtn->setIcon(UIThemeManager::instance()->getIcon(u"list-add"_qs));
-    m_ui->addCategoryBtn->setIcon(UIThemeManager::instance()->getIcon(u"list-add"_qs));
+    m_ui->renameRuleBtn->setIcon(UIThemeManager::instance()->getIcon(u"edit-rename"_s));
+    m_ui->removeRuleBtn->setIcon(UIThemeManager::instance()->getIcon(u"edit-clear"_s, u"list-remove"_s));
+    m_ui->addRuleBtn->setIcon(UIThemeManager::instance()->getIcon(u"list-add"_s));
 
     // Ui Settings
-    m_ui->listRules->setSortingEnabled(true);
-    m_ui->listRules->setSelectionMode(QAbstractItemView::ExtendedSelection);
-    m_ui->treeMatchingArticles->setSortingEnabled(true);
-    m_ui->treeMatchingArticles->sortByColumn(0, Qt::AscendingOrder);
-    m_ui->hsplitter->setCollapsible(0, false);
-    m_ui->hsplitter->setCollapsible(1, false);
-    m_ui->hsplitter->setCollapsible(2, true); // Only the preview list is collapsible
-    m_ui->lineSavePath->setDialogCaption(tr("Destination directory"));
-    m_ui->lineSavePath->setMode(FileSystemPathEdit::Mode::DirectorySave);
+    m_ui->ruleList->setSortingEnabled(true);
+    m_ui->ruleList->setSelectionMode(QAbstractItemView::ExtendedSelection);
+    m_ui->matchingArticlesTree->setSortingEnabled(true);
+    m_ui->matchingArticlesTree->sortByColumn(0, Qt::AscendingOrder);
+    m_ui->mainSplitter->setCollapsible(0, false);
+    m_ui->mainSplitter->setCollapsible(1, false);
+    m_ui->mainSplitter->setCollapsible(2, true); // Only the preview list is collapsible
 
     connect(m_ui->checkRegex, &QAbstractButton::toggled, this, &AutomatedRssDownloader::updateFieldsToolTips);
-    connect(m_ui->listRules, &QWidget::customContextMenuRequested, this, &AutomatedRssDownloader::displayRulesListMenu);
+    connect(m_ui->ruleList, &QWidget::customContextMenuRequested, this, &AutomatedRssDownloader::displayRulesListMenu);
 
-    m_episodeRegex = new QRegularExpression(u"^(^\\d{1,4}x(\\d{1,4}(-(\\d{1,4})?)?;){1,}){1,1}"_qs
+    m_episodeRegex = new QRegularExpression(u"^(^\\d{1,4}x(\\d{1,4}(-(\\d{1,4})?)?;){1,}){1,1}"_s
                                             , QRegularExpression::CaseInsensitiveOption);
     const QString tip = u"<p>" + tr("Matches articles based on episode filter.") + u"</p><p><b>" + tr("Example: ")
         + u"1x2;8-15;5;30-;</b>" + tr(" will match 2, 5, 8 through 15, 30 and onward episodes of season one", "example X will match") + u"</p>"
@@ -106,7 +115,6 @@ AutomatedRssDownloader::AutomatedRssDownloader(QWidget *parent)
         + u"<li>" + tr("Infinite range: <b>1x25-;</b> matches episodes 25 and upward of season one, and all episodes of later seasons") + u"</li>" + u"</ul></li></ul>";
     m_ui->lineEFilter->setToolTip(tip);
 
-    initCategoryCombobox();
     loadSettings();
 
     connect(RSS::AutoDownloader::instance(), &RSS::AutoDownloader::ruleAdded, this, &AutomatedRssDownloader::handleRuleAdded);
@@ -130,27 +138,28 @@ AutomatedRssDownloader::AutomatedRssDownloader(QWidget *parent)
 
     connect(m_ui->listFeeds, &QListWidget::itemChanged, this, &AutomatedRssDownloader::handleFeedCheckStateChange);
 
-    connect(m_ui->listRules, &QListWidget::itemSelectionChanged, this, &AutomatedRssDownloader::updateRuleDefinitionBox);
-    connect(m_ui->listRules, &QListWidget::itemChanged, this, &AutomatedRssDownloader::handleRuleCheckStateChange);
+    connect(m_ui->ruleList, &QListWidget::itemSelectionChanged, this, &AutomatedRssDownloader::updateRuleDefinitionBox);
+    connect(m_ui->ruleList, &QListWidget::itemChanged, this, &AutomatedRssDownloader::handleRuleCheckStateChange);
 
-    const auto *editHotkey = new QShortcut(Qt::Key_F2, m_ui->listRules, nullptr, nullptr, Qt::WidgetShortcut);
+    const auto *editHotkey = new QShortcut(Qt::Key_F2, m_ui->ruleList, nullptr, nullptr, Qt::WidgetShortcut);
     connect(editHotkey, &QShortcut::activated, this, &AutomatedRssDownloader::renameSelectedRule);
-    const auto *deleteHotkey = new QShortcut(QKeySequence::Delete, m_ui->listRules, nullptr, nullptr, Qt::WidgetShortcut);
-    connect(deleteHotkey, &QShortcut::activated, this, &AutomatedRssDownloader::on_removeRuleBtn_clicked);
+    const auto *deleteHotkey = new QShortcut(QKeySequence::Delete, m_ui->ruleList, nullptr, nullptr, Qt::WidgetShortcut);
+    connect(deleteHotkey, &QShortcut::activated, this, &AutomatedRssDownloader::onRemoveRuleBtnClicked);
 
-    connect(m_ui->listRules, &QAbstractItemView::doubleClicked, this, &AutomatedRssDownloader::renameSelectedRule);
+    connect(m_ui->ruleList, &QAbstractItemView::doubleClicked, this, &AutomatedRssDownloader::renameSelectedRule);
 
     loadFeedList();
 
-    m_ui->listRules->blockSignals(true);
+    m_ui->ruleList->blockSignals(true);
     for (const RSS::AutoDownloadRule &rule : asConst(RSS::AutoDownloader::instance()->rules()))
         createRuleItem(rule);
-    m_ui->listRules->blockSignals(false);
+    m_ui->ruleList->blockSignals(false);
 
     updateRuleDefinitionBox();
 
     if (RSS::AutoDownloader::instance()->isProcessingEnabled())
         m_ui->labelWarn->hide();
+
     connect(RSS::AutoDownloader::instance(), &RSS::AutoDownloader::processingStateChanged
             , this, &AutomatedRssDownloader::handleProcessingStateChanged);
 }
@@ -170,19 +179,23 @@ void AutomatedRssDownloader::loadSettings()
     if (const QSize dialogSize = m_storeDialogSize; dialogSize.isValid())
         resize(dialogSize);
 
-    if (const QByteArray hSplitterSize = m_storeHSplitterSize; !hSplitterSize.isEmpty())
-        m_ui->hsplitter->restoreState(hSplitterSize);
+    if (const QByteArray mainSplitterSize = m_storeMainSplitterState; !mainSplitterSize.isEmpty())
+        m_ui->mainSplitter->restoreState(mainSplitterSize);
+
+    if (const QByteArray ruleDefSplitterSize = m_storeRuleDefSplitterState; !ruleDefSplitterSize.isEmpty())
+        m_ui->ruleDefSplitter->restoreState(ruleDefSplitterSize);
 }
 
 void AutomatedRssDownloader::saveSettings()
 {
     m_storeDialogSize = size();
-    m_storeHSplitterSize = m_ui->hsplitter->saveState();
+    m_storeMainSplitterState = m_ui->mainSplitter->saveState();
+    m_storeRuleDefSplitterState = m_ui->ruleDefSplitter->saveState();
 }
 
 void AutomatedRssDownloader::createRuleItem(const RSS::AutoDownloadRule &rule)
 {
-    QListWidgetItem *item = new QListWidgetItem(rule.name(), m_ui->listRules);
+    QListWidgetItem *item = new QListWidgetItem(rule.name(), m_ui->ruleList);
     m_itemsByRuleName.insert(rule.name(), item);
     item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
     item->setCheckState(rule.isEnabled() ? Qt::Checked : Qt::Unchecked);
@@ -192,7 +205,7 @@ void AutomatedRssDownloader::loadFeedList()
 {
     const QSignalBlocker feedListSignalBlocker(m_ui->listFeeds);
 
-    for (const auto feed : asConst(RSS::Session::instance()->feeds()))
+    for (const auto *feed : asConst(RSS::Session::instance()->feeds()))
     {
         QListWidgetItem *item = new QListWidgetItem(feed->name(), m_ui->listFeeds);
         item->setData(Qt::UserRole, feed->url());
@@ -211,7 +224,7 @@ void AutomatedRssDownloader::updateFeedList()
     if (m_currentRuleItem)
         selection << m_currentRuleItem;
     else
-        selection = m_ui->listRules->selectedItems();
+        selection = m_ui->ruleList->selectedItems();
 
     bool enable = !selection.isEmpty();
 
@@ -248,13 +261,20 @@ void AutomatedRssDownloader::updateFeedList()
 
 void AutomatedRssDownloader::updateRuleDefinitionBox()
 {
-    const QList<QListWidgetItem *> selection = m_ui->listRules->selectedItems();
+    const QList<QListWidgetItem *> selection = m_ui->ruleList->selectedItems();
     QListWidgetItem *currentRuleItem = ((selection.count() == 1) ? selection.first() : nullptr);
+
+    // Enable the edit rule button but only if we have 1 rule selected
+    if (selection.count() == 1)
+        m_ui->renameRuleBtn->setEnabled(true);
+    else
+        m_ui->renameRuleBtn->setEnabled(false);
+
     if (m_currentRuleItem != currentRuleItem)
     {
         saveEditedRule(); // Save previous rule first
         m_currentRuleItem = currentRuleItem;
-        //m_ui->listRules->setCurrentItem(m_currentRuleItem);
+        //m_ui->ruleList->setCurrentItem(m_currentRuleItem);
     }
 
     // Update rule definition box
@@ -262,31 +282,22 @@ void AutomatedRssDownloader::updateRuleDefinitionBox()
     {
         m_currentRule = RSS::AutoDownloader::instance()->ruleByName(m_currentRuleItem->text());
 
+        m_ui->prioritySpinBox->setValue(m_currentRule.priority());
+
+        m_addTorrentParamsWidget->setAddTorrentParams(m_currentRule.addTorrentParams());
+
         m_ui->lineContains->setText(m_currentRule.mustContain());
         m_ui->lineNotContains->setText(m_currentRule.mustNotContain());
         if (!m_currentRule.episodeFilter().isEmpty())
             m_ui->lineEFilter->setText(m_currentRule.episodeFilter());
         else
             m_ui->lineEFilter->clear();
-        m_ui->checkBoxSaveDiffDir->setChecked(!m_currentRule.savePath().isEmpty());
-        m_ui->lineSavePath->setSelectedPath(m_currentRule.savePath());
         m_ui->checkRegex->blockSignals(true);
         m_ui->checkRegex->setChecked(m_currentRule.useRegex());
         m_ui->checkRegex->blockSignals(false);
         m_ui->checkSmart->blockSignals(true);
         m_ui->checkSmart->setChecked(m_currentRule.useSmartFilter());
         m_ui->checkSmart->blockSignals(false);
-        m_ui->comboCategory->setCurrentIndex(m_ui->comboCategory->findText(m_currentRule.assignedCategory()));
-        if (m_currentRule.assignedCategory().isEmpty())
-            m_ui->comboCategory->clearEditText();
-        int index = 0;
-        if (m_currentRule.addPaused().has_value())
-            index = (*m_currentRule.addPaused() ? 1 : 2);
-        m_ui->comboAddPaused->setCurrentIndex(index);
-        index = 0;
-        if (m_currentRule.torrentContentLayout())
-            index = static_cast<int>(*m_currentRule.torrentContentLayout()) + 1;
-        m_ui->comboContentLayout->setCurrentIndex(index);
         m_ui->spinIgnorePeriod->setValue(m_currentRule.ignoreDays());
         QDateTime dateTime = m_currentRule.lastMatch();
         QString lMatch;
@@ -300,13 +311,13 @@ void AutomatedRssDownloader::updateRuleDefinitionBox()
         updateEpisodeFilterValidity();
 
         updateFieldsToolTips(m_ui->checkRegex->isChecked());
-        m_ui->ruleDefBox->setEnabled(true);
+        m_ui->ruleScrollArea->setEnabled(true);
     }
     else
     {
         m_currentRule = RSS::AutoDownloadRule();
         clearRuleDefinitionBox();
-        m_ui->ruleDefBox->setEnabled(false);
+        m_ui->ruleScrollArea->setEnabled(false);
     }
 
     updateFeedList();
@@ -315,71 +326,46 @@ void AutomatedRssDownloader::updateRuleDefinitionBox()
 
 void AutomatedRssDownloader::clearRuleDefinitionBox()
 {
+    m_addTorrentParamsWidget->setAddTorrentParams({});
+    m_ui->prioritySpinBox->setValue(0);
     m_ui->lineContains->clear();
     m_ui->lineNotContains->clear();
     m_ui->lineEFilter->clear();
-    m_ui->checkBoxSaveDiffDir->setChecked(false);
-    m_ui->lineSavePath->clear();
-    m_ui->comboCategory->clearEditText();
-    m_ui->comboCategory->setCurrentIndex(-1);
     m_ui->checkRegex->setChecked(false);
     m_ui->checkSmart->setChecked(false);
     m_ui->spinIgnorePeriod->setValue(0);
-    m_ui->comboAddPaused->clearEditText();
-    m_ui->comboAddPaused->setCurrentIndex(-1);
-    m_ui->comboContentLayout->clearEditText();
-    m_ui->comboContentLayout->setCurrentIndex(-1);
     updateFieldsToolTips(m_ui->checkRegex->isChecked());
     updateMustLineValidity();
     updateMustNotLineValidity();
     updateEpisodeFilterValidity();
 }
 
-void AutomatedRssDownloader::initCategoryCombobox()
-{
-    // Load torrent categories
-    QStringList categories = BitTorrent::Session::instance()->categories();
-    std::sort(categories.begin(), categories.end(), Utils::Compare::NaturalLessThan<Qt::CaseInsensitive>());
-    m_ui->comboCategory->addItem(u""_qs);
-    m_ui->comboCategory->addItems(categories);
-}
-
 void AutomatedRssDownloader::updateEditedRule()
 {
-    if (!m_currentRuleItem || !m_ui->ruleDefBox->isEnabled()) return;
+    if (!m_currentRuleItem || !m_ui->ruleScrollArea->isEnabled())
+        return;
 
     m_currentRule.setEnabled(m_currentRuleItem->checkState() != Qt::Unchecked);
+    m_currentRule.setPriority(m_ui->prioritySpinBox->value());
     m_currentRule.setUseRegex(m_ui->checkRegex->isChecked());
     m_currentRule.setUseSmartFilter(m_ui->checkSmart->isChecked());
     m_currentRule.setMustContain(m_ui->lineContains->text());
     m_currentRule.setMustNotContain(m_ui->lineNotContains->text());
     m_currentRule.setEpisodeFilter(m_ui->lineEFilter->text());
-    m_currentRule.setSavePath(m_ui->checkBoxSaveDiffDir->isChecked() ? m_ui->lineSavePath->selectedPath() : Path());
-    m_currentRule.setCategory(m_ui->comboCategory->currentText());
-    std::optional<bool> addPaused;
-    if (m_ui->comboAddPaused->currentIndex() == 1)
-        addPaused = true;
-    else if (m_ui->comboAddPaused->currentIndex() == 2)
-        addPaused = false;
-    m_currentRule.setAddPaused(addPaused);
-
-    std::optional<BitTorrent::TorrentContentLayout> contentLayout;
-    if (m_ui->comboContentLayout->currentIndex() > 0)
-        contentLayout = static_cast<BitTorrent::TorrentContentLayout>(m_ui->comboContentLayout->currentIndex() - 1);
-    m_currentRule.setTorrentContentLayout(contentLayout);
-
     m_currentRule.setIgnoreDays(m_ui->spinIgnorePeriod->value());
+
+    m_currentRule.setAddTorrentParams(m_addTorrentParamsWidget->addTorrentParams());
 }
 
 void AutomatedRssDownloader::saveEditedRule()
 {
-    if (!m_currentRuleItem || !m_ui->ruleDefBox->isEnabled()) return;
+    if (!m_currentRuleItem || !m_ui->ruleScrollArea->isEnabled()) return;
 
     updateEditedRule();
-    RSS::AutoDownloader::instance()->insertRule(m_currentRule);
+    RSS::AutoDownloader::instance()->setRule(m_currentRule);
 }
 
-void AutomatedRssDownloader::on_addRuleBtn_clicked()
+void AutomatedRssDownloader::onAddRuleBtnClicked()
 {
 //    saveEditedRule();
 
@@ -396,12 +382,12 @@ void AutomatedRssDownloader::on_addRuleBtn_clicked()
         return;
     }
 
-    RSS::AutoDownloader::instance()->insertRule(RSS::AutoDownloadRule(ruleName));
+    RSS::AutoDownloader::instance()->setRule(RSS::AutoDownloadRule(ruleName));
 }
 
-void AutomatedRssDownloader::on_removeRuleBtn_clicked()
+void AutomatedRssDownloader::onRemoveRuleBtnClicked()
 {
-    const QList<QListWidgetItem *> selection = m_ui->listRules->selectedItems();
+    const QList<QListWidgetItem *> selection = m_ui->ruleList->selectedItems();
     if (selection.isEmpty()) return;
 
     // Ask for confirmation
@@ -416,18 +402,12 @@ void AutomatedRssDownloader::on_removeRuleBtn_clicked()
         RSS::AutoDownloader::instance()->removeRule(item->text());
 }
 
-void AutomatedRssDownloader::on_addCategoryBtn_clicked()
+void AutomatedRssDownloader::onRenameRuleBtnClicked()
 {
-    const QString newCategoryName = TorrentCategoryDialog::createCategory(this);
-
-    if (!newCategoryName.isEmpty())
-    {
-        m_ui->comboCategory->addItem(newCategoryName);
-        m_ui->comboCategory->setCurrentText(newCategoryName);
-    }
+    renameSelectedRule();
 }
 
-void AutomatedRssDownloader::on_exportBtn_clicked()
+void AutomatedRssDownloader::onExportBtnClicked()
 {
     if (RSS::AutoDownloader::instance()->rules().isEmpty())
     {
@@ -439,7 +419,7 @@ void AutomatedRssDownloader::on_exportBtn_clicked()
     QString selectedFilter {m_formatFilterJSON};
     Path path {QFileDialog::getSaveFileName(
                 this, tr("Export RSS rules"), QDir::homePath()
-                , u"%1;;%2"_qs.arg(m_formatFilterJSON, m_formatFilterLegacy), &selectedFilter)};
+                , u"%1;;%2"_s.arg(m_formatFilterJSON, m_formatFilterLegacy), &selectedFilter)};
 
     if (path.isEmpty()) return;
 
@@ -470,21 +450,22 @@ void AutomatedRssDownloader::on_exportBtn_clicked()
     }
 }
 
-void AutomatedRssDownloader::on_importBtn_clicked()
+void AutomatedRssDownloader::onImportBtnClicked()
 {
     QString selectedFilter {m_formatFilterJSON};
     const Path path {QFileDialog::getOpenFileName(
                     this, tr("Import RSS rules"), QDir::homePath()
-                    , u"%1;;%2"_qs.arg(m_formatFilterJSON, m_formatFilterLegacy), &selectedFilter)};
-    if (!path.exists())
-        return;
+                    , u"%1;;%2"_s.arg(m_formatFilterJSON, m_formatFilterLegacy), &selectedFilter)};
 
-    QFile file {path.data()};
-    if (!file.open(QIODevice::ReadOnly))
+    const int fileMaxSize = 10 * 1024 * 1024;
+    const auto readResult = Utils::IO::readFile(path, fileMaxSize);
+    if (!readResult)
     {
-        QMessageBox::critical(
-                    this, tr("I/O Error")
-                    , tr("Failed to open the file. Reason: %1").arg(file.errorString()));
+        if (readResult.error().status == Utils::IO::ReadError::NotExist)
+            return;
+
+        QMessageBox::critical(this, tr("Import error")
+            , tr("Failed to read the file. %1").arg(readResult.error().message));
         return;
     }
 
@@ -497,13 +478,12 @@ void AutomatedRssDownloader::on_importBtn_clicked()
 
     try
     {
-        RSS::AutoDownloader::instance()->importRules(file.readAll(),format);
+        RSS::AutoDownloader::instance()->importRules(readResult.value(), format);
     }
     catch (const RSS::ParsingError &error)
     {
-        QMessageBox::critical(
-                    this, tr("Import Error")
-                    , tr("Failed to import the selected rules file. Reason: %1").arg(error.message()));
+        QMessageBox::critical(this, tr("Import error")
+            , tr("Failed to import the selected rules file. Reason: %1").arg(error.message()));
     }
 }
 
@@ -512,29 +492,29 @@ void AutomatedRssDownloader::displayRulesListMenu()
     QMenu *menu = new QMenu(this);
     menu->setAttribute(Qt::WA_DeleteOnClose);
 
-    menu->addAction(UIThemeManager::instance()->getIcon(u"list-add"_qs), tr("Add new rule...")
-        , this, &AutomatedRssDownloader::on_addRuleBtn_clicked);
+    menu->addAction(UIThemeManager::instance()->getIcon(u"list-add"_s), tr("Add new rule...")
+                    , this, &AutomatedRssDownloader::onAddRuleBtnClicked);
 
-    const QList<QListWidgetItem *> selection = m_ui->listRules->selectedItems();
+    const QList<QListWidgetItem *> selection = m_ui->ruleList->selectedItems();
 
     if (!selection.isEmpty())
     {
         if (selection.count() == 1)
         {
-            menu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_qs, u"list-remove"_qs), tr("Delete rule")
-                , this, &AutomatedRssDownloader::on_removeRuleBtn_clicked);
+            menu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_s, u"list-remove"_s), tr("Delete rule")
+                            , this, &AutomatedRssDownloader::onRemoveRuleBtnClicked);
             menu->addSeparator();
-            menu->addAction(UIThemeManager::instance()->getIcon(u"edit-rename"_qs), tr("Rename rule...")
+            menu->addAction(UIThemeManager::instance()->getIcon(u"edit-rename"_s), tr("Rename rule...")
                 , this, &AutomatedRssDownloader::renameSelectedRule);
         }
         else
         {
-            menu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_qs, u"list-remove"_qs), tr("Delete selected rules")
-                , this, &AutomatedRssDownloader::on_removeRuleBtn_clicked);
+            menu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_s, u"list-remove"_s), tr("Delete selected rules")
+                            , this, &AutomatedRssDownloader::onRemoveRuleBtnClicked);
         }
 
         menu->addSeparator();
-        menu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_qs), tr("Clear downloaded episodes...")
+        menu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_s), tr("Clear downloaded episodes...")
             , this, &AutomatedRssDownloader::clearSelectedRuleDownloadedEpisodeList);
     }
 
@@ -543,7 +523,7 @@ void AutomatedRssDownloader::displayRulesListMenu()
 
 void AutomatedRssDownloader::renameSelectedRule()
 {
-    const QList<QListWidgetItem *> selection = m_ui->listRules->selectedItems();
+    const QList<QListWidgetItem *> selection = m_ui->ruleList->selectedItems();
     if (selection.isEmpty()) return;
 
     QListWidgetItem *item = selection.first();
@@ -571,7 +551,7 @@ void AutomatedRssDownloader::renameSelectedRule()
 
 void AutomatedRssDownloader::handleRuleCheckStateChange(QListWidgetItem *ruleItem)
 {
-    m_ui->listRules->setCurrentItem(ruleItem);
+    m_ui->ruleList->setCurrentItem(ruleItem);
 }
 
 void AutomatedRssDownloader::clearSelectedRuleDownloadedEpisodeList()
@@ -592,7 +572,7 @@ void AutomatedRssDownloader::clearSelectedRuleDownloadedEpisodeList()
 void AutomatedRssDownloader::handleFeedCheckStateChange(QListWidgetItem *feedItem)
 {
     const QString feedURL = feedItem->data(Qt::UserRole).toString();
-    for (QListWidgetItem *ruleItem : asConst(m_ui->listRules->selectedItems()))
+    for (QListWidgetItem *ruleItem : asConst(m_ui->ruleList->selectedItems()))
     {
         RSS::AutoDownloadRule rule = (ruleItem == m_currentRuleItem
                                        ? m_currentRule
@@ -605,7 +585,7 @@ void AutomatedRssDownloader::handleFeedCheckStateChange(QListWidgetItem *feedIte
 
         rule.setFeedURLs(affectedFeeds);
         if (ruleItem != m_currentRuleItem)
-            RSS::AutoDownloader::instance()->insertRule(rule);
+            RSS::AutoDownloader::instance()->setRule(rule);
         else
             m_currentRule = rule;
     }
@@ -615,20 +595,20 @@ void AutomatedRssDownloader::handleFeedCheckStateChange(QListWidgetItem *feedIte
 
 void AutomatedRssDownloader::updateMatchingArticles()
 {
-    m_ui->treeMatchingArticles->clear();
+    m_ui->matchingArticlesTree->clear();
 
-    for (const QListWidgetItem *ruleItem : asConst(m_ui->listRules->selectedItems()))
+    for (const QListWidgetItem *ruleItem : asConst(m_ui->ruleList->selectedItems()))
     {
         RSS::AutoDownloadRule rule = (ruleItem == m_currentRuleItem
                                        ? m_currentRule
                                        : RSS::AutoDownloader::instance()->ruleByName(ruleItem->text()));
         for (const QString &feedURL : asConst(rule.feedURLs()))
         {
-            auto feed = RSS::Session::instance()->feedByURL(feedURL);
+            auto *feed = RSS::Session::instance()->feedByURL(feedURL);
             if (!feed) continue; // feed doesn't exist
 
             QStringList matchingArticles;
-            for (const auto article : asConst(feed->articles()))
+            for (const auto *article : asConst(feed->articles()))
                 if (rule.matches(article->data()))
                     matchingArticles << article->title();
             if (!matchingArticles.isEmpty())
@@ -642,13 +622,13 @@ void AutomatedRssDownloader::updateMatchingArticles()
 void AutomatedRssDownloader::addFeedArticlesToTree(RSS::Feed *feed, const QStringList &articles)
 {
     // Turn off sorting while inserting
-    m_ui->treeMatchingArticles->setSortingEnabled(false);
+    m_ui->matchingArticlesTree->setSortingEnabled(false);
 
     // Check if this feed is already in the tree
     QTreeWidgetItem *treeFeedItem = nullptr;
-    for (int i = 0; i < m_ui->treeMatchingArticles->topLevelItemCount(); ++i)
+    for (int i = 0; i < m_ui->matchingArticlesTree->topLevelItemCount(); ++i)
     {
-        QTreeWidgetItem *item = m_ui->treeMatchingArticles->topLevelItem(i);
+        QTreeWidgetItem *item = m_ui->matchingArticlesTree->topLevelItem(i);
         if (item->data(0, Qt::UserRole).toString() == feed->url())
         {
             treeFeedItem = item;
@@ -664,9 +644,9 @@ void AutomatedRssDownloader::addFeedArticlesToTree(RSS::Feed *feed, const QStrin
         QFont f = treeFeedItem->font(0);
         f.setBold(true);
         treeFeedItem->setFont(0, f);
-        treeFeedItem->setData(0, Qt::DecorationRole, UIThemeManager::instance()->getIcon(u"directory"_qs));
+        treeFeedItem->setData(0, Qt::DecorationRole, UIThemeManager::instance()->getIcon(u"directory"_s));
         treeFeedItem->setData(0, Qt::UserRole, feed->url());
-        m_ui->treeMatchingArticles->addTopLevelItem(treeFeedItem);
+        m_ui->matchingArticlesTree->addTopLevelItem(treeFeedItem);
     }
 
     // Insert the articles
@@ -683,9 +663,9 @@ void AutomatedRssDownloader::addFeedArticlesToTree(RSS::Feed *feed, const QStrin
         }
     }
 
-    m_ui->treeMatchingArticles->expandItem(treeFeedItem);
-    m_ui->treeMatchingArticles->sortItems(0, Qt::AscendingOrder);
-    m_ui->treeMatchingArticles->setSortingEnabled(true);
+    m_ui->matchingArticlesTree->expandItem(treeFeedItem);
+    m_ui->matchingArticlesTree->sortItems(0, Qt::AscendingOrder);
+    m_ui->matchingArticlesTree->setSortingEnabled(true);
 }
 
 void AutomatedRssDownloader::updateFieldsToolTips(bool regex)
@@ -712,7 +692,7 @@ void AutomatedRssDownloader::updateFieldsToolTips(bool regex)
     tip += tr("An expression with an empty %1 clause (e.g. %2)",
               "We talk about regex/wildcards in the RSS filters section here."
               " So a valid sentence would be: An expression with an empty | clause (e.g. expr|)"
-              ).arg(u"<tt>|</tt>"_qs, u"<tt>expr|</tt>"_qs);
+              ).arg(u"<tt>|</tt>"_s, u"<tt>expr|</tt>"_s);
     m_ui->lineContains->setToolTip(tip + tr(" will match all articles.") + u"</p>");
     m_ui->lineNotContains->setToolTip(tip + tr(" will exclude all articles.") + u"</p>");
 }
@@ -758,8 +738,8 @@ void AutomatedRssDownloader::updateMustLineValidity()
     }
     else
     {
-        m_ui->lineContains->setStyleSheet(u"QLineEdit { color: #ff0000; }"_qs);
-        m_ui->labelMustStat->setPixmap(UIThemeManager::instance()->getIcon(u"dialog-warning"_qs, u"task-attention"_qs).pixmap(16, 16));
+        m_ui->lineContains->setStyleSheet(u"QLineEdit { color: #ff0000; }"_s);
+        m_ui->labelMustStat->setPixmap(UIThemeManager::instance()->getIcon(u"dialog-warning"_s, u"task-attention"_s).pixmap(16, 16));
         m_ui->labelMustStat->setToolTip(error);
     }
 }
@@ -805,8 +785,8 @@ void AutomatedRssDownloader::updateMustNotLineValidity()
     }
     else
     {
-        m_ui->lineNotContains->setStyleSheet(u"QLineEdit { color: #ff0000; }"_qs);
-        m_ui->labelMustNotStat->setPixmap(UIThemeManager::instance()->getIcon(u"dialog-warning"_qs, u"task-attention"_qs).pixmap(16, 16));
+        m_ui->lineNotContains->setStyleSheet(u"QLineEdit { color: #ff0000; }"_s);
+        m_ui->labelMustNotStat->setPixmap(UIThemeManager::instance()->getIcon(u"dialog-warning"_s, u"task-attention"_s).pixmap(16, 16));
         m_ui->labelMustNotStat->setToolTip(error);
     }
 }
@@ -823,8 +803,8 @@ void AutomatedRssDownloader::updateEpisodeFilterValidity()
     }
     else
     {
-        m_ui->lineEFilter->setStyleSheet(u"QLineEdit { color: #ff0000; }"_qs);
-        m_ui->labelEpFilterStat->setPixmap(UIThemeManager::instance()->getIcon(u"dialog-warning"_qs, u"task-attention"_qs).pixmap(16, 16));
+        m_ui->lineEFilter->setStyleSheet(u"QLineEdit { color: #ff0000; }"_s);
+        m_ui->labelEpFilterStat->setPixmap(UIThemeManager::instance()->getIcon(u"dialog-warning"_s, u"task-attention"_s).pixmap(16, 16));
     }
 }
 
@@ -841,7 +821,7 @@ void AutomatedRssDownloader::handleRuleAdded(const QString &ruleName)
 
 void AutomatedRssDownloader::handleRuleRenamed(const QString &ruleName, const QString &oldRuleName)
 {
-    auto item = m_itemsByRuleName.take(oldRuleName);
+    auto *item = m_itemsByRuleName.take(oldRuleName);
     m_itemsByRuleName.insert(ruleName, item);
     if (m_currentRule.name() == oldRuleName)
         m_currentRule.setName(ruleName);
@@ -850,7 +830,7 @@ void AutomatedRssDownloader::handleRuleRenamed(const QString &ruleName, const QS
 
 void AutomatedRssDownloader::handleRuleChanged(const QString &ruleName)
 {
-    auto item = m_itemsByRuleName.value(ruleName);
+    auto *item = m_itemsByRuleName.value(ruleName);
     if (item && (item != m_currentRuleItem))
         item->setCheckState(RSS::AutoDownloader::instance()->ruleByName(ruleName).isEnabled() ? Qt::Checked : Qt::Unchecked);
 }
