@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2015  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2015-2024  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2010  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -70,8 +70,8 @@ namespace
             {BitTorrent::TorrentState::CheckingDownloading, u"TransferList.CheckingDownloading"_s},
             {BitTorrent::TorrentState::CheckingUploading, u"TransferList.CheckingUploading"_s},
             {BitTorrent::TorrentState::CheckingResumeData, u"TransferList.CheckingResumeData"_s},
-            {BitTorrent::TorrentState::PausedDownloading, u"TransferList.PausedDownloading"_s},
-            {BitTorrent::TorrentState::PausedUploading, u"TransferList.PausedUploading"_s},
+            {BitTorrent::TorrentState::StoppedDownloading, u"TransferList.StoppedDownloading"_s},
+            {BitTorrent::TorrentState::StoppedUploading, u"TransferList.StoppedUploading"_s},
             {BitTorrent::TorrentState::Moving, u"TransferList.Moving"_s},
             {BitTorrent::TorrentState::MissingFiles, u"TransferList.MissingFiles"_s},
             {BitTorrent::TorrentState::Error, u"TransferList.Error"_s}
@@ -106,27 +106,23 @@ TransferListModel::TransferListModel(QObject *parent)
           {BitTorrent::TorrentState::CheckingDownloading, tr("Checking", "Torrent local data is being checked")},
           {BitTorrent::TorrentState::CheckingUploading, tr("Checking", "Torrent local data is being checked")},
           {BitTorrent::TorrentState::CheckingResumeData, tr("Checking resume data", "Used when loading the torrents from disk after qbt is launched. It checks the correctness of the .fastresume file. Normally it is completed in a fraction of a second, unless loading many many torrents.")},
-          {BitTorrent::TorrentState::PausedDownloading, tr("Paused")},
-          {BitTorrent::TorrentState::PausedUploading, tr("Completed")},
+          {BitTorrent::TorrentState::StoppedDownloading, tr("Stopped")},
+          {BitTorrent::TorrentState::StoppedUploading, tr("Completed")},
           {BitTorrent::TorrentState::Moving, tr("Moving", "Torrent local data are being moved/relocated")},
           {BitTorrent::TorrentState::MissingFiles, tr("Missing Files")},
           {BitTorrent::TorrentState::Error, tr("Errored", "Torrent status, the torrent has an error")}
     }
-    , m_stateThemeColors {torrentStateColorsFromUITheme()}
-    , m_checkingIcon {UIThemeManager::instance()->getIcon(u"force-recheck"_s, u"checking"_s)}
-    , m_completedIcon {UIThemeManager::instance()->getIcon(u"checked-completed"_s, u"completed"_s)}
-    , m_downloadingIcon {UIThemeManager::instance()->getIcon(u"downloading"_s)}
-    , m_errorIcon {UIThemeManager::instance()->getIcon(u"error"_s)}
-    , m_movingIcon {UIThemeManager::instance()->getIcon(u"set-location"_s)}
-    , m_pausedIcon {UIThemeManager::instance()->getIcon(u"stopped"_s, u"media-playback-pause"_s)}
-    , m_queuedIcon {UIThemeManager::instance()->getIcon(u"queued"_s)}
-    , m_stalledDLIcon {UIThemeManager::instance()->getIcon(u"stalledDL"_s)}
-    , m_stalledUPIcon {UIThemeManager::instance()->getIcon(u"stalledUP"_s)}
-    , m_uploadingIcon {UIThemeManager::instance()->getIcon(u"upload"_s, u"uploading"_s)}
 {
     configure();
     connect(Preferences::instance(), &Preferences::changed, this, &TransferListModel::configure);
 
+    loadUIThemeResources();
+    connect(UIThemeManager::instance(), &UIThemeManager::themeChanged, this, [this]
+    {
+        loadUIThemeResources();
+        emit dataChanged(index(0, 0), index((rowCount() - 1), (columnCount() - 1)), {Qt::DecorationRole, Qt::ForegroundRole});
+    });
+
     // Load the torrents
     using namespace BitTorrent;
     addTorrents(Session::instance()->torrents());
@@ -138,8 +134,8 @@ TransferListModel::TransferListModel(QObject *parent)
 
     connect(Session::instance(), &Session::torrentFinished, this, &TransferListModel::handleTorrentStatusUpdated);
     connect(Session::instance(), &Session::torrentMetadataReceived, this, &TransferListModel::handleTorrentStatusUpdated);
-    connect(Session::instance(), &Session::torrentResumed, this, &TransferListModel::handleTorrentStatusUpdated);
-    connect(Session::instance(), &Session::torrentPaused, this, &TransferListModel::handleTorrentStatusUpdated);
+    connect(Session::instance(), &Session::torrentStarted, this, &TransferListModel::handleTorrentStatusUpdated);
+    connect(Session::instance(), &Session::torrentStopped, this, &TransferListModel::handleTorrentStatusUpdated);
     connect(Session::instance(), &Session::torrentFinishedChecking, this, &TransferListModel::handleTorrentStatusUpdated);
 }
 
@@ -165,12 +161,13 @@ QVariant TransferListModel::headerData(const int section, const Qt::Orientation
             case TR_NAME: return tr("Name", "i.e: torrent name");
             case TR_SIZE: return tr("Size", "i.e: torrent size");
             case TR_PROGRESS: return tr("Progress", "% Done");
-            case TR_STATUS: return tr("Status", "Torrent status (e.g. downloading, seeding, paused)");
+            case TR_STATUS: return tr("Status", "Torrent status (e.g. downloading, seeding, stopped)");
             case TR_SEEDS: return tr("Seeds", "i.e. full sources (often untranslated)");
             case TR_PEERS: return tr("Peers", "i.e. partial sources (often untranslated)");
             case TR_DLSPEED: return tr("Down Speed", "i.e: Download speed");
             case TR_UPSPEED: return tr("Up Speed", "i.e: Upload speed");
             case TR_RATIO: return tr("Ratio", "Share ratio");
+            case TR_POPULARITY: return tr("Popularity");
             case TR_ETA: return tr("ETA", "i.e: Estimated Time of Arrival / Time left");
             case TR_CATEGORY: return tr("Category");
             case TR_TAGS: return tr("Tags");
@@ -184,7 +181,7 @@ QVariant TransferListModel::headerData(const int section, const Qt::Orientation
             case TR_AMOUNT_DOWNLOADED_SESSION: return tr("Session Download", "Amount of data downloaded since program open (e.g. in MB)");
             case TR_AMOUNT_UPLOADED_SESSION: return tr("Session Upload", "Amount of data uploaded since program open (e.g. in MB)");
             case TR_AMOUNT_LEFT: return tr("Remaining", "Amount of data left to download (e.g. in MB)");
-            case TR_TIME_ELAPSED: return tr("Time Active", "Time (duration) the torrent is active (not paused)");
+            case TR_TIME_ELAPSED: return tr("Time Active", "Time (duration) the torrent is active (not stopped)");
             case TR_SAVE_PATH: return tr("Save Path", "Torrent save path");
             case TR_DOWNLOAD_PATH: return tr("Incomplete Save Path", "Torrent incomplete save path");
             case TR_COMPLETED: return tr("Completed", "Amount of data completed (e.g. in MB)");
@@ -195,6 +192,16 @@ QVariant TransferListModel::headerData(const int section, const Qt::Orientation
             case TR_AVAILABILITY: return tr("Availability", "The number of distributed copies of the torrent");
             case TR_INFOHASH_V1: return tr("Info Hash v1", "i.e: torrent info hash v1");
             case TR_INFOHASH_V2: return tr("Info Hash v2", "i.e: torrent info hash v2");
+            case TR_REANNOUNCE: return tr("Reannounce In", "Indicates the time until next trackers reannounce");
+            case TR_PRIVATE: return tr("Private", "Flags private torrents");
+            default: return {};
+            }
+        }
+        else if (role == Qt::ToolTipRole)
+        {
+            switch (section)
+            {
+            case TR_POPULARITY: return tr("Ratio / Time Active (in months), indicates how popular the torrent is");
             default: return {};
             }
         }
@@ -219,9 +226,11 @@ QVariant TransferListModel::headerData(const int section, const Qt::Orientation
             case TR_DLLIMIT:
             case TR_RATIO_LIMIT:
             case TR_RATIO:
+            case TR_POPULARITY:
             case TR_QUEUE_POSITION:
             case TR_LAST_ACTIVITY:
             case TR_AVAILABILITY:
+            case TR_REANNOUNCE:
                 return QVariant(Qt::AlignRight | Qt::AlignVCenter);
             default:
                 return QAbstractListModel::headerData(section, orientation, role);
@@ -237,8 +246,8 @@ QString TransferListModel::displayValue(const BitTorrent::Torrent *torrent, cons
     bool hideValues = false;
     if (m_hideZeroValuesMode == HideZeroValuesMode::Always)
         hideValues = true;
-    else if (m_hideZeroValuesMode == HideZeroValuesMode::Paused)
-        hideValues = (torrent->state() == BitTorrent::TorrentState::PausedDownloading);
+    else if (m_hideZeroValuesMode == HideZeroValuesMode::Stopped)
+        hideValues = (torrent->state() == BitTorrent::TorrentState::StoppedDownloading);
 
     const auto availabilityString = [hideValues](const qreal value) -> QString
     {
@@ -342,6 +351,22 @@ QString TransferListModel::displayValue(const BitTorrent::Torrent *torrent, cons
         return hash.isValid() ? hash.toString() : tr("N/A");
     };
 
+    const auto reannounceString = [hideValues](const qint64 time) -> QString
+    {
+        if (hideValues && (time == 0))
+            return {};
+        return Utils::Misc::userFriendlyDuration(time);
+    };
+
+    const auto privateString = [hideValues](const bool isPrivate, const bool hasMetadata) -> QString
+    {
+        if (hideValues && !isPrivate)
+            return {};
+        if (hasMetadata)
+            return isPrivate ? tr("Yes") : tr("No");
+        return tr("N/A");
+    };
+
     switch (column)
     {
     case TR_NAME:
@@ -368,10 +393,12 @@ QString TransferListModel::displayValue(const BitTorrent::Torrent *torrent, cons
         return ratioString(torrent->realRatio());
     case TR_RATIO_LIMIT:
         return ratioString(torrent->maxRatio());
+    case TR_POPULARITY:
+        return ratioString(torrent->popularity());
     case TR_CATEGORY:
         return torrent->category();
     case TR_TAGS:
-        return torrent->tags().join(u", "_s);
+        return Utils::String::joinIntoString(torrent->tags(), u", "_s);
     case TR_ADD_DATE:
         return QLocale().toString(torrent->addedTime().toLocalTime(), QLocale::ShortFormat);
     case TR_SEED_DATE:
@@ -412,6 +439,10 @@ QString TransferListModel::displayValue(const BitTorrent::Torrent *torrent, cons
         return hashString(torrent->infoHash().v1());
     case TR_INFOHASH_V2:
         return hashString(torrent->infoHash().v2());
+    case TR_REANNOUNCE:
+        return reannounceString(torrent->nextAnnounce());
+    case TR_PRIVATE:
+        return privateString(torrent->isPrivate(), torrent->hasMetadata());
     }
 
     return {};
@@ -443,6 +474,8 @@ QVariant TransferListModel::internalValue(const BitTorrent::Torrent *torrent, co
         return torrent->eta();
     case TR_RATIO:
         return torrent->realRatio();
+    case TR_POPULARITY:
+        return torrent->popularity();
     case TR_CATEGORY:
         return torrent->category();
     case TR_TAGS:
@@ -489,6 +522,10 @@ QVariant TransferListModel::internalValue(const BitTorrent::Torrent *torrent, co
         return QVariant::fromValue(torrent->infoHash().v1());
     case TR_INFOHASH_V2:
         return QVariant::fromValue(torrent->infoHash().v2());
+    case TR_REANNOUNCE:
+        return torrent->nextAnnounce();
+    case TR_PRIVATE:
+        return (torrent->hasMetadata() ? torrent->isPrivate() : QVariant());
     }
 
     return {};
@@ -550,9 +587,11 @@ QVariant TransferListModel::data(const QModelIndex &index, const int role) const
         case TR_DLLIMIT:
         case TR_RATIO_LIMIT:
         case TR_RATIO:
+        case TR_POPULARITY:
         case TR_QUEUE_POSITION:
         case TR_LAST_ACTIVITY:
         case TR_AVAILABILITY:
+        case TR_REANNOUNCE:
             return QVariant(Qt::AlignRight | Qt::AlignVCenter);
         }
         break;
@@ -673,7 +712,7 @@ void TransferListModel::configure()
     if (pref->getHideZeroValues())
     {
         if (pref->getHideZeroComboValues() == 1)
-            hideZeroValuesMode = HideZeroValuesMode::Paused;
+            hideZeroValuesMode = HideZeroValuesMode::Stopped;
         else
             hideZeroValuesMode = HideZeroValuesMode::Always;
     }
@@ -685,6 +724,23 @@ void TransferListModel::configure()
     }
 }
 
+void TransferListModel::loadUIThemeResources()
+{
+    m_stateThemeColors = torrentStateColorsFromUITheme();
+
+    const auto *themeManager = UIThemeManager::instance();
+    m_checkingIcon = themeManager->getIcon(u"force-recheck"_s, u"checking"_s);
+    m_completedIcon = themeManager->getIcon(u"checked-completed"_s, u"completed"_s);
+    m_downloadingIcon = themeManager->getIcon(u"downloading"_s);
+    m_errorIcon = themeManager->getIcon(u"error"_s);
+    m_movingIcon = themeManager->getIcon(u"set-location"_s);
+    m_stoppedIcon = themeManager->getIcon(u"stopped"_s, u"media-playback-pause"_s);
+    m_queuedIcon = themeManager->getIcon(u"queued"_s);
+    m_stalledDLIcon = themeManager->getIcon(u"stalledDL"_s);
+    m_stalledUPIcon = themeManager->getIcon(u"stalledUP"_s);
+    m_uploadingIcon = themeManager->getIcon(u"upload"_s, u"uploading"_s);
+}
+
 QIcon TransferListModel::getIconByState(const BitTorrent::TorrentState state) const
 {
     switch (state)
@@ -701,9 +757,9 @@ QIcon TransferListModel::getIconByState(const BitTorrent::TorrentState state) co
     case BitTorrent::TorrentState::Uploading:
     case BitTorrent::TorrentState::ForcedUploading:
         return m_uploadingIcon;
-    case BitTorrent::TorrentState::PausedDownloading:
-        return m_pausedIcon;
-    case BitTorrent::TorrentState::PausedUploading:
+    case BitTorrent::TorrentState::StoppedDownloading:
+        return m_stoppedIcon;
+    case BitTorrent::TorrentState::StoppedUploading:
         return m_completedIcon;
     case BitTorrent::TorrentState::QueuedDownloading:
     case BitTorrent::TorrentState::QueuedUploading:
