@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2015  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2015-2024  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2010  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -62,6 +62,7 @@ public:
         TR_UPSPEED,
         TR_ETA,
         TR_RATIO,
+        TR_POPULARITY,
         TR_CATEGORY,
         TR_TAGS,
         TR_ADD_DATE,
@@ -84,6 +85,8 @@ public:
         TR_DOWNLOAD_PATH,
         TR_INFOHASH_V1,
         TR_INFOHASH_V2,
+        TR_REANNOUNCE,
+        TR_PRIVATE,
 
         NB_COLUMNS
     };
@@ -113,6 +116,7 @@ private slots:
 
 private:
     void configure();
+    void loadUIThemeResources();
     QString displayValue(const BitTorrent::Torrent *torrent, int column) const;
     QVariant internalValue(const BitTorrent::Torrent *torrent, int column, bool alt) const;
     QIcon getIconByState(BitTorrent::TorrentState state) const;
@@ -121,12 +125,12 @@ private:
     QHash<BitTorrent::Torrent *, int> m_torrentMap;  // maps torrent handle to row number
     const QHash<BitTorrent::TorrentState, QString> m_statusStrings;
     // row text colors
-    const QHash<BitTorrent::TorrentState, QColor> m_stateThemeColors;
+    QHash<BitTorrent::TorrentState, QColor> m_stateThemeColors;
 
     enum class HideZeroValuesMode
     {
         Never,
-        Paused,
+        Stopped,
         Always
     };
 
@@ -138,7 +142,7 @@ private:
     QIcon m_downloadingIcon;
     QIcon m_errorIcon;
     QIcon m_movingIcon;
-    QIcon m_pausedIcon;
+    QIcon m_stoppedIcon;
     QIcon m_queuedIcon;
     QIcon m_stalledDLIcon;
     QIcon m_stalledUPIcon;
