@@ -1,5 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2023  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -63,17 +64,17 @@ public:
 
 public slots:
     void setSelectionCategory(const QString &category);
-    void addSelectionTag(const QString &tag);
-    void removeSelectionTag(const QString &tag);
+    void addSelectionTag(const Tag &tag);
+    void removeSelectionTag(const Tag &tag);
     void clearSelectionTags();
     void setSelectedTorrentsLocation();
-    void pauseAllTorrents();
-    void resumeAllTorrents();
+    void pauseSession();
+    void resumeSession();
     void startSelectedTorrents();
     void forceStartSelectedTorrents();
     void startVisibleTorrents();
-    void pauseSelectedTorrents();
-    void pauseVisibleTorrents();
+    void stopSelectedTorrents();
+    void stopVisibleTorrents();
     void softDeleteSelectedTorrents();
     void permDeleteSelectedTorrents();
     void deleteSelectedTorrents(bool deleteLocalFiles);
@@ -86,6 +87,7 @@ public slots:
     void copySelectedNames() const;
     void copySelectedInfohashes(CopyInfohashPolicy policy) const;
     void copySelectedIDs() const;
+    void copySelectedComments() const;
     void openSelectedTorrentsFolder() const;
     void recheckSelectedTorrents();
     void reannounceSelectedTorrents();
@@ -93,9 +95,9 @@ public slots:
     void previewSelectedTorrents();
     void hideQueuePosColumn(bool hide);
     void applyFilter(const QString &name, const TransferListModel::Column &type);
-    void applyStatusFilter(int f);
+    void applyStatusFilter(int filterIndex);
     void applyCategoryFilter(const QString &category);
-    void applyTagFilter(const QString &tag);
+    void applyTagFilter(const std::optional<Tag> &tag);
     void applyTrackerFilterAll();
     void applyTrackerFilter(const QSet<BitTorrent::TorrentID> &torrentIDs);
     void previewFile(const Path &filePath);
@@ -127,7 +129,7 @@ private:
     void editTorrentTrackers();
     void exportTorrent();
     void confirmRemoveAllTagsForSelection();
-    QStringList askTagsForSelection(const QString &dialogTitle);
+    TagSet askTagsForSelection(const QString &dialogTitle);
     void applyToSelectedTorrents(const std::function<void (BitTorrent::Torrent *const)> &fn);
     QVector<BitTorrent::Torrent *> getVisibleTorrents() const;
     int visibleColumnsCount() const;
