@@ -1,5 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2024  Jonathan Ketchker
  * Copyright (C) 2018  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2006-2012  Christophe Dumez <chris@qbittorrent.org>
  * Copyright (C) 2006-2012  Ishan Arora <ishan@qbittorrent.org>
@@ -35,6 +36,8 @@
 
 #include <QCoreApplication>
 #include <QDebug>
+#include <QDir>
+#include <QDirIterator>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
@@ -61,6 +64,7 @@
 #include "base/utils/password.h"
 #include "base/utils/string.h"
 #include "base/version.h"
+#include "apierror.h"
 #include "../webapplication.h"
 
 using namespace std::chrono_literals;
@@ -77,6 +81,17 @@ void AppController::versionAction()
 
 void AppController::buildInfoAction()
 {
+    const QString platformName =
+#ifdef Q_OS_LINUX
+        u"linux"_s;
+#elif defined(Q_OS_MACOS)
+        u"macos"_s;
+#elif defined(Q_OS_WIN)
+        u"windows"_s;
+#else
+        u"unknown"_s;
+#endif
+
     const QJsonObject versions =
     {
         {u"qt"_s, QStringLiteral(QT_VERSION_STR)},
@@ -84,7 +99,8 @@ void AppController::buildInfoAction()
         {u"boost"_s, Utils::Misc::boostVersionString()},
         {u"openssl"_s, Utils::Misc::opensslVersionString()},
         {u"zlib"_s, Utils::Misc::zlibVersionString()},
-        {u"bitness"_s, (QT_POINTER_SIZE * 8)}
+        {u"bitness"_s, (QT_POINTER_SIZE * 8)},
+        {u"platform"_s, platformName}
     };
     setResult(versions);
 }
@@ -119,17 +135,20 @@ void AppController::preferencesAction()
     data[u"file_log_delete_old"_s] = app()->isFileLoggerDeleteOld();
     data[u"file_log_age"_s] = app()->fileLoggerAge();
     data[u"file_log_age_type"_s] = app()->fileLoggerAgeType();
+    // Delete torrent contents files on torrent removal
+    data[u"delete_torrent_content_files"_s] = pref->removeTorrentContent();
 
     // Downloads
     // When adding a torrent
     data[u"torrent_content_layout"_s] = Utils::String::fromEnum(session->torrentContentLayout());
     data[u"add_to_top_of_queue"_s] = session->isAddTorrentToQueueTop();
-    data[u"start_paused_enabled"_s] = session->isAddTorrentPaused();
+    data[u"add_stopped_enabled"_s] = session->isAddTorrentStopped();
     data[u"torrent_stop_condition"_s] = Utils::String::fromEnum(session->torrentStopCondition());
     data[u"merge_trackers"_s] = session->isMergeTrackersEnabled();
     data[u"auto_delete_mode"_s] = static_cast<int>(TorrentFileGuard::autoDeleteMode());
     data[u"preallocate_all"_s] = session->isPreallocationEnabled();
     data[u"incomplete_files_ext"_s] = session->isAppendExtensionEnabled();
+    data[u"use_unwanted_folder"_s] = session->isUnwantedFolderEnabled();
     // Saving Management
     data[u"auto_tmm_enabled"_s] = !session->isAutoTMMDisabledByDefault();
     data[u"torrent_changed_tmm_enabled"_s] = !session->isDisableAutoTMMWhenCategoryChanged();
@@ -185,6 +204,8 @@ void AppController::preferencesAction()
     // Connection
     // Listening Port
     data[u"listen_port"_s] = session->port();
+    data[u"ssl_enabled"_s] = session->isSSLEnabled();
+    data[u"ssl_listen_port"_s] = session->sslPort();
     data[u"random_port"_s] = (session->port() == 0);  // deprecated
     data[u"upnp"_s] = Net::PortForwarder::instance()->isEnabled();
     // Connections Limits
@@ -270,7 +291,7 @@ void AppController::preferencesAction()
     data[u"max_seeding_time"_s] = session->globalMaxSeedingMinutes();
     data[u"max_inactive_seeding_time_enabled"_s] = (session->globalMaxInactiveSeedingMinutes() >= 0.);
     data[u"max_inactive_seeding_time"_s] = session->globalMaxInactiveSeedingMinutes();
-    data[u"max_ratio_act"_s] = session->maxRatioAction();
+    data[u"max_ratio_act"_s] = static_cast<int>(session->shareLimitAction());
     // Add trackers
     data[u"add_trackers_enabled"_s] = session->isAddTrackersEnabled();
     data[u"add_trackers"_s] = session->additionalTrackers();
@@ -318,6 +339,7 @@ void AppController::preferencesAction()
 
     // RSS settings
     data[u"rss_refresh_interval"_s] = RSS::Session::instance()->refreshInterval();
+    data[u"rss_fetch_delay"_s] = static_cast<qlonglong>(RSS::Session::instance()->fetchDelay().count());
     data[u"rss_max_articles_per_feed"_s] = RSS::Session::instance()->maxArticlesPerFeed();
     data[u"rss_processing_enabled"_s] = RSS::Session::instance()->isProcessingEnabled();
     data[u"rss_auto_downloading_enabled"_s] = RSS::AutoDownloader::instance()->isProcessingEnabled();
@@ -328,6 +350,8 @@ void AppController::preferencesAction()
     // qBitorrent preferences
     // Resume data storage type
     data[u"resume_data_storage_type"_s] = Utils::String::fromEnum(session->resumeDataStorageType());
+    // Torrent content removing mode
+    data[u"torrent_content_remove_option"_s] = Utils::String::fromEnum(session->torrentContentRemoveOption());
     // Physical memory (RAM) usage limit
     data[u"memory_working_set_limit"_s] = app()->memoryWorkingSetLimit();
     // Current network interface
@@ -342,12 +366,24 @@ void AppController::preferencesAction()
     data[u"torrent_file_size_limit"_s] = pref->getTorrentFileSizeLimit();
     // Recheck completed torrents
     data[u"recheck_completed_torrents"_s] = pref->recheckTorrentsOnCompletion();
+    // Customize application instance name
+    data[u"app_instance_name"_s] = app()->instanceName();
     // Refresh interval
     data[u"refresh_interval"_s] = session->refreshInterval();
     // Resolve peer countries
     data[u"resolve_peer_countries"_s] = pref->resolvePeerCountries();
     // Reannounce to all trackers when ip/port changed
     data[u"reannounce_when_address_changed"_s] = session->isReannounceWhenAddressChangedEnabled();
+    // Embedded tracker
+    data[u"enable_embedded_tracker"_s] = session->isTrackerEnabled();
+    data[u"embedded_tracker_port"_s] = pref->getTrackerPort();
+    data[u"embedded_tracker_port_forwarding"_s] = pref->isTrackerPortForwardingEnabled();
+    // Mark-of-the-Web
+    data[u"mark_of_the_web"_s] = pref->isMarkOfTheWebEnabled();
+    // Ignore SSL errors
+    data[u"ignore_ssl_errors"_s] = pref->isIgnoreSSLErrors();
+    // Python executable path
+    data[u"python_executable_path"_s] = pref->getPythonExecutablePath().toString();
 
     // libtorrent preferences
     // Bdecode depth limit
@@ -410,10 +446,6 @@ void AppController::preferencesAction()
     data[u"ssrf_mitigation"_s] = session->isSSRFMitigationEnabled();
     // Disallow connection to peers on privileged ports
     data[u"block_peers_on_privileged_ports"_s] = session->blockPeersOnPrivilegedPorts();
-    // Embedded tracker
-    data[u"enable_embedded_tracker"_s] = session->isTrackerEnabled();
-    data[u"embedded_tracker_port"_s] = pref->getTrackerPort();
-    data[u"embedded_tracker_port_forwarding"_s] = pref->isTrackerPortForwardingEnabled();
     // Choking algorithm
     data[u"upload_slots_behavior"_s] = static_cast<int>(session->chokingAlgorithm());
     // Seed choking algorithm
@@ -430,6 +462,8 @@ void AppController::preferencesAction()
     data[u"peer_turnover_interval"_s] = session->peerTurnoverInterval();
     // Maximum outstanding requests to a single peer
     data[u"request_queue_size"_s] = session->requestQueueSize();
+    // DHT bootstrap nodes
+    data[u"dht_bootstrap_nodes"_s] = session->getDHTBootstrapNodes();
 
     setResult(data);
 }
@@ -487,6 +521,9 @@ void AppController::setPreferencesAction()
         app()->setFileLoggerAge(it.value().toInt());
     if (hasKey(u"file_log_age_type"_s))
         app()->setFileLoggerAgeType(it.value().toInt());
+    // Delete torrent content files on torrent removal
+    if (hasKey(u"delete_torrent_content_files"_s))
+        pref->setRemoveTorrentContent(it.value().toBool());
 
     // Downloads
     // When adding a torrent
@@ -494,8 +531,8 @@ void AppController::setPreferencesAction()
         session->setTorrentContentLayout(Utils::String::toEnum(it.value().toString(), BitTorrent::TorrentContentLayout::Original));
     if (hasKey(u"add_to_top_of_queue"_s))
         session->setAddTorrentToQueueTop(it.value().toBool());
-    if (hasKey(u"start_paused_enabled"_s))
-        session->setAddTorrentPaused(it.value().toBool());
+    if (hasKey(u"add_stopped_enabled"_s))
+        session->setAddTorrentStopped(it.value().toBool());
     if (hasKey(u"torrent_stop_condition"_s))
         session->setTorrentStopCondition(Utils::String::toEnum(it.value().toString(), BitTorrent::Torrent::StopCondition::None));
     if (hasKey(u"merge_trackers"_s))
@@ -507,6 +544,8 @@ void AppController::setPreferencesAction()
         session->setPreallocationEnabled(it.value().toBool());
     if (hasKey(u"incomplete_files_ext"_s))
         session->setAppendExtensionEnabled(it.value().toBool());
+    if (hasKey(u"use_unwanted_folder"_s))
+        session->setUnwantedFolderEnabled(it.value().toBool());
 
     // Saving Management
     if (hasKey(u"auto_tmm_enabled"_s))
@@ -626,6 +665,11 @@ void AppController::setPreferencesAction()
     {
         session->setPort(it.value().toInt());
     }
+    // SSL Torrents
+    if (hasKey(u"ssl_enabled"_s))
+        session->setSSLEnabled(it.value().toBool());
+    if (hasKey(u"ssl_listen_port"_s))
+        session->setSSLPort(it.value().toInt());
     if (hasKey(u"upnp"_s))
         Net::PortForwarder::instance()->setEnabled(it.value().toBool());
     // Connections Limits
@@ -775,7 +819,24 @@ void AppController::setPreferencesAction()
             ? m[u"max_inactive_seeding_time"_s].toInt() : -1);
     }
     if (hasKey(u"max_ratio_act"_s))
-        session->setMaxRatioAction(static_cast<MaxRatioAction>(it.value().toInt()));
+    {
+        switch (it.value().toInt())
+        {
+        default:
+        case 0:
+            session->setShareLimitAction(BitTorrent::ShareLimitAction::Stop);
+            break;
+        case 1:
+            session->setShareLimitAction(BitTorrent::ShareLimitAction::Remove);
+            break;
+        case 2:
+            session->setShareLimitAction(BitTorrent::ShareLimitAction::EnableSuperSeeding);
+            break;
+        case 3:
+            session->setShareLimitAction(BitTorrent::ShareLimitAction::RemoveWithContent);
+            break;
+        }
+    }
     // Add trackers
     if (hasKey(u"add_trackers_enabled"_s))
         session->setAddTrackersEnabled(it.value().toBool());
@@ -856,6 +917,8 @@ void AppController::setPreferencesAction()
 
     if (hasKey(u"rss_refresh_interval"_s))
         RSS::Session::instance()->setRefreshInterval(it.value().toInt());
+    if (hasKey(u"rss_fetch_delay"_s))
+        RSS::Session::instance()->setFetchDelay(std::chrono::seconds(it.value().toLongLong()));
     if (hasKey(u"rss_max_articles_per_feed"_s))
         RSS::Session::instance()->setMaxArticlesPerFeed(it.value().toInt());
     if (hasKey(u"rss_processing_enabled"_s))
@@ -872,6 +935,9 @@ void AppController::setPreferencesAction()
     // Resume data storage type
     if (hasKey(u"resume_data_storage_type"_s))
         session->setResumeDataStorageType(Utils::String::toEnum(it.value().toString(), BitTorrent::ResumeDataStorageType::Legacy));
+    // Torrent content removing mode
+    if (hasKey(u"torrent_content_remove_option"_s))
+        session->setTorrentContentRemoveOption(Utils::String::toEnum(it.value().toString(), BitTorrent::TorrentContentRemoveOption::MoveToTrash));
     // Physical memory (RAM) usage limit
     if (hasKey(u"memory_working_set_limit"_s))
         app()->setMemoryWorkingSetLimit(it.value().toInt());
@@ -906,6 +972,9 @@ void AppController::setPreferencesAction()
     // Recheck completed torrents
     if (hasKey(u"recheck_completed_torrents"_s))
         pref->recheckTorrentsOnCompletion(it.value().toBool());
+    // Customize application instance name
+    if (hasKey(u"app_instance_name"_s))
+        app()->setInstanceName(it.value().toString());
     // Refresh interval
     if (hasKey(u"refresh_interval"_s))
         session->setRefreshInterval(it.value().toInt());
@@ -915,6 +984,22 @@ void AppController::setPreferencesAction()
     // Reannounce to all trackers when ip/port changed
     if (hasKey(u"reannounce_when_address_changed"_s))
         session->setReannounceWhenAddressChangedEnabled(it.value().toBool());
+    // Embedded tracker
+    if (hasKey(u"embedded_tracker_port"_s))
+        pref->setTrackerPort(it.value().toInt());
+    if (hasKey(u"embedded_tracker_port_forwarding"_s))
+        pref->setTrackerPortForwardingEnabled(it.value().toBool());
+    if (hasKey(u"enable_embedded_tracker"_s))
+        session->setTrackerEnabled(it.value().toBool());
+    // Mark-of-the-Web
+    if (hasKey(u"mark_of_the_web"_s))
+        pref->setMarkOfTheWebEnabled(it.value().toBool());
+    // Ignore SLL errors
+    if (hasKey(u"ignore_ssl_errors"_s))
+        pref->setIgnoreSSLErrors(it.value().toBool());
+    // Python executable path
+    if (hasKey(u"python_executable_path"_s))
+        pref->setPythonExecutablePath(Path(it.value().toString()));
 
     // libtorrent preferences
     // Bdecode depth limit
@@ -1009,13 +1094,6 @@ void AppController::setPreferencesAction()
     // Disallow connection to peers on privileged ports
     if (hasKey(u"block_peers_on_privileged_ports"_s))
         session->setBlockPeersOnPrivilegedPorts(it.value().toBool());
-    // Embedded tracker
-    if (hasKey(u"embedded_tracker_port"_s))
-        pref->setTrackerPort(it.value().toInt());
-    if (hasKey(u"embedded_tracker_port_forwarding"_s))
-        pref->setTrackerPortForwardingEnabled(it.value().toBool());
-    if (hasKey(u"enable_embedded_tracker"_s))
-        session->setTrackerEnabled(it.value().toBool());
     // Choking algorithm
     if (hasKey(u"upload_slots_behavior"_s))
         session->setChokingAlgorithm(static_cast<BitTorrent::ChokingAlgorithm>(it.value().toInt()));
@@ -1046,6 +1124,9 @@ void AppController::setPreferencesAction()
     // Maximum outstanding requests to a single peer
     if (hasKey(u"request_queue_size"_s))
         session->setRequestQueueSize(it.value().toInt());
+    // DHT bootstrap nodes
+    if (hasKey(u"dht_bootstrap_nodes"_s))
+        session->setDHTBootstrapNodes(it.value().toString());
 
     // Save preferences
     pref->apply();
@@ -1056,6 +1137,46 @@ void AppController::defaultSavePathAction()
     setResult(BitTorrent::Session::instance()->savePath().toString());
 }
 
+void AppController::sendTestEmailAction()
+{
+    app()->sendTestEmail();
+}
+
+
+void AppController::getDirectoryContentAction()
+{
+    requireParams({u"dirPath"_s});
+
+    const QString dirPath = params().value(u"dirPath"_s);
+    if (dirPath.isEmpty() || dirPath.startsWith(u':'))
+        throw APIError(APIErrorType::BadParams, tr("Invalid directory path"));
+
+    const QDir dir {dirPath};
+    if (!dir.isAbsolute())
+        throw APIError(APIErrorType::BadParams, tr("Invalid directory path"));
+    if (!dir.exists())
+        throw APIError(APIErrorType::NotFound, tr("Directory does not exist"));
+
+    const QString visibility = params().value(u"mode"_s, u"all"_s);
+
+    const auto parseDirectoryContentMode = [](const QString &visibility) -> QDir::Filters
+    {
+        if (visibility == u"dirs")
+            return QDir::Dirs;
+        if (visibility == u"files")
+            return QDir::Files;
+        if (visibility == u"all")
+            return (QDir::Dirs | QDir::Files);
+        throw APIError(APIErrorType::BadParams, tr("Invalid mode, allowed values: %1").arg(u"all, dirs, files"_s));
+    };
+
+    QJsonArray ret;
+    QDirIterator it {dirPath, (QDir::NoDotAndDotDot | parseDirectoryContentMode(visibility))};
+    while (it.hasNext())
+        ret.append(it.next());
+    setResult(ret);
+}
+
 void AppController::networkInterfaceListAction()
 {
     QJsonArray ifaceList;
