@@ -47,8 +47,8 @@ private slots:
     void filesAction();
     void pieceHashesAction();
     void pieceStatesAction();
-    void resumeAction();
-    void pauseAction();
+    void startAction();
+    void stopAction();
     void recheckAction();
     void reannounceAction();
     void renameAction();
@@ -89,4 +89,6 @@ private slots:
     void renameFileAction();
     void renameFolderAction();
     void exportAction();
+    void SSLParametersAction();
+    void setSSLParametersAction();
 };
