@@ -26,11 +26,10 @@
  * exception statement from your version.
  */
 
-'use strict';
+"use strict";
 
-if (window.qBittorrent === undefined) {
+if (window.qBittorrent === undefined)
     window.qBittorrent = {};
-}
 
 window.qBittorrent.FileTree = (function() {
     const exports = function() {
@@ -77,13 +76,12 @@ window.qBittorrent.FileTree = (function() {
 
         generateNodeMap: function(node) {
             // don't store root node in map
-            if (node.root !== null) {
+            if (node.root !== null)
                 this.nodeMap[node.rowId] = node;
-            }
 
-            node.children.each(function(child) {
+            node.children.each((child) => {
                 this.generateNodeMap(child);
-            }.bind(this));
+            });
         },
 
         getNode: function(rowId) {
@@ -101,17 +99,17 @@ window.qBittorrent.FileTree = (function() {
          */
         toArray: function() {
             const nodes = [];
-            this.root.children.each(function(node) {
+            this.root.children.each((node) => {
                 this._getArrayOfNodes(node, nodes);
-            }.bind(this));
+            });
             return nodes;
         },
 
         _getArrayOfNodes: function(node, array) {
             array.push(node);
-            node.children.each(function(child) {
+            node.children.each((child) => {
                 this._getArrayOfNodes(child, array);
-            }.bind(this));
+            });
         }
     });
 
@@ -135,6 +133,11 @@ window.qBittorrent.FileTree = (function() {
     const FolderNode = new Class({
         Extends: FileNode,
 
+        /**
+         * Will automatically tick the checkbox for a folder if all subfolders and files are also ticked
+         */
+        autoCheckFolders: true,
+
         initialize: function() {
             this.isFolder = true;
         },
@@ -156,7 +159,7 @@ window.qBittorrent.FileTree = (function() {
 
             let isFirstFile = true;
 
-            this.children.each(function(node) {
+            this.children.each((node) => {
                 if (node.isFolder)
                     node.calculateSize();
 
@@ -180,11 +183,11 @@ window.qBittorrent.FileTree = (function() {
                     progress += (node.progress * node.size);
                     availability += (node.availability * node.size);
                 }
-            }.bind(this));
+            });
 
             this.size = size;
             this.remaining = remaining;
-            this.checked = checked;
+            this.checked = this.autoCheckFolders ? checked : TriState.Checked;
             this.progress = (progress / size);
             this.priority = priority;
             this.availability = (availability / size);
