@@ -26,11 +26,10 @@
  * exception statement from your version.
  */
 
-'use strict';
+"use strict";
 
-if (window.qBittorrent === undefined) {
+if (window.qBittorrent === undefined)
     window.qBittorrent = {};
-}
 
 window.qBittorrent.PropFiles = (function() {
     const exports = function() {
@@ -82,7 +81,7 @@ window.qBittorrent.PropFiles = (function() {
 
         const getChildFiles = function(node) {
             if (node.isFolder) {
-                node.children.each(function(child) {
+                node.children.each((child) => {
                     getChildFiles(child);
                 });
             }
@@ -92,7 +91,7 @@ window.qBittorrent.PropFiles = (function() {
             }
         };
 
-        node.children.each(function(child) {
+        node.children.each((child) => {
             getChildFiles(child);
         });
 
@@ -107,8 +106,8 @@ window.qBittorrent.PropFiles = (function() {
 
         const checkbox = e.target;
         const priority = checkbox.checked ? FilePriority.Normal : FilePriority.Ignored;
-        const id = checkbox.get('data-id');
-        const fileId = checkbox.get('data-file-id');
+        const id = checkbox.get("data-id");
+        const fileId = checkbox.get("data-file-id");
 
         const rows = getAllChildren(id, fileId);
 
@@ -119,8 +118,8 @@ window.qBittorrent.PropFiles = (function() {
     const fileComboboxChanged = function(e) {
         const combobox = e.target;
         const priority = combobox.value;
-        const id = combobox.get('data-id');
-        const fileId = combobox.get('data-file-id');
+        const id = combobox.get("data-id");
+        const fileId = combobox.get("data-file-id");
 
         const rows = getAllChildren(id, fileId);
 
@@ -129,24 +128,24 @@ window.qBittorrent.PropFiles = (function() {
     };
 
     const isDownloadCheckboxExists = function(id) {
-        return ($('cbPrio' + id) !== null);
+        return ($("cbPrio" + id) !== null);
     };
 
     const createDownloadCheckbox = function(id, fileId, checked) {
-        const checkbox = new Element('input');
-        checkbox.set('type', 'checkbox');
-        checkbox.set('id', 'cbPrio' + id);
-        checkbox.set('data-id', id);
-        checkbox.set('data-file-id', fileId);
-        checkbox.set('class', 'DownloadedCB');
-        checkbox.addEvent('click', fileCheckboxClicked);
+        const checkbox = new Element("input");
+        checkbox.set("type", "checkbox");
+        checkbox.set("id", "cbPrio" + id);
+        checkbox.set("data-id", id);
+        checkbox.set("data-file-id", fileId);
+        checkbox.set("class", "DownloadedCB");
+        checkbox.addEvent("click", fileCheckboxClicked);
 
         updateCheckbox(checkbox, checked);
         return checkbox;
     };
 
     const updateDownloadCheckbox = function(id, checked) {
-        const checkbox = $('cbPrio' + id);
+        const checkbox = $("cbPrio" + id);
         updateCheckbox(checkbox, checked);
     };
 
@@ -165,58 +164,53 @@ window.qBittorrent.PropFiles = (function() {
     };
 
     const isPriorityComboExists = function(id) {
-        return ($('comboPrio' + id) !== null);
+        return ($("comboPrio" + id) !== null);
     };
 
     const createPriorityOptionElement = function(priority, selected, html) {
-        const elem = new Element('option');
-        elem.set('value', priority.toString());
-        elem.set('html', html);
+        const elem = new Element("option");
+        elem.set("value", priority.toString());
+        elem.set("html", html);
         if (selected)
-            elem.setAttribute('selected', '');
+            elem.selected = true;
         return elem;
     };
 
     const createPriorityCombo = function(id, fileId, selectedPriority) {
-        const select = new Element('select');
-        select.set('id', 'comboPrio' + id);
-        select.set('data-id', id);
-        select.set('data-file-id', fileId);
-        select.set('disabled', is_seed);
-        select.addClass('combo_priority');
-        select.addEvent('change', fileComboboxChanged);
+        const select = new Element("select");
+        select.set("id", "comboPrio" + id);
+        select.set("data-id", id);
+        select.set("data-file-id", fileId);
+        select.addClass("combo_priority");
+        select.addEvent("change", fileComboboxChanged);
 
-        createPriorityOptionElement(FilePriority.Ignored, (FilePriority.Ignored === selectedPriority), 'QBT_TR(Do not download)QBT_TR[CONTEXT=PropListDelegate]').injectInside(select);
-        createPriorityOptionElement(FilePriority.Normal, (FilePriority.Normal === selectedPriority), 'QBT_TR(Normal)QBT_TR[CONTEXT=PropListDelegate]').injectInside(select);
-        createPriorityOptionElement(FilePriority.High, (FilePriority.High === selectedPriority), 'QBT_TR(High)QBT_TR[CONTEXT=PropListDelegate]').injectInside(select);
-        createPriorityOptionElement(FilePriority.Maximum, (FilePriority.Maximum === selectedPriority), 'QBT_TR(Maximum)QBT_TR[CONTEXT=PropListDelegate]').injectInside(select);
+        createPriorityOptionElement(FilePriority.Ignored, (FilePriority.Ignored === selectedPriority), "QBT_TR(Do not download)QBT_TR[CONTEXT=PropListDelegate]").injectInside(select);
+        createPriorityOptionElement(FilePriority.Normal, (FilePriority.Normal === selectedPriority), "QBT_TR(Normal)QBT_TR[CONTEXT=PropListDelegate]").injectInside(select);
+        createPriorityOptionElement(FilePriority.High, (FilePriority.High === selectedPriority), "QBT_TR(High)QBT_TR[CONTEXT=PropListDelegate]").injectInside(select);
+        createPriorityOptionElement(FilePriority.Maximum, (FilePriority.Maximum === selectedPriority), "QBT_TR(Maximum)QBT_TR[CONTEXT=PropListDelegate]").injectInside(select);
 
         // "Mixed" priority is for display only; it shouldn't be selectable
-        const mixedPriorityOption = createPriorityOptionElement(FilePriority.Mixed, (FilePriority.Mixed === selectedPriority), 'QBT_TR(Mixed)QBT_TR[CONTEXT=PropListDelegate]');
-        mixedPriorityOption.set('disabled', true);
+        const mixedPriorityOption = createPriorityOptionElement(FilePriority.Mixed, (FilePriority.Mixed === selectedPriority), "QBT_TR(Mixed)QBT_TR[CONTEXT=PropListDelegate]");
+        mixedPriorityOption.set("disabled", true);
         mixedPriorityOption.injectInside(select);
 
         return select;
     };
 
     const updatePriorityCombo = function(id, selectedPriority) {
-        const combobox = $('comboPrio' + id);
-
-        if (parseInt(combobox.value) !== selectedPriority)
+        const combobox = $("comboPrio" + id);
+        if (parseInt(combobox.value, 10) !== selectedPriority)
             selectComboboxPriority(combobox, selectedPriority);
-
-        if (combobox.disabled !== is_seed)
-            combobox.disabled = is_seed;
     };
 
     const selectComboboxPriority = function(combobox, priority) {
         const options = combobox.options;
         for (let i = 0; i < options.length; ++i) {
             const option = options[i];
-            if (parseInt(option.value) === priority)
-                option.setAttribute('selected', '');
+            if (parseInt(option.value, 10) === priority)
+                option.selected = true;
             else
-                option.removeAttribute('selected');
+                option.selected = false;
         }
 
         combobox.value = priority;
@@ -228,12 +222,12 @@ window.qBittorrent.PropFiles = (function() {
         const rowIds = [];
         const fileIds = [];
         let priority = FilePriority.Ignored;
-        const checkbox = $('tristate_cb');
+        const checkbox = $("tristate_cb");
 
         if (checkbox.state === "checked") {
             setCheckboxUnchecked(checkbox);
             // set file priority for all checked to Ignored
-            torrentFilesTable.getFilteredAndSortedRows().forEach(function(row) {
+            torrentFilesTable.getFilteredAndSortedRows().forEach((row) => {
                 const rowId = row.rowId;
                 const fileId = row.full_data.fileId;
                 const isChecked = (row.full_data.checked === TriState.Checked);
@@ -248,7 +242,7 @@ window.qBittorrent.PropFiles = (function() {
             setCheckboxChecked(checkbox);
             priority = FilePriority.Normal;
             // set file priority for all unchecked to Normal
-            torrentFilesTable.getFilteredAndSortedRows().forEach(function(row) {
+            torrentFilesTable.getFilteredAndSortedRows().forEach((row) => {
                 const rowId = row.rowId;
                 const fileId = row.full_data.fileId;
                 const isUnchecked = (row.full_data.checked === TriState.Unchecked);
@@ -265,7 +259,7 @@ window.qBittorrent.PropFiles = (function() {
     };
 
     const updateGlobalCheckbox = function() {
-        const checkbox = $('tristate_cb');
+        const checkbox = $("tristate_cb");
         if (isAllCheckboxesChecked())
             setCheckboxChecked(checkbox);
         else if (isAllCheckboxesUnchecked())
@@ -292,7 +286,7 @@ window.qBittorrent.PropFiles = (function() {
     };
 
     const isAllCheckboxesChecked = function() {
-        const checkboxes = $$('input.DownloadedCB');
+        const checkboxes = $$("input.DownloadedCB");
         for (let i = 0; i < checkboxes.length; ++i) {
             if (!checkboxes[i].checked)
                 return false;
@@ -301,7 +295,7 @@ window.qBittorrent.PropFiles = (function() {
     };
 
     const isAllCheckboxesUnchecked = function() {
-        const checkboxes = $$('input.DownloadedCB');
+        const checkboxes = $$("input.DownloadedCB");
         for (let i = 0; i < checkboxes.length; ++i) {
             if (checkboxes[i].checked)
                 return false;
@@ -310,16 +304,17 @@ window.qBittorrent.PropFiles = (function() {
     };
 
     const setFilePriority = function(ids, fileIds, priority) {
-        if (current_hash === "") return;
+        if (current_hash === "")
+            return;
 
         clearTimeout(loadTorrentFilesDataTimer);
         new Request({
-            url: 'api/v2/torrents/filePrio',
-            method: 'post',
+            url: "api/v2/torrents/filePrio",
+            method: "post",
             data: {
-                'hash': current_hash,
-                'id': fileIds.join('|'),
-                'priority': priority
+                "hash": current_hash,
+                "id": fileIds.join("|"),
+                "priority": priority
             },
             onComplete: function() {
                 loadTorrentFilesDataTimer = loadTorrentFilesData.delay(1000);
@@ -327,10 +322,10 @@ window.qBittorrent.PropFiles = (function() {
         }).send();
 
         const ignore = (priority === FilePriority.Ignored);
-        ids.forEach(function(_id) {
+        ids.forEach((_id) => {
             torrentFilesTable.setIgnored(_id, ignore);
 
-            const combobox = $('comboPrio' + _id);
+            const combobox = $("comboPrio" + _id);
             if (combobox !== null)
                 selectComboboxPriority(combobox, priority);
         });
@@ -340,8 +335,8 @@ window.qBittorrent.PropFiles = (function() {
 
     let loadTorrentFilesDataTimer;
     const loadTorrentFilesData = function() {
-        if ($('prop_files').hasClass('invisible')
-            || $('propertiesPanel_collapseToggle').hasClass('panel-expand')) {
+        if ($("prop_files").hasClass("invisible")
+            || $("propertiesPanel_collapseToggle").hasClass("panel-expand")) {
             // Tab changed, don't do anything
             return;
         }
@@ -353,22 +348,23 @@ window.qBittorrent.PropFiles = (function() {
             return;
         }
         let loadedNewTorrent = false;
-        if (new_hash != current_hash) {
+        if (new_hash !== current_hash) {
             torrentFilesTable.clear();
             current_hash = new_hash;
             loadedNewTorrent = true;
         }
-        const url = new URI('api/v2/torrents/files?hash=' + current_hash);
+        const url = new URI("api/v2/torrents/files?hash=" + current_hash);
         new Request.JSON({
             url: url,
+            method: "get",
             noCache: true,
-            method: 'get',
             onComplete: function() {
                 clearTimeout(loadTorrentFilesDataTimer);
                 loadTorrentFilesDataTimer = loadTorrentFilesData.delay(5000);
             },
             onSuccess: function(files) {
                 clearTimeout(torrentFilesFilterInputTimer);
+                torrentFilesFilterInputTimer = -1;
 
                 if (files.length === 0) {
                     torrentFilesTable.clear();
@@ -390,7 +386,7 @@ window.qBittorrent.PropFiles = (function() {
     const handleNewTorrentFiles = function(files) {
         is_seed = (files.length > 0) ? files[0].is_seed : true;
 
-        const rows = files.map(function(file, index) {
+        const rows = files.map((file, index) => {
             let progress = (file.progress * 100).round(1);
             if ((progress === 100) && (file.progress < 1))
                 progress = 99.9;
@@ -423,13 +419,13 @@ window.qBittorrent.PropFiles = (function() {
 
         const rootNode = new window.qBittorrent.FileTree.FolderNode();
 
-        rows.forEach(function(row) {
+        rows.forEach((row) => {
             const pathItems = row.fileName.split(window.qBittorrent.Filesystem.PathSeparator);
 
             pathItems.pop(); // remove last item (i.e. file name)
             let parent = rootNode;
-            pathItems.forEach(function(folderName) {
-                if (folderName === '.unwanted')
+            pathItems.forEach((folderName) => {
+                if (folderName === ".unwanted")
                     return;
 
                 let folderNode = null;
@@ -476,7 +472,7 @@ window.qBittorrent.PropFiles = (function() {
             parent.addChild(childNode);
 
             ++rowId;
-        }.bind(this));
+        });
 
         torrentFilesTable.populateTable(rootNode);
         torrentFilesTable.updateTable(false);
@@ -499,26 +495,25 @@ window.qBittorrent.PropFiles = (function() {
 
     const expandFolder = function(id) {
         const node = torrentFilesTable.getNode(id);
-        if (node.isFolder) {
+        if (node.isFolder)
             expandNode(node);
-        }
     };
 
     const collapseFolder = function(id) {
         const node = torrentFilesTable.getNode(id);
-        if (node.isFolder) {
+        if (node.isFolder)
             collapseNode(node);
-        }
     };
 
     const filesPriorityMenuClicked = function(priority) {
         const selectedRows = torrentFilesTable.selectedRowsIds();
-        if (selectedRows.length === 0) return;
+        if (selectedRows.length === 0)
+            return;
 
         const rowIds = [];
         const fileIds = [];
-        selectedRows.forEach(function(rowId) {
-            const elem = $('comboPrio' + rowId);
+        selectedRows.forEach((rowId) => {
+            const elem = $("comboPrio" + rowId);
             rowIds.push(rowId);
             fileIds.push(elem.get("data-file-id"));
         });
@@ -527,10 +522,10 @@ window.qBittorrent.PropFiles = (function() {
         const uniqueFileIds = {};
         for (let i = 0; i < rowIds.length; ++i) {
             const rows = getAllChildren(rowIds[i], fileIds[i]);
-            rows.rowIds.forEach(function(rowId) {
+            rows.rowIds.forEach((rowId) => {
                 uniqueRowIds[rowId] = true;
             });
-            rows.fileIds.forEach(function(fileId) {
+            rows.fileIds.forEach((fileId) => {
                 uniqueFileIds[fileId] = true;
             });
         }
@@ -538,35 +533,64 @@ window.qBittorrent.PropFiles = (function() {
         setFilePriority(Object.keys(uniqueRowIds), Object.keys(uniqueFileIds), priority);
     };
 
+    const singleFileRename = function(hash) {
+        const rowId = torrentFilesTable.selectedRowsIds()[0];
+        if (rowId === undefined)
+            return;
+        const row = torrentFilesTable.rows[rowId];
+        if (!row)
+            return;
+
+        const node = torrentFilesTable.getNode(rowId);
+        const path = node.path;
+
+        new MochaUI.Window({
+            id: "renamePage",
+            title: "QBT_TR(Renaming)QBT_TR[CONTEXT=TorrentContentTreeView]",
+            loadMethod: "iframe",
+            contentURL: "rename_file.html?hash=" + hash + "&isFolder=" + node.isFolder
+                + "&path=" + encodeURIComponent(path),
+            scrollbars: false,
+            resizable: true,
+            maximizable: false,
+            paddingVertical: 0,
+            paddingHorizontal: 0,
+            width: 400,
+            height: 100
+        });
+    };
+
+    const multiFileRename = function(hash) {
+        new MochaUI.Window({
+            id: "multiRenamePage",
+            title: "QBT_TR(Renaming)QBT_TR[CONTEXT=TorrentContentTreeView]",
+            data: { hash: hash, selectedRows: torrentFilesTable.selectedRows },
+            loadMethod: "xhr",
+            contentURL: "rename_files.html",
+            scrollbars: false,
+            resizable: true,
+            maximizable: false,
+            paddingVertical: 0,
+            paddingHorizontal: 0,
+            width: 800,
+            height: 420,
+            resizeLimit: { "x": [800], "y": [420] }
+        });
+    };
+
     const torrentFilesContextMenu = new window.qBittorrent.ContextMenu.ContextMenu({
-        targets: '#torrentFilesTableDiv tr',
-        menu: 'torrentFilesMenu',
+        targets: "#torrentFilesTableDiv tr",
+        menu: "torrentFilesMenu",
         actions: {
             Rename: function(element, ref) {
                 const hash = torrentsTable.getCurrentTorrentID();
-                if (!hash) return;
-                const rowId = torrentFilesTable.selectedRowsIds()[0];
-                if (rowId === undefined) return;
-                const row = torrentFilesTable.rows[rowId];
-                if (!row) return;
+                if (!hash)
+                    return;
 
-                const node = torrentFilesTable.getNode(rowId);
-                const path = node.path;
-
-                new MochaUI.Window({
-                    id: 'renamePage',
-                    title: "QBT_TR(Renaming)QBT_TR[CONTEXT=TorrentContentTreeView]",
-                    loadMethod: 'iframe',
-                    contentURL: 'rename_file.html?hash=' + hash + '&isFolder=' + node.isFolder
-                        + '&path=' + encodeURIComponent(path),
-                    scrollbars: false,
-                    resizable: false,
-                    maximizable: false,
-                    paddingVertical: 0,
-                    paddingHorizontal: 0,
-                    width: 250,
-                    height: 100
-                });
+                if (torrentFilesTable.selectedRowsIds().length > 1)
+                    multiFileRename(hash);
+                else
+                    singleFileRename(hash);
             },
 
             FilePrioIgnore: function(element, ref) {
@@ -588,20 +612,20 @@ window.qBittorrent.PropFiles = (function() {
         },
         onShow: function() {
             if (is_seed)
-                this.hideItem('FilePrio');
+                this.hideItem("FilePrio");
             else
-                this.showItem('FilePrio');
+                this.showItem("FilePrio");
         }
     });
 
-    torrentFilesTable.setup('torrentFilesTableDiv', 'torrentFilesTableFixedHeaderDiv', torrentFilesContextMenu);
+    torrentFilesTable.setup("torrentFilesTableDiv", "torrentFilesTableFixedHeaderDiv", torrentFilesContextMenu);
     // inject checkbox into table header
-    const tableHeaders = $$('#torrentFilesTableFixedHeaderDiv .dynamicTableHeader th');
+    const tableHeaders = $$("#torrentFilesTableFixedHeaderDiv .dynamicTableHeader th");
     if (tableHeaders.length > 0) {
-        const checkbox = new Element('input');
-        checkbox.set('type', 'checkbox');
-        checkbox.set('id', 'tristate_cb');
-        checkbox.addEvent('click', switchCheckboxState);
+        const checkbox = new Element("input");
+        checkbox.set("type", "checkbox");
+        checkbox.set("id", "tristate_cb");
+        checkbox.addEvent("click", switchCheckboxState);
 
         const checkboxTH = tableHeaders[0];
         checkbox.injectInside(checkboxTH);
@@ -609,34 +633,36 @@ window.qBittorrent.PropFiles = (function() {
 
     // default sort by name column
     if (torrentFilesTable.getSortedColumn() === null)
-        torrentFilesTable.setSortedColumn('name');
+        torrentFilesTable.setSortedColumn("name");
 
-    let prevTorrentFilesFilterValue;
-    let torrentFilesFilterInputTimer = null;
     // listen for changes to torrentFilesFilterInput
-    $('torrentFilesFilterInput').addEvent('input', function() {
-        const value = $('torrentFilesFilterInput').get("value");
-        if (value !== prevTorrentFilesFilterValue) {
-            prevTorrentFilesFilterValue = value;
-            torrentFilesTable.setFilter(value);
-            clearTimeout(torrentFilesFilterInputTimer);
-            torrentFilesFilterInputTimer = setTimeout(function() {
-                if (current_hash === "") return;
-                torrentFilesTable.updateTable(false);
+    let torrentFilesFilterInputTimer = -1;
+    $("torrentFilesFilterInput").addEvent("input", () => {
+        clearTimeout(torrentFilesFilterInputTimer);
 
-                if (value.trim() === "")
-                    collapseAllNodes();
-                else
-                    expandAllNodes();
-            }, 400);
-        }
+        const value = $("torrentFilesFilterInput").get("value");
+        torrentFilesTable.setFilter(value);
+
+        torrentFilesFilterInputTimer = setTimeout(() => {
+            torrentFilesFilterInputTimer = -1;
+
+            if (current_hash === "")
+                return;
+
+            torrentFilesTable.updateTable();
+
+            if (value.trim() === "")
+                collapseAllNodes();
+            else
+                expandAllNodes();
+        }, window.qBittorrent.Misc.FILTER_INPUT_DELAY);
     });
 
     /**
      * Show/hide a node's row
      */
     const _hideNode = function(node, shouldHide) {
-        const span = $('filesTablefileName' + node.rowId);
+        const span = $("filesTablefileName" + node.rowId);
         // span won't exist if row has been filtered out
         if (span === null)
             return;
@@ -651,7 +677,7 @@ window.qBittorrent.PropFiles = (function() {
      * Update a node's collapsed state and icon
      */
     const _updateNodeState = function(node, isCollapsed) {
-        const span = $('filesTablefileName' + node.rowId);
+        const span = $("filesTablefileName" + node.rowId);
         // span won't exist if row has been filtered out
         if (span === null)
             return;
@@ -669,7 +695,7 @@ window.qBittorrent.PropFiles = (function() {
     };
 
     const _isCollapsed = function(node) {
-        const span = $('filesTablefileName' + node.rowId);
+        const span = $("filesTablefileName" + node.rowId);
         if (span === null)
             return true;
 
@@ -689,8 +715,8 @@ window.qBittorrent.PropFiles = (function() {
 
     const expandAllNodes = function() {
         const root = torrentFilesTable.getRoot();
-        root.children.each(function(node) {
-            node.children.each(function(child) {
+        root.children.each((node) => {
+            node.children.each((child) => {
                 _collapseNode(child, false, true, false);
             });
         });
@@ -699,8 +725,8 @@ window.qBittorrent.PropFiles = (function() {
 
     const collapseAllNodes = function() {
         const root = torrentFilesTable.getRoot();
-        root.children.each(function(node) {
-            node.children.each(function(child) {
+        root.children.each((node) => {
+            node.children.each((child) => {
                 _collapseNode(child, true, true, false);
             });
         });
@@ -725,7 +751,7 @@ window.qBittorrent.PropFiles = (function() {
         if (!isChildNode || applyToChildren || !canSkipNode)
             _updateNodeState(node, shouldCollapse);
 
-        node.children.each(function(child) {
+        node.children.each((child) => {
             _hideNode(child, shouldCollapse);
 
             if (!child.isFolder)
