@@ -26,11 +26,10 @@
  * exception statement from your version.
  */
 
-'use strict';
+"use strict";
 
-if (window.qBittorrent === undefined) {
+if (window.qBittorrent === undefined)
     window.qBittorrent = {};
-}
 
 window.qBittorrent.PropTrackers = (function() {
     const exports = function() {
@@ -45,8 +44,8 @@ window.qBittorrent.PropTrackers = (function() {
     let loadTrackersDataTimer;
 
     const loadTrackersData = function() {
-        if ($('prop_trackers').hasClass('invisible')
-            || $('propertiesPanel_collapseToggle').hasClass('panel-expand')) {
+        if ($("prop_trackers").hasClass("invisible")
+            || $("propertiesPanel_collapseToggle").hasClass("panel-expand")) {
             // Tab changed, don't do anything
             return;
         }
@@ -57,15 +56,15 @@ window.qBittorrent.PropTrackers = (function() {
             loadTrackersDataTimer = loadTrackersData.delay(10000);
             return;
         }
-        if (new_hash != current_hash) {
+        if (new_hash !== current_hash) {
             torrentTrackersTable.clear();
             current_hash = new_hash;
         }
-        const url = new URI('api/v2/torrents/trackers?hash=' + current_hash);
+        const url = new URI("api/v2/torrents/trackers?hash=" + current_hash);
         new Request.JSON({
             url: url,
+            method: "get",
             noCache: true,
-            method: 'get',
             onComplete: function() {
                 clearTimeout(loadTrackersDataTimer);
                 loadTrackersDataTimer = loadTrackersData.delay(10000);
@@ -75,7 +74,7 @@ window.qBittorrent.PropTrackers = (function() {
                 torrentTrackersTable.clear();
 
                 if (trackers) {
-                    trackers.each(function(tracker) {
+                    trackers.each((tracker) => {
                         let status;
                         switch (tracker.status) {
                             case 0:
@@ -100,7 +99,7 @@ window.qBittorrent.PropTrackers = (function() {
                             tier: (tracker.tier >= 0) ? tracker.tier : "",
                             url: tracker.url,
                             status: status,
-                            peers: tracker.num_peers,
+                            peers: (tracker.num_peers >= 0) ? tracker.num_peers : "QBT_TR(N/A)QBT_TR[CONTEXT=TrackerListWidget]",
                             seeds: (tracker.num_seeds >= 0) ? tracker.num_seeds : "QBT_TR(N/A)QBT_TR[CONTEXT=TrackerListWidget]",
                             leeches: (tracker.num_leeches >= 0) ? tracker.num_leeches : "QBT_TR(N/A)QBT_TR[CONTEXT=TrackerListWidget]",
                             downloaded: (tracker.num_downloaded >= 0) ? tracker.num_downloaded : "QBT_TR(N/A)QBT_TR[CONTEXT=TrackerListWidget]",
@@ -126,8 +125,8 @@ window.qBittorrent.PropTrackers = (function() {
     };
 
     const torrentTrackersContextMenu = new window.qBittorrent.ContextMenu.ContextMenu({
-        targets: '#torrentTrackersTableDiv',
-        menu: 'torrentTrackersMenu',
+        targets: "#torrentTrackersTableDiv",
+        menu: "torrentTrackersMenu",
         actions: {
             AddTracker: function(element, ref) {
                 addTrackerFN();
@@ -147,30 +146,31 @@ window.qBittorrent.PropTrackers = (function() {
         },
         onShow: function() {
             const selectedTrackers = torrentTrackersTable.selectedRowsIds();
-            const containsStaticTracker = selectedTrackers.some(function(tracker) {
+            const containsStaticTracker = selectedTrackers.some((tracker) => {
                 return (tracker.indexOf("** [") === 0);
             });
 
             if (containsStaticTracker || (selectedTrackers.length === 0)) {
-                this.hideItem('EditTracker');
-                this.hideItem('RemoveTracker');
-                this.hideItem('CopyTrackerUrl');
+                this.hideItem("EditTracker");
+                this.hideItem("RemoveTracker");
+                this.hideItem("CopyTrackerUrl");
             }
             else {
-                this.showItem('EditTracker');
-                this.showItem('RemoveTracker');
-                this.showItem('CopyTrackerUrl');
+                this.showItem("EditTracker");
+                this.showItem("RemoveTracker");
+                this.showItem("CopyTrackerUrl");
             }
         }
     });
 
     const addTrackerFN = function() {
-        if (current_hash.length === 0) return;
+        if (current_hash.length === 0)
+            return;
         new MochaUI.Window({
-            id: 'trackersPage',
-            title: "QBT_TR(Trackers addition dialog)QBT_TR[CONTEXT=TrackersAdditionDialog]",
-            loadMethod: 'iframe',
-            contentURL: 'addtrackers.html?hash=' + current_hash,
+            id: "trackersPage",
+            title: "QBT_TR(Add trackers)QBT_TR[CONTEXT=TrackersAdditionDialog]",
+            loadMethod: "iframe",
+            contentURL: "addtrackers.html?hash=" + current_hash,
             scrollbars: true,
             resizable: false,
             maximizable: false,
@@ -186,14 +186,15 @@ window.qBittorrent.PropTrackers = (function() {
     };
 
     const editTrackerFN = function(element) {
-        if (current_hash.length === 0) return;
+        if (current_hash.length === 0)
+            return;
 
         const trackerUrl = encodeURIComponent(element.childNodes[1].innerText);
         new MochaUI.Window({
-            id: 'trackersPage',
+            id: "trackersPage",
             title: "QBT_TR(Tracker editing)QBT_TR[CONTEXT=TrackerListWidget]",
-            loadMethod: 'iframe',
-            contentURL: 'edittracker.html?hash=' + current_hash + '&url=' + trackerUrl,
+            loadMethod: "iframe",
+            contentURL: "edittracker.html?hash=" + current_hash + "&url=" + trackerUrl,
             scrollbars: true,
             resizable: false,
             maximizable: false,
@@ -209,15 +210,16 @@ window.qBittorrent.PropTrackers = (function() {
     };
 
     const removeTrackerFN = function(element) {
-        if (current_hash.length === 0) return;
+        if (current_hash.length === 0)
+            return;
 
         const selectedTrackers = torrentTrackersTable.selectedRowsIds();
         new Request({
-            url: 'api/v2/torrents/removeTrackers',
-            method: 'post',
+            url: "api/v2/torrents/removeTrackers",
+            method: "post",
             data: {
                 hash: current_hash,
-                urls: selectedTrackers.join("|")
+                urls: selectedTrackers.map(encodeURIComponent).join("|")
             },
             onSuccess: function() {
                 updateData();
@@ -225,13 +227,13 @@ window.qBittorrent.PropTrackers = (function() {
         }).send();
     };
 
-    new ClipboardJS('#CopyTrackerUrl', {
+    new ClipboardJS("#CopyTrackerUrl", {
         text: function(trigger) {
             return torrentTrackersTable.selectedRowsIds().join("\n");
         }
     });
 
-    torrentTrackersTable.setup('torrentTrackersTableDiv', 'torrentTrackersTableFixedHeaderDiv', torrentTrackersContextMenu);
+    torrentTrackersTable.setup("torrentTrackersTableDiv", "torrentTrackersTableFixedHeaderDiv", torrentTrackersContextMenu);
 
     return exports();
 })();
