@@ -26,16 +26,16 @@
  * exception statement from your version.
  */
 
-'use strict';
+"use strict";
 
 MochaUI.extend({
     addUpLimitSlider: function(hashes) {
-        if ($('uplimitSliderarea')) {
+        if ($("uplimitSliderarea")) {
             // Get global upload limit
             let maximum = 500;
             new Request({
-                url: 'api/v2/transfer/uploadLimit',
-                method: 'post',
+                url: "api/v2/transfer/uploadLimit",
+                method: "get",
                 data: {},
                 onSuccess: function(data) {
                     if (data) {
@@ -44,7 +44,7 @@ MochaUI.extend({
                             maximum = tmp / 1024.0;
                         }
                         else {
-                            if (hashes[0] == "global")
+                            if (hashes[0] === "global")
                                 maximum = 10000;
                             else
                                 maximum = 1000;
@@ -52,75 +52,77 @@ MochaUI.extend({
                     }
                     // Get torrents upload limit
                     // And create slider
-                    if (hashes[0] == 'global') {
+                    if (hashes[0] === "global") {
                         let up_limit = maximum;
-                        if (up_limit < 0) up_limit = 0;
+                        if (up_limit < 0)
+                            up_limit = 0;
                         maximum = 10000;
-                        new Slider($('uplimitSliderarea'), $('uplimitSliderknob'), {
+                        new Slider($("uplimitSliderarea"), $("uplimitSliderknob"), {
                             steps: maximum,
                             offset: 0,
                             initialStep: up_limit.round(),
                             onChange: function(pos) {
                                 if (pos > 0) {
-                                    $('uplimitUpdatevalue').value = pos;
-                                    $('upLimitUnit').style.visibility = "visible";
+                                    $("uplimitUpdatevalue").value = pos;
+                                    $("upLimitUnit").style.visibility = "visible";
                                 }
                                 else {
-                                    $('uplimitUpdatevalue').value = '∞';
-                                    $('upLimitUnit').style.visibility = "hidden";
+                                    $("uplimitUpdatevalue").value = "∞";
+                                    $("upLimitUnit").style.visibility = "hidden";
                                 }
                             }.bind(this)
                         });
                         // Set default value
                         if (up_limit === 0) {
-                            $('uplimitUpdatevalue').value = '∞';
-                            $('upLimitUnit').style.visibility = "hidden";
+                            $("uplimitUpdatevalue").value = "∞";
+                            $("upLimitUnit").style.visibility = "hidden";
                         }
                         else {
-                            $('uplimitUpdatevalue').value = up_limit.round();
-                            $('upLimitUnit').style.visibility = "visible";
+                            $("uplimitUpdatevalue").value = up_limit.round();
+                            $("upLimitUnit").style.visibility = "visible";
                         }
                     }
                     else {
                         new Request.JSON({
-                            url: 'api/v2/torrents/uploadLimit',
-                            noCache: true,
-                            method: 'post',
+                            url: "api/v2/torrents/uploadLimit",
+                            method: "post",
                             data: {
-                                hashes: hashes.join('|')
+                                hashes: hashes.join("|")
                             },
                             onSuccess: function(data) {
                                 if (data) {
                                     let up_limit = data[hashes[0]];
-                                    for (const key in data)
-                                        if (up_limit != data[key]) {
+                                    for (const key in data) {
+                                        if (up_limit !== data[key]) {
                                             up_limit = 0;
                                             break;
                                         }
-                                    if (up_limit < 0) up_limit = 0;
-                                    new Slider($('uplimitSliderarea'), $('uplimitSliderknob'), {
+                                    }
+                                    if (up_limit < 0)
+                                        up_limit = 0;
+                                    new Slider($("uplimitSliderarea"), $("uplimitSliderknob"), {
                                         steps: maximum,
                                         offset: 0,
                                         initialStep: (up_limit / 1024.0).round(),
                                         onChange: function(pos) {
                                             if (pos > 0) {
-                                                $('uplimitUpdatevalue').value = pos;
-                                                $('upLimitUnit').style.visibility = "visible";
+                                                $("uplimitUpdatevalue").value = pos;
+                                                $("upLimitUnit").style.visibility = "visible";
                                             }
                                             else {
-                                                $('uplimitUpdatevalue').value = '∞';
-                                                $('upLimitUnit').style.visibility = "hidden";
+                                                $("uplimitUpdatevalue").value = "∞";
+                                                $("upLimitUnit").style.visibility = "hidden";
                                             }
                                         }.bind(this)
                                     });
                                     // Set default value
                                     if (up_limit === 0) {
-                                        $('uplimitUpdatevalue').value = '∞';
-                                        $('upLimitUnit').style.visibility = "hidden";
+                                        $("uplimitUpdatevalue").value = "∞";
+                                        $("upLimitUnit").style.visibility = "hidden";
                                     }
                                     else {
-                                        $('uplimitUpdatevalue').value = (up_limit / 1024.0).round();
-                                        $('upLimitUnit').style.visibility = "visible";
+                                        $("uplimitUpdatevalue").value = (up_limit / 1024.0).round();
+                                        $("upLimitUnit").style.visibility = "visible";
                                     }
                                 }
                             }
@@ -132,12 +134,12 @@ MochaUI.extend({
     },
 
     addDlLimitSlider: function(hashes) {
-        if ($('dllimitSliderarea')) {
+        if ($("dllimitSliderarea")) {
             // Get global upload limit
             let maximum = 500;
             new Request({
-                url: 'api/v2/transfer/downloadLimit',
-                method: 'post',
+                url: "api/v2/transfer/downloadLimit",
+                method: "get",
                 data: {},
                 onSuccess: function(data) {
                     if (data) {
@@ -146,7 +148,7 @@ MochaUI.extend({
                             maximum = tmp / 1024.0;
                         }
                         else {
-                            if (hashes[0] == "global")
+                            if (hashes[0] === "global")
                                 maximum = 10000;
                             else
                                 maximum = 1000;
@@ -154,75 +156,77 @@ MochaUI.extend({
                     }
                     // Get torrents download limit
                     // And create slider
-                    if (hashes[0] == 'global') {
+                    if (hashes[0] === "global") {
                         let dl_limit = maximum;
-                        if (dl_limit < 0) dl_limit = 0;
+                        if (dl_limit < 0)
+                            dl_limit = 0;
                         maximum = 10000;
-                        new Slider($('dllimitSliderarea'), $('dllimitSliderknob'), {
+                        new Slider($("dllimitSliderarea"), $("dllimitSliderknob"), {
                             steps: maximum,
                             offset: 0,
                             initialStep: dl_limit.round(),
                             onChange: function(pos) {
                                 if (pos > 0) {
-                                    $('dllimitUpdatevalue').value = pos;
-                                    $('dlLimitUnit').style.visibility = "visible";
+                                    $("dllimitUpdatevalue").value = pos;
+                                    $("dlLimitUnit").style.visibility = "visible";
                                 }
                                 else {
-                                    $('dllimitUpdatevalue').value = '∞';
-                                    $('dlLimitUnit').style.visibility = "hidden";
+                                    $("dllimitUpdatevalue").value = "∞";
+                                    $("dlLimitUnit").style.visibility = "hidden";
                                 }
                             }.bind(this)
                         });
                         // Set default value
                         if (dl_limit === 0) {
-                            $('dllimitUpdatevalue').value = '∞';
-                            $('dlLimitUnit').style.visibility = "hidden";
+                            $("dllimitUpdatevalue").value = "∞";
+                            $("dlLimitUnit").style.visibility = "hidden";
                         }
                         else {
-                            $('dllimitUpdatevalue').value = dl_limit.round();
-                            $('dlLimitUnit').style.visibility = "visible";
+                            $("dllimitUpdatevalue").value = dl_limit.round();
+                            $("dlLimitUnit").style.visibility = "visible";
                         }
                     }
                     else {
                         new Request.JSON({
-                            url: 'api/v2/torrents/downloadLimit',
-                            noCache: true,
-                            method: 'post',
+                            url: "api/v2/torrents/downloadLimit",
+                            method: "post",
                             data: {
-                                hashes: hashes.join('|')
+                                hashes: hashes.join("|")
                             },
                             onSuccess: function(data) {
                                 if (data) {
                                     let dl_limit = data[hashes[0]];
-                                    for (const key in data)
-                                        if (dl_limit != data[key]) {
+                                    for (const key in data) {
+                                        if (dl_limit !== data[key]) {
                                             dl_limit = 0;
                                             break;
                                         }
-                                    if (dl_limit < 0) dl_limit = 0;
-                                    new Slider($('dllimitSliderarea'), $('dllimitSliderknob'), {
+                                    }
+                                    if (dl_limit < 0)
+                                        dl_limit = 0;
+                                    new Slider($("dllimitSliderarea"), $("dllimitSliderknob"), {
                                         steps: maximum,
                                         offset: 0,
                                         initialStep: (dl_limit / 1024.0).round(),
                                         onChange: function(pos) {
                                             if (pos > 0) {
-                                                $('dllimitUpdatevalue').value = pos;
-                                                $('dlLimitUnit').style.visibility = "visible";
+                                                $("dllimitUpdatevalue").value = pos;
+                                                $("dlLimitUnit").style.visibility = "visible";
                                             }
                                             else {
-                                                $('dllimitUpdatevalue').value = '∞';
-                                                $('dlLimitUnit').style.visibility = "hidden";
+                                                $("dllimitUpdatevalue").value = "∞";
+                                                $("dlLimitUnit").style.visibility = "hidden";
                                             }
                                         }.bind(this)
                                     });
                                     // Set default value
                                     if (dl_limit === 0) {
-                                        $('dllimitUpdatevalue').value = '∞';
-                                        $('dlLimitUnit').style.visibility = "hidden";
+                                        $("dllimitUpdatevalue").value = "∞";
+                                        $("dlLimitUnit").style.visibility = "hidden";
                                     }
                                     else {
-                                        $('dllimitUpdatevalue').value = (dl_limit / 1024.0).round();
-                                        $('dlLimitUnit').style.visibility = "visible";
+                                        $("dllimitUpdatevalue").value = (dl_limit / 1024.0).round();
+                                        $("dlLimitUnit").style.visibility = "visible";
                                     }
                                 }
                             }
